@@ -20,8 +20,7 @@
 #include "internals.h"
 #include "irc.h"
 
-static void free_network_nick(struct network_state *, struct network_nick *);
-static void free_channel(struct channel_state *c);
+static void free_network_nick(struct irc_network_state *, struct network_nick *);
 
 enum mode_type { REMOVE = 0, ADD = 1 };
 
@@ -131,7 +130,7 @@ static void free_channel_nick(struct channel_nick *n)
 	g_free(n);
 }
 
-static void free_invitelist(struct channel_state *c)
+static void free_invitelist(struct irc_channel_state *c)
 {
 	GList *g;
 	g_assert(c);
@@ -144,7 +143,8 @@ static void free_invitelist(struct channel_state *c)
 	c->invitelist = NULL;
 }
 
-static void free_exceptlist(struct channel_state *c)
+
+static void free_exceptlist(struct irc_channel_state *c)
 {
 	GList *g;
 	g_assert(c);
@@ -157,7 +157,45 @@ static void free_exceptlist(struct channel_state *c)
 	c->exceptlist = NULL;
 }
 
-static void free_banlist(struct channel_state *c)
+static void free_banlist_entry(struct banlist_entry *be)
+{
+	g_free(be->hostmask);
+	g_free(be->by);
+	g_free(be);
+}
+
+static char *find_exceptlist_entry(GList *entries, const char *entry)
+{
+	GList *gl;
+	for (gl = entries; gl; gl = gl->next) {
+		if (!strcmp(gl->data, entry))
+			return gl->data;
+	}
+	return NULL;
+}
+
+static char *find_realnamebanlist_entry(GList *entries, const char *entry)
+{
+	GList *gl;
+	for (gl = entries; gl; gl = gl->next) {
+		if (!strcmp(gl->data, entry))
+			return gl->data;
+	}
+	return NULL;
+}
+
+static struct banlist_entry *find_banlist_entry(GList *entries, const char *hostmask)
+{
+	GList *gl;
+	for (gl = entries; gl; gl = gl->next) {
+		struct banlist_entry *be = gl->data;
+		if (!strcmp(be->hostmask, hostmask))
+			return be;
+	}
+	return NULL;
+}
+
+static void free_banlist(struct irc_channel_state *c)
 {
 	GList *g;
 	
@@ -165,15 +203,13 @@ static void free_banlist(struct channel_state *c)
 	g = c->banlist;
 	while(g) {
 		struct banlist_entry *be = g->data;
-		g_free(be->hostmask);
-		g_free(be->by);
 		g = g_list_remove(g, be);
-		g_free(be);
+		free_banlist_entry(be);
 	}
 	c->banlist = NULL;
 }
 
-static void free_names(struct channel_state *c)
+static void free_names(struct irc_channel_state *c)
 {
 	while(c->nicks) {
 		free_channel_nick((struct channel_nick *)c->nicks->data);
@@ -181,7 +217,7 @@ static void free_names(struct channel_state *c)
 	c->nicks = NULL;
 }
 
-static void free_channel(struct channel_state *c)
+void free_channel_state(struct irc_channel_state *c)
 {
 	if (c == NULL)
 		return;
@@ -195,33 +231,43 @@ static void free_channel(struct channel_state *c)
 	g_free(c);
 }
 
-struct channel_state *find_channel(struct network_state *st, const char *name)
+struct irc_channel_state *find_channel(struct irc_network_state *st, const char *name)
 {
 	GList *cl;
 	g_assert(st);
 	g_assert(name);
 	for (cl = st->channels; cl; cl = cl->next) {
-		struct channel_state *c = (struct channel_state *)cl->data;
+		struct irc_channel_state *c = (struct irc_channel_state *)cl->data;
 
-		if (!irccmp(&st->info, c->name, name)) 
+		if (!irccmp(st->info, c->name, name)) 
 			return c;
 	}
 	return NULL;
 }
 
-struct channel_state *find_add_channel(struct network_state *st, char *name) 
+struct irc_channel_state *irc_channel_state_new(const char *name)
 {
-	struct channel_state *c;
+	struct irc_channel_state *c;
+	g_assert(name != NULL);
+
+	c = g_new0(struct irc_channel_state ,1);
+	c->name = g_strdup(name);
+
+	return c;
+}
+
+struct irc_channel_state *find_add_channel(struct irc_network_state *st, char *name) 
+{
+	struct irc_channel_state *c;
 	g_assert(st);
 	g_assert(name);
 	
 	c = find_channel(st, name);
 	if (c)
 		return c;
-	c = g_new0(struct channel_state ,1);
+	c = irc_channel_state_new(name);
 	c->network = st;
 	st->channels = g_list_append(st->channels, c);
-	c->name = g_strdup(name);
 
 	return c;
 }
@@ -233,7 +279,7 @@ struct channel_state *find_add_channel(struct network_state *st, char *name)
  * @param name Name of the nick to search for
  * @return NULL if not found, channel_nick if found
  */
-struct channel_nick *find_channel_nick(struct channel_state *c, 
+struct channel_nick *find_channel_nick(struct irc_channel_state *c, 
 									   const char *name) 
 {
 	GList *l;
@@ -242,12 +288,12 @@ struct channel_nick *find_channel_nick(struct channel_state *c,
 	g_assert(c);
 
 	g_assert(c->network);
-	if (is_prefix(realname[0], &c->network->info))
+	if (is_prefix(realname[0], c->network->info))
 		realname++;
 
 	for (l = c->nicks; l; l = l->next) {
 		struct channel_nick *n = (struct channel_nick *)l->data;
-		if (!irccmp(&c->network->info, n->global_nick->nick, realname))
+		if (!irccmp(c->network->info, n->global_nick->nick, realname))
 			return n;
 	}
 
@@ -261,7 +307,7 @@ struct channel_nick *find_channel_nick(struct channel_state *c,
  * @param hm Hostmask of the nick to search for
  * @return NULL if not found, channel_nick if found
  */
-struct channel_nick *find_channel_nick_hostmask(struct channel_state *c, 
+struct channel_nick *find_channel_nick_hostmask(struct irc_channel_state *c, 
 									   		    const char *hm) 
 {
 	GList *l;
@@ -271,7 +317,7 @@ struct channel_nick *find_channel_nick_hostmask(struct channel_state *c,
 	g_assert(c->network);
 	for (l = c->nicks; l; l = l->next) {
 		struct channel_nick *n = (struct channel_nick *)l->data;
-		if (!irccmp(&c->network->info, n->global_nick->hostmask, hm))
+		if (!irccmp(c->network->info, n->global_nick->hostmask, hm))
 			return n;
 	}
 
@@ -285,7 +331,7 @@ struct channel_nick *find_channel_nick_hostmask(struct channel_state *c,
  * @param name Name of the nick to search for
  * @return NULL if not found, network_nick otherwise
  */
-struct network_nick *find_network_nick(struct network_state *n, 
+struct network_nick *find_network_nick(struct irc_network_state *n, 
 									   const char *name)
 {
 	GList *gl;
@@ -293,12 +339,12 @@ struct network_nick *find_network_nick(struct network_state *n,
 	g_assert(name);
 	g_assert(n);
 
-	if (!irccmp(&n->info, n->me.nick, name))
+	if (!irccmp(n->info, n->me.nick, name))
 		return &n->me;
 
 	for (gl = n->nicks; gl; gl = gl->next) {
 		struct network_nick *ndd = (struct network_nick*)gl->data;
-		if (!irccmp(&n->info, ndd->nick, name)) {
+		if (!irccmp(n->info, ndd->nick, name)) {
 			return ndd;
 		}
 	}
@@ -313,7 +359,7 @@ struct network_nick *find_network_nick(struct network_state *n,
  * @param name Name of the nick to search for
  * @return network_nick structure, or NULL if out of memory.
  */
-struct network_nick *find_add_network_nick(struct network_state *n, 
+struct network_nick *find_add_network_nick(struct irc_network_state *n, 
 										   const char *name)
 {
 	struct network_nick *nd;
@@ -327,7 +373,7 @@ struct network_nick *find_add_network_nick(struct network_state *n,
 
 	/* create one, if it doesn't exist */
 	nd = g_new0(struct network_nick,1);
-	g_assert(!is_prefix(name[0], &n->info));
+	g_assert(!is_prefix(name[0], n->info));
 	nd->nick = g_strdup(name);
 	nd->hops = -1;
 	
@@ -342,20 +388,20 @@ struct network_nick *find_add_network_nick(struct network_state *n,
  * @param name Name of the nick to search for
  * @return channel_nick structure, or NULL if out of memory.
  */
-struct channel_nick *find_add_channel_nick(struct channel_state *c, 
+struct channel_nick *find_add_channel_nick(struct irc_channel_state *c, 
 										   const char *name) 
 {
 	struct channel_nick *n;
 	const char *realname = name;
-	char mymode = 0;
+	char prefix = 0;
 
 	g_assert(c);
 	g_assert(name);
 	g_assert(strlen(name) > 0);
 	g_assert(c->network);
 
-	if (is_prefix(realname[0], &c->network->info)) {
-		mymode = realname[0];
+	if (is_prefix(realname[0], c->network->info)) {
+		prefix = realname[0];
 		realname++;
 	}
 
@@ -366,16 +412,20 @@ struct channel_nick *find_add_channel_nick(struct channel_state *c,
 	n = g_new0(struct channel_nick,1);
 	
 	n->channel = c;
-	n->mode = mymode;
 	n->global_nick = find_add_network_nick(c->network, realname);
+	if (prefix != 0) {
+		char mode = get_mode_by_prefix(prefix, c->network->info);
+		if (mode) 
+			modes_set_mode(n->modes, mode);
+    }
 	c->nicks = g_list_append(c->nicks, n);
 	n->global_nick->channel_nicks = g_list_append(n->global_nick->channel_nicks, n);
 	return n;
 }
 
-static void handle_join(struct network_state *s, const struct line *l)
+static void handle_join(struct irc_network_state *s, const struct irc_line *l)
 {
-	struct channel_state *c;
+	struct irc_channel_state *c;
 	struct channel_nick *ni;
 	int i;
 	char **channels;
@@ -395,8 +445,8 @@ static void handle_join(struct network_state *s, const struct line *l)
 		network_nick_set_hostmask(ni->global_nick, l->origin);
 
 		/* The user is joining a channel */
-		if (!irccmp(&s->info, nick, s->me.nick)) {
-			network_state_log(LOG_INFO, s, "Joining channel %s", c->name);
+		if (!irccmp(s->info, nick, s->me.nick)) {
+			network_state_log(LOG_TRACE, s, "Joining channel %s", c->name);
 		} else {
 			network_state_log(LOG_TRACE, s, "%s joins channel %s", nick, 
 							  c->name);
@@ -407,9 +457,9 @@ static void handle_join(struct network_state *s, const struct line *l)
 }
 
 
-static void handle_part(struct network_state *s, const struct line *l)
+static void handle_part(struct irc_network_state *s, const struct irc_line *l)
 {
-	struct channel_state *c;
+	struct irc_channel_state *c;
 	struct channel_nick *n;
 	char **channels;
 	int i;
@@ -443,9 +493,9 @@ static void handle_part(struct network_state *s, const struct line *l)
 				nick, channels[i]);
 		}
 
-		if (!irccmp(&s->info, nick, s->me.nick) && c) {
-			network_state_log(LOG_INFO, s, "Leaving %s", channels[i]);
-			free_channel(c);
+		if (!irccmp(s->info, nick, s->me.nick) && c) {
+			network_state_log(LOG_TRACE, s, "Leaving %s", channels[i]);
+			free_channel_state(c);
 		} else {
 			network_state_log(LOG_TRACE, s, "%s leaves %s", nick, channels[i]);
 		}
@@ -454,9 +504,9 @@ static void handle_part(struct network_state *s, const struct line *l)
 	g_strfreev(channels);
 }
 
-static void handle_kick(struct network_state *s, const struct line *l) 
+static void handle_kick(struct irc_network_state *s, const struct irc_line *l) 
 {
-	struct channel_state *c;
+	struct irc_channel_state *c;
 	struct channel_nick *n;
 	char **channels, **nicks;
 	int i;
@@ -485,9 +535,9 @@ static void handle_kick(struct network_state *s, const struct line *l)
 
 		free_channel_nick(n);
 
-		if (!irccmp(&s->info, nicks[i], s->me.nick)) {
+		if (!irccmp(s->info, nicks[i], s->me.nick)) {
 			network_state_log(LOG_INFO, s, "Kicked off %s by %s", c->name, nick);
-			free_channel(c);
+			free_channel_state(c);
 		} else {
 			network_state_log(LOG_TRACE, s, "%s kicked off %s by %s", nicks[i], channels[i], nick);
 		}
@@ -503,9 +553,9 @@ static void handle_kick(struct network_state *s, const struct line *l)
 	g_strfreev(channels);
 }
 
-static void handle_topic(struct network_state *s, const struct line *l) 
+static void handle_topic(struct irc_network_state *s, const struct irc_line *l) 
 {
-	struct channel_state *c = find_channel(s, l->args[1]);
+	struct irc_channel_state *c = find_channel(s, l->args[1]);
 	
 	CHECK_ORIGIN(s, l, "TOPIC");
 
@@ -518,9 +568,9 @@ static void handle_topic(struct network_state *s, const struct line *l)
 	c->topic_set_by = line_get_nick(l);
 }
 
-static void handle_332(struct network_state *s, const struct line *l) 
+static void handle_332(struct irc_network_state *s, const struct irc_line *l) 
 {
-	struct channel_state *c = find_channel(s, l->args[2]);
+	struct irc_channel_state *c = find_channel(s, l->args[2]);
 
 	if (c == NULL) {
 		network_state_log(LOG_WARNING, s, 
@@ -532,9 +582,9 @@ static void handle_332(struct network_state *s, const struct line *l)
 	c->topic = g_strdup(l->args[3]);
 }
 
-static void handle_333(struct network_state *s, const struct line *l) 
+static void handle_333(struct irc_network_state *s, const struct irc_line *l) 
 {
-	struct channel_state *c = find_channel(s, l->args[2]);
+	struct irc_channel_state *c = find_channel(s, l->args[2]);
 
 	if (!c) {
 		network_state_log(LOG_WARNING, s, 
@@ -543,13 +593,13 @@ static void handle_333(struct network_state *s, const struct line *l)
 		return;
 	}
 
-	c->topic_set_time = atol(l->args[4]);
+	c->topic_set_time = strtoul(l->args[4], NULL, 0);
 	c->topic_set_by = g_strdup(l->args[3]);
 }
 
-static void handle_no_topic(struct network_state *s, const struct line *l) 
+static void handle_no_topic(struct irc_network_state *s, const struct irc_line *l) 
 {
-	struct channel_state *c = find_channel(s, l->args[1]);
+	struct irc_channel_state *c = find_channel(s, l->args[1]);
 
 	if (c == NULL) {
 		network_state_log(LOG_WARNING, s, 
@@ -561,11 +611,11 @@ static void handle_no_topic(struct network_state *s, const struct line *l)
 	c->topic = NULL;
 }
 
-static void handle_namreply(struct network_state *s, const struct line *l) 
+static void handle_namreply(struct irc_network_state *s, const struct irc_line *l) 
 {
 	gchar **names;
 	int i;
-	struct channel_state *c = find_channel(s, l->args[3]);
+	struct irc_channel_state *c = find_channel(s, l->args[3]);
 
 	if (c == NULL) {
 		network_state_log(LOG_WARNING, s, 
@@ -587,9 +637,9 @@ static void handle_namreply(struct network_state *s, const struct line *l)
 	g_strfreev(names);
 }
 
-static void handle_end_names(struct network_state *s, const struct line *l) 
+static void handle_end_names(struct irc_network_state *s, const struct irc_line *l) 
 {
-	struct channel_state *c = find_channel(s, l->args[2]);
+	struct irc_channel_state *c = find_channel(s, l->args[2]);
 	if (c != NULL)
 		c->namreply_started = FALSE;
 	else 
@@ -598,9 +648,9 @@ static void handle_end_names(struct network_state *s, const struct line *l)
 				  l->args[2]);
 }
 
-static void handle_invitelist_entry(struct network_state *s, const struct line *l) 
+static void handle_invitelist_entry(struct irc_network_state *s, const struct irc_line *l) 
 {
-	struct channel_state *c = find_channel(s, l->args[2]);
+	struct irc_channel_state *c = find_channel(s, l->args[2]);
 	
 	if (c == NULL) {
 		network_state_log(LOG_WARNING, s, 
@@ -617,9 +667,9 @@ static void handle_invitelist_entry(struct network_state *s, const struct line *
 	c->invitelist = g_list_append(c->invitelist, g_strdup(l->args[3]));
 }
 
-static void handle_end_invitelist(struct network_state *s, const struct line *l) 
+static void handle_end_invitelist(struct irc_network_state *s, const struct irc_line *l) 
 {
-	struct channel_state *c = find_channel(s, l->args[2]);
+	struct irc_channel_state *c = find_channel(s, l->args[2]);
 	if (c != NULL)
 		c->invitelist_started = FALSE;
 	else 
@@ -627,9 +677,9 @@ static void handle_end_invitelist(struct network_state *s, const struct line *l)
 			  "Can't end invitelist for %s: channel not found", l->args[2]);
 }
 
-static void handle_exceptlist_entry(struct network_state *s, const struct line *l) 
+static void handle_exceptlist_entry(struct irc_network_state *s, const struct irc_line *l) 
 {
-	struct channel_state *c = find_channel(s, l->args[2]);
+	struct irc_channel_state *c = find_channel(s, l->args[2]);
 	
 	if (c == NULL) {
 		network_state_log(LOG_WARNING, s, 
@@ -646,9 +696,9 @@ static void handle_exceptlist_entry(struct network_state *s, const struct line *
 	c->exceptlist = g_list_append(c->exceptlist, g_strdup(l->args[3]));
 }
 
-static void handle_end_exceptlist(struct network_state *s, const struct line *l) 
+static void handle_end_exceptlist(struct irc_network_state *s, const struct irc_line *l) 
 {
-	struct channel_state *c = find_channel(s, l->args[2]);
+	struct irc_channel_state *c = find_channel(s, l->args[2]);
 	if (c != NULL)
 		c->exceptlist_started = FALSE;
 	else 
@@ -658,9 +708,9 @@ static void handle_end_exceptlist(struct network_state *s, const struct line *l)
 
 
 
-static void handle_banlist_entry(struct network_state *s, const struct line *l) 
+static void handle_banlist_entry(struct irc_network_state *s, const struct irc_line *l) 
 {
-	struct channel_state *c = find_channel(s, l->args[2]);
+	struct irc_channel_state *c = find_channel(s, l->args[2]);
 	struct banlist_entry *be;
 	
 	if (c == NULL) {
@@ -686,9 +736,9 @@ static void handle_banlist_entry(struct network_state *s, const struct line *l)
 	c->banlist = g_list_append(c->banlist, be);
 }
 
-static void handle_end_banlist(struct network_state *s, const struct line *l) 
+static void handle_end_banlist(struct irc_network_state *s, const struct irc_line *l) 
 {
-	struct channel_state *c = find_channel(s, l->args[2]);
+	struct irc_channel_state *c = find_channel(s, l->args[2]);
 
 	if (c != NULL)
 		c->banlist_started = FALSE;
@@ -697,9 +747,9 @@ static void handle_end_banlist(struct network_state *s, const struct line *l)
 				"Can't end banlist for %s: channel not found", l->args[2]);
 }
 
-static void handle_whoreply(struct network_state *s, const struct line *l) 
+static void handle_whoreply(struct irc_network_state *s, const struct irc_line *l) 
 {
-	struct channel_state *cs;
+	struct irc_channel_state *cs;
 	struct network_nick *nn;
 	struct channel_nick *cn;
 	char *fullname;
@@ -743,21 +793,21 @@ static void handle_whoreply(struct network_state *s, const struct line *l)
 	cn->last_update = time(NULL);
 }
 
-static void handle_end_who(struct network_state *s, const struct line *l) 
+static void handle_end_who(struct irc_network_state *s, const struct irc_line *l) 
 {
 }
 
-static void handle_nowaway(struct network_state *s, const struct line *l)
+static void handle_nowaway(struct irc_network_state *s, const struct irc_line *l)
 {
 	s->is_away = TRUE;
 }
 
-static void handle_unaway(struct network_state *s, const struct line *l)
+static void handle_unaway(struct irc_network_state *s, const struct irc_line *l)
 {
 	s->is_away = FALSE;
 }
 
-static void handle_quit(struct network_state *s, const struct line *l) 
+static void handle_quit(struct irc_network_state *s, const struct irc_line *l) 
 {
 	char *nick;
 	struct network_nick *nn;
@@ -768,127 +818,260 @@ static void handle_quit(struct network_state *s, const struct line *l)
 	nn = find_network_nick(s, nick);
 	g_free(nick);
 
-	g_assert(nn != &s->me);
-
-	if (nn != NULL) 
+	if (nn == &s->me) {
+		while (nn->channel_nicks) {
+			struct channel_nick *n = nn->channel_nicks->data;
+			free_channel_nick(n);
+		}
+	} else if (nn != NULL) 
 		free_network_nick(s, nn);
 }
 
-static void handle_mode(struct network_state *s, const struct line *l)
+gboolean modes_change_mode(irc_modes_t modes, gboolean set, char newmode)
+{
+	if (modes[(unsigned char)newmode] == set)
+		return FALSE;
+
+	modes[(unsigned char)newmode] = set;
+
+	return TRUE;
+}
+
+static int channel_state_change_mode(struct irc_network_state *s, struct network_nick *by, struct irc_channel_state *c, gboolean set, char mode, const char *opt_arg)
+{
+	struct irc_network_info *info = s->info;
+
+	if (!is_channel_mode(info, mode)) {
+		network_state_log(LOG_WARNING, s, "Mode '%c' set on channel %s is not in the supported list of channel modes from the server", mode, c->name);
+	}
+
+	if (mode == 'b') { /* Ban */
+		struct banlist_entry *be;
+
+		if (opt_arg == NULL) {
+			network_state_log(LOG_WARNING, s, "Missing argument for ban MODE set/unset");
+			return -1;
+		}
+
+		if (set) {
+			be = g_new0(struct banlist_entry, 1);
+			be->time_set = time(NULL);
+			be->hostmask = g_strdup(opt_arg);
+			be->by = (by?g_strdup(by->nick):NULL);
+			c->banlist = g_list_append(c->banlist, be);
+		} else {
+			be = find_banlist_entry(c->banlist, opt_arg);
+			if (be == NULL) {
+				network_state_log(LOG_WARNING, s, "Unable to remove nonpresent banlist entry '%s'", opt_arg);
+				return 1;
+			}
+			c->banlist = g_list_remove(c->banlist, be);
+			free_banlist_entry(be);
+		}
+		return 1;
+	} else if (mode == 'e') { /* Ban exemption */ 
+		if (opt_arg == NULL) {
+			network_state_log(LOG_WARNING, s, "Missing argument for ban exception MODE set/unset");
+			return -1;
+		}
+
+		if (set) {
+			c->exceptlist = g_list_append(c->exceptlist, g_strdup(opt_arg));
+		} else {
+			char *be = find_exceptlist_entry(c->exceptlist, opt_arg);
+			if (be == NULL) {
+				network_state_log(LOG_WARNING, s, "Unable to remove nonpresent ban except list entry '%s'", opt_arg);
+				return 1;
+			}
+			c->exceptlist = g_list_remove(c->exceptlist, be);
+			g_free(be);
+		}
+		return 1;
+	} else if (mode == 'J') { /* join throttling */
+		if (set) {
+			char **join_args;
+			if (opt_arg == NULL) {
+				network_state_log(LOG_WARNING, s, "Missing argument for realname ban MODE set/unset");
+				return -1;
+			}
+
+			join_args = g_strsplit(opt_arg, ",", 2);
+
+			if (join_args == NULL) {
+				network_state_log(LOG_ERROR, s, "Unable to split argument %s", opt_arg);
+				return -1;
+			}
+
+			if (join_args[0]) 
+				c->throttle_x = atoi(join_args[0]);
+			if (join_args[0] && join_args[1]) 
+				c->throttle_y = atoi(join_args[1]);
+		} else {
+			c->throttle_x = 0;
+			c->throttle_y = 0;
+		}
+
+		return 1;
+	} else if (mode == 'd') { /* Realname ban */
+		if (opt_arg == NULL) {
+			network_state_log(LOG_WARNING, s, "Missing argument for realname ban MODE set/unset");
+			return -1;
+		}
+
+		if (set) {
+			c->realnamebanlist = g_list_append(c->realnamebanlist, g_strdup(opt_arg));
+		} else {
+			char *be = find_realnamebanlist_entry(c->realnamebanlist, opt_arg);
+			if (be == NULL) {
+				network_state_log(LOG_WARNING, s, "Unable to remove nonpresent realname ban list entry '%s'", opt_arg);
+				return 1;
+			}
+			c->realnamebanlist = g_list_remove(c->realnamebanlist, be);
+			g_free(be);
+		}
+		return 1;	
+	} else if (mode == 'l') { /* Limit */
+		modes_change_mode(c->modes, set, 'l');
+		if (set) {
+			if (!opt_arg) {
+				network_state_log(LOG_WARNING, s, "Mode +l requires argument, but no argument found");
+				return -1;
+			}
+			c->limit = atol(opt_arg);
+		} else {
+			c->limit = 0;
+		}
+		return 1;
+	} else if (mode == 'k') {
+		modes_change_mode(c->modes, set, 'k');
+		if (set) {
+			if (opt_arg == NULL) {
+				network_state_log(LOG_WARNING, s, "Mode k requires argument, but no argument found");
+				return -1;
+			}
+
+			g_free(c->key);
+			c->key = g_strdup(opt_arg);
+		} else {
+			g_free(c->key);
+			c->key = NULL;
+		}
+		return 1;
+	} else if (is_prefix_mode(info, mode)) {
+		struct channel_nick *n;
+		
+		if (opt_arg == NULL) {
+			network_state_log(LOG_WARNING, s, "Mode %c requires nick argument, but no argument found", mode);
+			return -1;
+		}
+
+		n = find_channel_nick(c, opt_arg); 
+		if (!n) {
+			network_state_log(LOG_WARNING, s, "Can't set mode %c%c on nick %s on channel %s, because nick does not exist!", set?'+':'-', mode, opt_arg, c->name);
+			return -1;
+		}
+		if (set) {
+			modes_set_mode(n->modes, mode);
+		} else {
+			modes_unset_mode(n->modes, mode);
+		} 
+		return 1;
+	} else {
+		modes_change_mode(c->modes, set, mode);
+		return 0;
+	}
+}
+
+static void handle_mode(struct irc_network_state *s, const struct irc_line *l)
 {
 	/* Format:
 	 * MODE %|<nick>|<channel> [<mode> [<mode parameters>]] */
-	enum mode_type t = ADD;
+	gboolean t = TRUE;
 	int i;
+	int arg = 3;
+
+	g_assert(s != NULL);
 
 	/* Channel modes */
-	if (is_channelname(l->args[1], &s->info)) {
-		struct channel_state *c = find_channel(s, l->args[1]);
-		struct channel_nick *n;
-		char p;
-		int arg = 2;
+	if (is_channelname(l->args[1], s->info)) {
+		struct irc_channel_state *c = find_channel(s, l->args[1]);
+		struct network_nick *by;
+		int ret;
+		char *by_name;
 
 		if (c == NULL) {
 			network_state_log(LOG_WARNING, s, 
 				"Unable to change mode for unknown channel '%s'", l->args[1]);
 			return;
 		}
+
+		by_name = line_get_nick(l);
+		by = find_network_nick(s, by_name);
+		g_free(by_name);
 		
 		for(i = 0; l->args[2][i]; i++) {
 			switch(l->args[2][i]) {
-				case '+': t = ADD; break;
-				case '-': t = REMOVE; break;
-				case 'b': /* Ban */
-						  {
-							  struct banlist_entry *be = g_new0(struct banlist_entry, 1);
-							  be->time_set = time(NULL);
-							  be->hostmask = g_strdup(l->args[arg]);
-							  be->by = line_get_nick(l);
-							  c->banlist = g_list_append(c->banlist, be);
-						  }
-												
-						  arg++;
-						  break;
-				case 'l':
-					c->modes[(unsigned char)'l'] = t;
-				    if (t) {
-						if (!l->args[++arg]) {
-							network_state_log(LOG_WARNING, s, "Mode +l requires argument, but no argument found");
-							break;
-						}
-						c->limit = atol(l->args[arg]);
-					} else {
-						c->limit = 0;
-					}
-					break;
-				case 'k':
-					c->modes[(unsigned char)'k'] = t;
-					if (t) {
-						if (!l->args[++arg]) {
-							network_state_log(LOG_WARNING, s, "Mode k requires argument, but no argument found");
-							break;
-						}
-
-						g_free(c->key);
-						c->key = g_strdup(l->args[arg]);
-					} else {
-						g_free(c->key);
-						c->key = NULL;
-					}
-
-					break;
+				case '+': t = TRUE; break;
+				case '-': t = FALSE; break;
 				default:
-					  p = get_prefix_by_mode(l->args[2][i], &s->info);
-					  if (p == ' ') {
-						  c->modes[(unsigned char)l->args[2][i]] = t;
-					  } else {
-							n = find_channel_nick(c, l->args[++arg]);
-							if (!n) {
-								network_state_log(LOG_WARNING, s, "Can't set mode %c%c on nick %s on channel %s, because nick does not exist!", t == ADD?'+':'-', l->args[2][i], l->args[arg], l->args[1]);
-								break;
-							}
-							n->mode = (t == ADD?p:' ');
-					  }
+					  ret = channel_state_change_mode(s, by, c, t, 
+													  l->args[2][i],
+													  l->args[arg]);
+					  if (ret == -1)
+						  return;
+					  arg += ret;
 					  break;
 			}
 		}
+
 		/* User modes */
 	} else {
 		struct network_nick *nn = find_add_network_nick(s, l->args[1]);
 
 		for(i = 0; l->args[2][i]; i++) {
 			switch(l->args[2][i]) {
-				case '+': t = ADD;break;
-				case '-': t = REMOVE; break;
+				case '+': t = TRUE;break;
+				case '-': t = FALSE; break;
 				default:
-					  nn->modes[(unsigned char)l->args[2][i]] = t;
+					  modes_change_mode(nn->modes, t, l->args[2][i]);
 					  break;
 			}
 		}
 	}
+
+	if (arg > l->argc) {
+		network_state_log(LOG_WARNING, s,
+						  "not enough arguments for mode change %s on %s", 
+						  l->args[2], l->args[1]);
+	} else if (l->args[arg] != NULL && strcmp(l->args[arg], "") != 0) {
+		network_state_log(LOG_WARNING, s, 
+						  "mode %s %s argument not consumed: %s", l->args[2], 
+						  l->args[1], 
+						  l->args[arg]);
+	}
 }
 
-static void handle_001(struct network_state *s, const struct line *l)
+static void handle_001(struct irc_network_state *s, const struct irc_line *l)
 {
 	g_free(s->me.nick);
 	s->me.nick = g_strdup(l->args[1]);
 }
 
-static void handle_004(struct network_state *s, const struct line *l)
+static void handle_004(struct irc_network_state *s, const struct irc_line *l)
 {
-	s->info.supported_user_modes = g_strdup(l->args[4]);
-	s->info.supported_channel_modes = g_strdup(l->args[5]);
-	s->info.server = g_strdup(l->args[2]);
+	s->info->supported_user_modes = g_strdup(l->args[4]);
+	s->info->supported_channel_modes = g_strdup(l->args[5]);
+	s->info->server = g_strdup(l->args[2]);
 }
 
-static void handle_privmsg(struct network_state *s, const struct line *l)
+static void handle_privmsg(struct irc_network_state *s, const struct irc_line *l)
 {
 	struct network_nick *nn;
 	char *nick;
 
 	CHECK_ORIGIN(s,l,"PRIVMSG");
 
-	if (irccmp(&s->info, l->args[1], s->me.nick) != 0) return;
+	if (irccmp(s->info, l->args[1], s->me.nick) != 0) return;
 
 	nick = line_get_nick(l);
 	nn = find_add_network_nick(s, nick);
@@ -896,7 +1079,7 @@ static void handle_privmsg(struct network_state *s, const struct line *l)
 	g_free(nick);
 }
 
-static void handle_nick(struct network_state *s, const struct line *l)
+static void handle_nick(struct irc_network_state *s, const struct irc_line *l)
 {
 	struct network_nick *nn;
 	char *nick;
@@ -909,18 +1092,17 @@ static void handle_nick(struct network_state *s, const struct line *l)
 	network_nick_set_nick(nn, l->args[1]);
 }
 
-static void handle_umodeis(struct network_state *s, const struct line *l)
+static void handle_umodeis(struct irc_network_state *s, const struct irc_line *l)
 {
-	int i;
-	memset(s->me.modes, 0, sizeof(s->me.modes));
-	for (i = 0; i < strlen(l->args[1]); i++) {
-		s->me.modes[(unsigned char)l->args[1][i]] = 1;
+	if (!string2mode(l->args[1], s->me.modes)) {
+		network_state_log(LOG_WARNING, s, 
+						  "Invalid MODE change %s for user", l->args[1]);
 	}
 }
 
-static void handle_324(struct network_state *s, const struct line *l)
+static void handle_324(struct irc_network_state *s, const struct irc_line *l)
 {
-	struct channel_state *ch = find_channel(s, l->args[2]);
+	struct irc_channel_state *ch = find_channel(s, l->args[2]);
 
 	if (ch == NULL) {
 		network_state_log(LOG_WARNING, s, 
@@ -928,14 +1110,17 @@ static void handle_324(struct network_state *s, const struct line *l)
 		return;
 	}
 
-	string2mode(l->args[3], ch->modes);
+	if (!string2mode(l->args[3], ch->modes)) {
+		network_state_log(LOG_WARNING, s, 
+						  "Invalid MODE change %s for channel %s", l->args[3], l->args[2]);
+	}
 
 	ch->mode_received = TRUE;
 }
 
-static void handle_329(struct network_state *s, const struct line *l)
+static void handle_329(struct irc_network_state *s, const struct irc_line *l)
 {
-	struct channel_state *ch = find_channel(s, l->args[2]);
+	struct irc_channel_state *ch = find_channel(s, l->args[2]);
 
 	if (ch == NULL) {
 		network_state_log(LOG_WARNING, s, 
@@ -946,27 +1131,33 @@ static void handle_329(struct network_state *s, const struct line *l)
 	ch->creation_time = atol(l->args[3]);
 }
 
-static void handle_302(struct network_state *s, const struct line *l)
+static void handle_302(struct irc_network_state *s, const struct irc_line *l)
 {
-	/* We got a USERHOST response, split it into nick and user@host, and check the nick */
-	gchar** tmp302 = g_strsplit(g_strstrip(l->args[2]), "=+", 2);
-	if (g_strv_length(tmp302) > 1) {
-		char *hm;
-		struct network_nick *nn = find_add_network_nick(s, tmp302[0]);
-		
-		hm = g_strdup_printf("%s!%s", tmp302[0], tmp302[1]);
-		network_nick_set_hostmask(nn, hm);
-		g_free(hm);
+	int i;
+	gchar **users = g_strsplit(g_strstrip(l->args[2]), " ", 0);
+	for (i = 0; users[i]; i++) {
+		/* We got a USERHOST response, split it into nick and user@host, and check the nick */
+		gchar** tmp302 = g_strsplit(users[i], "=+", 2);
+		if (g_strv_length(tmp302) > 1) {
+			char *hm;
+			struct network_nick *nn = find_add_network_nick(s, tmp302[0]);
+			
+			hm = g_strdup_printf("%s!%s", tmp302[0], tmp302[1]);
+			network_nick_set_hostmask(nn, hm);
+			g_free(hm);
+		}
+		g_strfreev(tmp302);
 	}
-	g_strfreev(tmp302);
+
+	g_strfreev(users);
 }
 
-extern void handle_005(struct network_state *s, const struct line *l);
+extern void handle_005(struct irc_network_state *s, const struct irc_line *l);
 
 static struct irc_command {
 	char *command;
 	int min_args;
-	void (*handler) (struct network_state *s, const struct line *l);
+	void (*handler) (struct irc_network_state *s, const struct irc_line *l);
 } irc_commands[] = {
 	{ "JOIN", 1, handle_join },
 	{ "PART", 1, handle_part },
@@ -1001,7 +1192,7 @@ static struct irc_command {
 	{ NULL }
 };
 
-gboolean state_handle_data(struct network_state *s, const struct line *l)
+gboolean state_handle_data(struct irc_network_state *s, const struct irc_line *l)
 {
 	int i,j;
 
@@ -1022,19 +1213,19 @@ gboolean state_handle_data(struct network_state *s, const struct line *l)
 	return FALSE;
 }
 
-struct network_state *network_state_init(const char *nick, 
+struct irc_network_state *network_state_init(const char *nick, 
 										 const char *username, 
 										 const char *hostname)
 {
-	struct network_state *state = g_new0(struct network_state, 1);
+	struct irc_network_state *state = g_new0(struct irc_network_state, 1);
 	state->me.query = 1;
 	network_nick_set_data(&state->me, nick, username, hostname);
-	network_info_init(&state->info);
+	state->info = network_info_init();
 
 	return state;
 }
 
-void free_network_nick(struct network_state *st, struct network_nick *nn)
+void free_network_nick(struct irc_network_state *st, struct network_nick *nn)
 {
 	g_assert(nn != &st->me);
 	g_assert(nn);
@@ -1057,13 +1248,13 @@ void free_network_nick(struct network_state *st, struct network_nick *nn)
 	g_free(nn);
 }
 
-void free_network_state(struct network_state *state)
+void free_network_state(struct irc_network_state *state)
 {
 	if (state == NULL)
 		return;
 
 	while (state->channels != NULL)
-		free_channel((struct channel_state *)state->channels->data);
+		free_channel_state((struct irc_channel_state *)state->channels->data);
 
 	g_free(state->me.nick);
 	g_free(state->me.username);
@@ -1076,12 +1267,12 @@ void free_network_state(struct network_state *state)
 		free_network_nick(state, nn);
 	}
 
-	free_network_info(&state->info);
+	free_network_info(state->info);
 	g_free(state);
 }
 
 void network_state_log(enum log_level l, 
-					   const struct network_state *st, const char *fmt, ...)
+					   const struct irc_network_state *st, const char *fmt, ...)
 {
 	char *ret;
 	va_list ap;
@@ -1101,10 +1292,59 @@ void network_state_log(enum log_level l,
 	g_free(ret);
 }
 
-void network_state_set_log_fn(struct network_state *st, 
+void network_state_set_log_fn(struct irc_network_state *st, 
 							  void (*fn) (enum log_level, void *, const char *),
 							  void *userdata)
 {
 	st->log = fn;
 	st->userdata = userdata;
 }
+
+gboolean string2mode(const char *modes, irc_modes_t ar)
+{
+	gboolean action = TRUE;
+	modes_clear(ar);
+
+	if (modes == NULL)
+		return TRUE;
+
+	if (strlen(modes) == 0)
+		return TRUE;
+
+	if (modes[0] != '+' && modes[0] != '-')
+		return FALSE;
+
+	for (; *modes; modes++) {
+		switch (*modes) {
+			case '+': action = TRUE; break;
+			case '-': action = FALSE; break;
+			default: modes_change_mode(ar, action, *modes); break;
+		}
+	}
+
+	return TRUE;
+}
+
+char *mode2string(irc_modes_t modes)
+{
+	char ret[256];
+	unsigned char i;
+	int pos = 0;
+	ret[0] = '\0';
+	for(i = 0; i < MAXMODES; i++) {
+		if (modes[i]) { ret[pos] = (char)i; pos++; }
+	}
+	ret[pos] = '\0';
+
+	if (strlen(ret) == 0) {
+		return NULL;
+	} else {
+		return g_strdup_printf("+%s", ret);
+	}
+}
+
+gboolean is_prefix_mode(const struct irc_network_info *info, char mode)
+{
+	return get_prefix_by_mode(mode, info) != ' ';
+}
+
