@@ -35,15 +35,18 @@ static void add_network_helper(admin_handle h, const char *name);
 static void del_network_helper(admin_handle h, const char *name);
 static void list_networks_helper(admin_handle h);
 
+#define ADMIN_USERNAME "ctrlproxy"
+#define ADMIN_NICK "ctrlproxy"
+
 /**
  * Determine the hostmask for the admin user.
  *
  * @param n Network for which to generate the hostmask.
  * @return Host mask, should be freed by the caller.
  */
-static char *admin_hostmask(struct network *n)
+static char *admin_hostmask(struct irc_network *n)
 {
-	return g_strdup_printf("ctrlproxy!ctrlproxy@%s", n->info.name);
+	return g_strdup_printf(ADMIN_NICK"!"ADMIN_USERNAME"@%s", n->info->name);
 }
 
 /**
@@ -54,7 +57,7 @@ static char *admin_hostmask(struct network *n)
  */
 static void privmsg_admin_out(admin_handle h, const char *data)
 {
-	struct client *c = h->client;
+	struct irc_client *c = h->client;
 	char *nick = c->state->me.nick;
 	char *hostmask;
 
@@ -74,7 +77,7 @@ static void privmsg_admin_out(admin_handle h, const char *data)
  */
 static void network_admin_out(admin_handle h, const char *data)
 {
-	struct client *c = h->client;
+	struct irc_client *c = h->client;
 	char *hostmask;
 
 	hostmask = admin_hostmask(c->network);
@@ -118,7 +121,7 @@ static void cmd_help(admin_handle h, char **args, void *userdata)
  * @param h Admin context handle
  * @return Client, or NULL if no client is associated.
  */
-struct client *admin_get_client(admin_handle h)
+struct irc_client *admin_get_client(admin_handle h)
 {
 	return h->client;
 }
@@ -133,7 +136,7 @@ struct global *admin_get_global(admin_handle h)
  * @param h Admin context handle
  * @return Network, or NULL if no network is associated.
  */
-struct network *admin_get_network(admin_handle h)
+struct irc_network *admin_get_network(admin_handle h)
 {
 	return h->network;
 }
@@ -186,8 +189,9 @@ static void cmd_del_network (admin_handle h, char **args, void *userdata)
 
 static void cmd_add_server (admin_handle h, char **args, void *userdata)
 {
-	struct network *n;
+	struct irc_network *n;
 	struct tcp_server_config *s;
+	struct network_config *nc;
 	char *t;
 
 	if (!args[1] || !args[2]) {
@@ -195,14 +199,16 @@ static void cmd_add_server (admin_handle h, char **args, void *userdata)
 		return;
 	}
 
-	n = find_network(admin_get_global(h), args[1]);
+	n = find_network(admin_get_global(h)->networks, args[1]);
 
 	if (!n) {
 		admin_out(h, "No such network '%s'", args[1]);
 		return;
 	}
 
-	if (n->config->type != NETWORK_TCP) {
+	nc = n->private_data;
+
+	if (nc->type != NETWORK_TCP) {
 		admin_out(h, "Not a TCP/IP network!");
 		return;
 	}
@@ -219,20 +225,20 @@ static void cmd_add_server (admin_handle h, char **args, void *userdata)
 	s->ssl = FALSE;
 	s->password = args[3]?g_strdup(args[3]):NULL;
 
-	n->config->type_settings.tcp_servers = g_list_append(n->config->type_settings.tcp_servers, s);
+	nc->type_settings.tcp.servers = g_list_append(nc->type_settings.tcp.servers, s);
 
 	admin_out(h, "Server added to `%s'", args[1]);
 }
 
 static void cmd_connect_network (admin_handle h, char **args, void *userdata)
 {
-	struct network *s;
+	struct irc_network *s;
 	if (!args[1]) {
 		 admin_out(h, "No network specified");
 		 return;
 	}
 
-	s = find_network(admin_get_global(h), args[1]);
+	s = find_network(admin_get_global(h)->networks, args[1]);
 
 	if (!s) {
 		admin_out(h, "No such network `%s'", args[1]);
@@ -263,10 +269,11 @@ static void cmd_connect_network (admin_handle h, char **args, void *userdata)
 
 static void cmd_disconnect_network (admin_handle h, char **args, void *userdata)
 {
-	struct network *n;
+	struct irc_network *n;
+	struct network_config *nc;
 
 	if (args[1] != NULL) {
-		n = find_network(admin_get_global(h), args[1]);
+		n = find_network(admin_get_global(h)->networks, args[1]);
 		if (!n) {
 			admin_out(h, "Can't find active network with that name");
 			return;
@@ -279,30 +286,36 @@ static void cmd_disconnect_network (admin_handle h, char **args, void *userdata)
 		}
 	}
 
+	nc = n->private_data;
+
 	if (n->connection.state == NETWORK_CONNECTION_STATE_NOT_CONNECTED) {
-		admin_out(h, "Already disconnected from `%s'", n->info.name);
-	} else if (n->config->type == NETWORK_VIRTUAL && 
+		admin_out(h, "Already disconnected from `%s'", n->info->name);
+	} else if (nc->type == NETWORK_VIRTUAL && 
 		n->connection.data.virtual.ops->not_disconnectable) {
 		admin_out(h, "Built-in network `%s' can't be disconnected", 
-				  n->info.name);
+				  n->info->name);
 	} else {
-		admin_out(h, "Disconnecting from `%s'", n->info.name);
+		admin_out(h, "Disconnecting from `%s'", n->info->name);
 		disconnect_network(n);
 	}
 }
 
 static void cmd_next_server (admin_handle h, char **args, void *userdata) 
 {
-	struct network *n;
+	struct irc_network *n;
 	const char *name;
 
 
 	if (args[1] != NULL) {
 		name = args[1];
-		n = find_network(admin_get_global(h), args[1]);
+		n = find_network(admin_get_global(h)->networks, args[1]);
 	} else {
 		n = admin_get_network(h);
-		name = n->info.name;
+		if (n == NULL) {
+			admin_out(h, "No current network");
+			return;
+		}
+		name = n->info->name;
 	}
 	if (!n) {
 		admin_out(h, "%s: Not connected", name);
@@ -326,9 +339,9 @@ static void cmd_save_config (admin_handle h, char **args, void *userdata)
 static void add_network_helper(admin_handle h, const char *name)
 {
 	struct network_config *nc;
-	struct network *n;
+	struct irc_network *n;
 
-	if (find_network(admin_get_global(h), name) != NULL) {
+	if (find_network(admin_get_global(h)->networks, name) != NULL) {
 		admin_out(h, "Network with name `%s' already exists", name);
 		return;
 	}
@@ -336,14 +349,13 @@ static void add_network_helper(admin_handle h, const char *name)
 	nc = network_config_init(admin_get_global(h)->config);
 	g_free(nc->name); nc->name = g_strdup(name);
 	n = load_network(admin_get_global(h), nc);
-	network_set_log_fn(n, (network_log_fn)handle_network_log, n);
 
 	admin_out(h, "Network `%s' added. Use ADDSERVER to add a server to this network.", name);
 }
 
 static void del_network_helper(admin_handle h, const char *name)
 {
-	struct network *n = find_network(admin_get_global(h), name);
+	struct irc_network *n = find_network(admin_get_global(h)->networks, name);
 	if (n == NULL) {
 		admin_out(h, "No such network `%s'", name);
 		return;
@@ -353,6 +365,8 @@ static void del_network_helper(admin_handle h, const char *name)
 
 	unload_network(n);
 
+	config_del_network(admin_get_global(h)->config, name);
+
 	admin_out(h, "Network `%s' deleted", name);
 }
 
@@ -360,28 +374,28 @@ static void list_networks_helper(admin_handle h)
 {
 	GList *gl;
 	for (gl = admin_get_global(h)->networks; gl; gl = gl->next) {
-		struct network *n = gl->data;
+		struct irc_network *n = gl->data;
 
 		switch (n->connection.state) {
 		case NETWORK_CONNECTION_STATE_CONNECTING:
-			admin_out(h, "%s: Connect in progress", n->info.name);
+			admin_out(h, "%s: Connect in progress", n->info->name);
 			break;
 		case NETWORK_CONNECTION_STATE_CONNECTED:
-			admin_out(h, "%s: Connected, logging in", n->info.name);
+			admin_out(h, "%s: Connected, logging in", n->info->name);
 			break;
 		case NETWORK_CONNECTION_STATE_NOT_CONNECTED:
 			if (n->connection.data.tcp.last_disconnect_reason)
-				admin_out(h, "%s: Not connected: %s", n->info.name, 
+				admin_out(h, "%s: Not connected: %s", n->info->name, 
 						  n->connection.data.tcp.last_disconnect_reason);
 			else
-				admin_out(h, "%s: Not connected", n->info.name);
+				admin_out(h, "%s: Not connected", n->info->name);
 			break;
 		case NETWORK_CONNECTION_STATE_RECONNECT_PENDING:
-			admin_out(h, "%s: Reconnecting", n->info.name);
+			admin_out(h, "%s: Reconnecting", n->info->name);
 			break;
 		case NETWORK_CONNECTION_STATE_LOGIN_SENT:
 		case NETWORK_CONNECTION_STATE_MOTD_RECVD:
-			admin_out(h, "%s: connected", n->info.name);
+			admin_out(h, "%s: connected", n->info->name);
 			break;
 		}
 	}
@@ -428,38 +442,42 @@ static void cmd_list_networks(admin_handle h, char **args, void *userdata)
 
 static void cmd_detach(admin_handle h, char **args, void *userdata)
 {
-	struct client *c = admin_get_client(h);
+	struct irc_client *c = admin_get_client(h);
 
 	if (c == NULL) {
-		admin_out(h, "No client set");
+		admin_out(h, "No client to detach.");
 		return;
 	}
 
-	disconnect_client(c, "Client exiting");
+	client_disconnect(c, "Client exiting");
 }
 
 static void dump_joined_channels(admin_handle h, char **args, void *userdata)
 {
-	struct network *n;
+	struct irc_network *n;
 	GList *gl;
 
 	if (args[1] != NULL) {
-		n = find_network(admin_get_global(h), args[1]);
+		n = find_network(admin_get_global(h)->networks, args[1]);
 		if (n == NULL) {
 			admin_out(h, "Can't find network '%s'", args[1]);
 			return;
 		}
 	} else {
 		n = admin_get_network(h);
+		if (n == NULL) {
+			admin_out(h, "No network specified and no current network.");
+			return;
+		}
 	}
 
 	if (!n->state) {
-		admin_out(h, "Network '%s' not connected", n->info.name);
+		admin_out(h, "Network '%s' not connected", n->info->name);
 		return;
 	}
 
 	for (gl = n->state->channels; gl; gl = gl->next) {
-		struct channel_state *ch = (struct channel_state *)gl->data;
+		struct irc_channel_state *ch = (struct irc_channel_state *)gl->data;
 		admin_out(h, "%s", ch->name);
 	}
 }
@@ -481,10 +499,20 @@ static GHashTable *markers = NULL;
 static void cmd_backlog(admin_handle h, char **args, void *userdata)
 {
 	struct linestack_marker *lm;
-	struct network *n;
+	struct irc_network *n;
+
+	if (admin_get_client(h) == NULL) {
+		admin_out(h, "No client to send backlog to");
+		return;
+	}
 	
 	n = admin_get_network(h);
 
+	if (n == NULL) {
+		admin_out(h, "No network connected.");
+		return;
+	}
+
 	lm = g_hash_table_lookup(markers, n);
 
 	if (n->linestack == NULL) {
@@ -493,10 +521,11 @@ static void cmd_backlog(admin_handle h, char **args, void *userdata)
 	}
 
 	if (!args[1] || strlen(args[1]) == 0) {
-		admin_out(h, "Sending backlog for network '%s'", n->info.name);
+		admin_out(h, "Sending backlog for network '%s'", n->info->name);
 
 		linestack_send(n->linestack, lm, NULL, admin_get_client(h),
-					   TRUE, n->global->config->report_time != REPORT_TIME_NEVER);
+					   TRUE, n->global->config->report_time != REPORT_TIME_NEVER,
+					   n->global->config->report_time_offset);
 
 		g_hash_table_replace(markers, n, linestack_get_marker(n->linestack));
 
@@ -508,11 +537,50 @@ static void cmd_backlog(admin_handle h, char **args, void *userdata)
 
 	linestack_send_object(n->linestack, args[1], lm, NULL, 
 						  admin_get_client(h), TRUE, 
-						  n->global->config->report_time != REPORT_TIME_NEVER);
+						  n->global->config->report_time != REPORT_TIME_NEVER,
+						  n->global->config->report_time_offset);
 
 	g_hash_table_replace(markers, n, linestack_get_marker(n->linestack));
 }
 
+static char *linestack_get(admin_handle h)
+{
+	struct global *g = admin_get_global(h);
+
+	return g_strdup(g->config->linestack_backend);
+}
+
+static gboolean linestack_set(admin_handle h, const char *value)
+{
+	struct global *g = admin_get_global(h);
+
+	g_free(g->config->linestack_backend);
+	/* FIXME: Check if this is a valid linestack backend name */
+	g->config->linestack_backend = g_strdup(g->config->linestack_backend);
+
+	return TRUE;
+}
+
+static char *logging_get(admin_handle h)
+{
+	struct global *g = admin_get_global(h);
+
+	if (g->config->log_file == NULL)
+		return g_strdup("none");
+
+	if (g->config->log_file->is_irssi)
+		return g_strdup("irssi");
+
+	return g_strdup("custom");
+}
+
+static gboolean logging_set(admin_handle h, const char *value)
+{
+	/* FIXME: Implement */
+
+	return FALSE;
+}
+
 static char *log_level_get(admin_handle h)
 {
 	extern enum log_level current_log_level;
@@ -523,8 +591,12 @@ static char *log_level_get(admin_handle h)
 static gboolean log_level_set(admin_handle h, const char *value)
 {
 	extern enum log_level current_log_level;
+	int x;
 
-	int x = atoi(value);
+	if (value == NULL)
+		return FALSE;
+	
+	x = atoi(value);
 	if (x < 0 || x > 5) {
 		admin_out(h, "Invalid log level %d", x);
 		return FALSE;
@@ -537,7 +609,7 @@ static gboolean log_level_set(admin_handle h, const char *value)
 
 static void cmd_charset(admin_handle h, char **args, void *userdata)
 {
-	struct client *c;
+	struct irc_client *c;
 
 	if (args[1] == NULL) {
 		admin_out(h, "No charset specified");
@@ -545,6 +617,10 @@ static void cmd_charset(admin_handle h, char **args, void *userdata)
 	}
 
 	c = admin_get_client(h);
+	if (c == NULL) {
+		admin_out(h, "The CHARSET command can only be used on client connections.");
+		return;
+	}
 
 	if (!client_set_charset(c, args[1])) {
 		admin_out(h, "Error setting charset: %s", args[1]);
@@ -608,7 +684,7 @@ gboolean process_cmd(admin_handle h, const char *cmd)
 	return TRUE;
 }
 
-gboolean admin_process_command(struct client *c, struct line *l, int cmdoffset)
+gboolean admin_process_command(struct irc_client *c, struct irc_line *l, int cmdoffset)
 {
 	int i;
 	char *tmp;
@@ -640,10 +716,11 @@ gboolean admin_process_command(struct client *c, struct line *l, int cmdoffset)
 	return ret;
 }
 
-static gboolean admin_net_init(struct network *n)
+static gboolean admin_net_init(struct irc_network *n)
 {
 	char *hostmask;
 	char *nicks;
+	struct network_config *nc = n->private_data;
 
 	hostmask = admin_hostmask(n);
 	
@@ -651,7 +728,7 @@ static gboolean admin_net_init(struct network *n)
 	virtual_network_recv_response(n, RPL_TOPIC, ADMIN_CHANNEL, 
 		"CtrlProxy administration channel | Type `help' for more information",
 							  NULL);
-	nicks = g_strdup_printf("@ctrlproxy %s", n->config->nick);
+	nicks = g_strdup_printf("@ctrlproxy %s", nc->nick);
 
 	virtual_network_recv_response(n, RPL_NAMREPLY, "=", ADMIN_CHANNEL, nicks, NULL);
 	g_free(nicks);
@@ -662,7 +739,7 @@ static gboolean admin_net_init(struct network *n)
 	return TRUE;
 }
 
-static gboolean admin_to_server (struct network *n, struct client *c, const struct line *l)
+static gboolean admin_to_server (struct irc_network *n, struct irc_client *c, const struct irc_line *l)
 {
 	if (!g_strcasecmp(l->args[0], "PRIVMSG") ||
 		!g_strcasecmp(l->args[0], "NOTICE")) {
@@ -718,14 +795,14 @@ static gboolean admin_to_server (struct network *n, struct client *c, const stru
 
 		for (i = 1; l->args[i]; i++) {
 			if (!g_strcasecmp(l->args[i], "ctrlproxy")) {
-				gl = g_list_append(gl, g_strdup_printf("%s=+%s", l->args[i], get_my_hostname()));
+				gl = g_list_append(gl, g_strdup_printf("%s=+%s@%s", l->args[i], ADMIN_USERNAME, get_my_hostname()));
 			}
 			if (!g_strcasecmp(l->args[i], n->state->me.nick)) {
-				gl = g_list_append(gl, g_strdup_printf("%s=+%s", l->args[i], n->state->me.hostname));
+				gl = g_list_append(gl, g_strdup_printf("%s=+%s@%s", l->args[i], n->state->me.username, n->state->me.hostname));
 			}
 		}
 
-		virtual_network_recv_response(n, RPL_ISON, tmp = list_make_string(gl), NULL);
+		virtual_network_recv_response(n, RPL_USERHOST, tmp = list_make_string(gl), NULL);
 		g_free(tmp);
 		while (gl) {
 			g_free(gl->data);
@@ -807,10 +884,10 @@ struct virtual_network_ops admin_network = {
 };
 
 
-void admin_log(enum log_level level, const struct network *n, const struct client *c, const char *data)
+void admin_log(enum log_level level, const struct irc_network *n, const struct irc_client *c, const char *data)
 {
 	extern struct global *my_global;
-	struct line *l;
+	struct irc_line *l;
 	char *tmp, *hostmask;
 	GList *gl;
 	static gboolean entered = FALSE;
@@ -830,13 +907,13 @@ void admin_log(enum log_level level, const struct network *n, const struct clien
 	tmp = g_strdup_printf("%s%s%s%s%s%s", 
 						  data, 
 						  n?" (":"",
-						  n?n->info.name:"", 
+						  n?n->info->name:"", 
 						  c?"/":"",
 						  c?c->description:"",
 						  n?")":"");
 
 	for (gl = my_global->networks; gl; gl = gl->next) {
-		struct network *network = gl->data;
+		struct irc_network *network = gl->data;
 
 		if (network->connection.data.virtual.ops != &admin_network)
 			continue;
@@ -859,7 +936,7 @@ static void cmd_start_listener(admin_handle h, char **args, void *userdata)
 {
 	char *b, *p;
 	struct listener_config *cfg;
-	struct listener *l;
+	struct irc_listener *l;
 
 	if (!args[1]) {
 		admin_out(h, "No port specified");
@@ -886,7 +963,7 @@ static void cmd_start_listener(admin_handle h, char **args, void *userdata)
 
 	if (args[3]) {
 		cfg->network = g_strdup(args[3]);
-		if (find_network(admin_get_global(h), args[3]) == NULL) {
+		if (find_network(admin_get_global(h)->networks, args[3]) == NULL) {
 			admin_out(h, "No such network `%s`", args[3]);
 			return;
 		}
@@ -896,7 +973,7 @@ static void cmd_start_listener(admin_handle h, char **args, void *userdata)
 	l->config->password = g_strdup(args[2]);
 	l->global = admin_get_global(h);
 
-	start_listener(l);
+	listener_start(l, l->config->address, l->config->port);
 }
 
 static void cmd_stop_listener(admin_handle h, char **args, void *userdata)
@@ -920,7 +997,7 @@ static void cmd_stop_listener(admin_handle h, char **args, void *userdata)
 	}
 
 	for (gl = admin_get_global(h)->listeners; gl; gl = gl->next) {
-		struct listener *l = gl->data;
+		struct irc_listener *l = gl->data;
 
 		if (b && l->config->address == NULL)
 			continue;
@@ -931,7 +1008,7 @@ static void cmd_stop_listener(admin_handle h, char **args, void *userdata)
 		if (strcmp(p, l->config->port) != 0)
 			continue;
 
-		stop_listener(l);
+		listener_stop(l);
 		free_listener(l);
 		i++;
 	}
@@ -946,10 +1023,10 @@ static void cmd_list_listener(admin_handle h, char **args, void *userdata)
 	GList *gl;
 
 	for (gl = admin_get_global(h)->listeners; gl; gl = gl->next) {
-		struct listener *l = gl->data;
+		struct irc_listener *l = gl->data;
 
 		admin_out(h, "%s:%s%s%s%s", l->config->address?l->config->address:"", l->config->port, 
-				  l->network?" (":"", l->network?l->network->info.name:"", 
+				  l->network?" (":"", l->network?l->network->info->name:"", 
 				  l->network?")":"");
 	}
 }
@@ -979,6 +1056,11 @@ static char *motd_file_get(admin_handle h)
 static gboolean interpret_boolean(admin_handle h, const char *value,
 								  gboolean *ret)
 {
+	if (value == NULL) {
+		admin_out(h, "Boolean settings can't be unset.");
+		return FALSE;
+	}
+
 	if (!g_strcasecmp(value, "true")) {
 		*ret = TRUE;
 		return TRUE;
@@ -1089,6 +1171,9 @@ BOOL_SETTING(learn_nickserv)
 static char *max_who_age_get(admin_handle h)
 {
 	struct global *g = admin_get_global(h);
+
+	if (g->config->max_who_age == 0)
+		return NULL;
 	
 	return g_strdup_printf("%d", g->config->max_who_age);
 }
@@ -1097,11 +1182,258 @@ static gboolean max_who_age_set(admin_handle h, const char *value)
 {
 	struct global *g = admin_get_global(h);
 
-	g->config->max_who_age = atoi(value);
+	g->config->max_who_age = (value == NULL?0:atoi(value));
 
 	return TRUE;
 }
 
+static char *auto_away_time_get(admin_handle h)
+{
+	struct global *g = admin_get_global(h);
+
+	if (g->config->auto_away.max_idle_time == 0)
+		return NULL;
+	
+	return g_strdup_printf("%d", g->config->auto_away.max_idle_time);
+}
+
+static gboolean auto_away_time_set(admin_handle h, const char *value)
+{
+	struct global *g = admin_get_global(h);
+
+	g->config->auto_away.max_idle_time = (value == NULL?-1:atoi(value));
+
+	/* FIXME: Restart auto-away */
+
+	return TRUE;
+}
+
+static char *auto_away_enable_get(admin_handle h)
+{
+	struct global *g = admin_get_global(h);
+	
+	return g_strdup(g->config->auto_away.enabled?"true":"false");
+}
+
+static gboolean auto_away_enable_set(admin_handle h, const char *value)
+{
+	struct global *g = admin_get_global(h);
+
+	return interpret_boolean(h, value, &g->config->auto_away.enabled);
+}
+
+static char *auto_away_message_get(admin_handle h)
+{
+	struct global *g = admin_get_global(h);
+	
+	return g_strdup(g->config->auto_away.message);
+}
+
+static gboolean auto_away_message_set(admin_handle h, const char *value)
+{
+	struct global *g = admin_get_global(h);
+
+	g_free(g->config->auto_away.message);
+
+	g->config->auto_away.message = g_strdup(value);
+
+	return TRUE;
+}
+
+static char *auto_away_nick_get(admin_handle h)
+{
+	struct global *g = admin_get_global(h);
+	
+	return g_strdup(g->config->auto_away.nick);
+}
+
+static gboolean auto_away_nick_set(admin_handle h, const char *value)
+{
+	struct global *g = admin_get_global(h);
+
+	g_free(g->config->auto_away.nick);
+
+	g->config->auto_away.nick = g_strdup(value);
+
+	return TRUE;
+}
+
+static char *report_time_offset_get(admin_handle h)
+{
+	struct global *g = admin_get_global(h);
+
+	if (g->config->report_time_offset == 0)
+		return NULL;
+	
+	return g_strdup_printf("%d", g->config->report_time_offset);
+}
+
+static gboolean report_time_offset_set(admin_handle h, const char *value)
+{
+	struct global *g = admin_get_global(h);
+
+	g->config->report_time_offset = (value == NULL?0:atoi(value));
+
+	return TRUE;
+}
+
+static char *port_get(admin_handle h)
+{
+	GList *gl;
+	struct global *g = admin_get_global(h);
+
+	for (gl = g->config->listeners; gl; gl = gl->next) {
+		struct listener_config *l = gl->data;
+
+		if (l->is_default)
+			return g_strdup_printf("%s", l->port);
+	}
+
+	return NULL;
+}
+
+static gboolean port_set(admin_handle h, const char *value)
+{
+	GList *gl;
+	struct global *g = admin_get_global(h);
+
+	/* FIXME: What if there is no default listener ? */
+	/* FIXME: Check that value is a valid service name or port number */
+
+	for (gl = g->config->listeners; gl; gl = gl->next) {
+		struct listener_config *l = gl->data;
+
+		if (l->is_default) {
+			l->port = g_strdup(value);
+			return TRUE;
+		}
+	}
+
+	return FALSE;
+}
+
+static char *bind_get(admin_handle h)
+{
+	GList *gl;
+	struct global *g = admin_get_global(h);
+
+	for (gl = g->config->listeners; gl; gl = gl->next) {
+		struct listener_config *l = gl->data;
+
+		if (l->is_default)
+			return g_strdup(l->address);
+	}
+
+	return NULL;
+}
+
+static gboolean bind_set(admin_handle h, const char *value)
+{
+	GList *gl;
+	struct global *g = admin_get_global(h);
+
+	/* FIXME: What if there is no default listener ? */
+	/* FIXME: Check that value is a valid service name or port number */
+
+	for (gl = g->config->listeners; gl; gl = gl->next) {
+		struct listener_config *l = gl->data;
+
+		if (l->is_default) {
+			l->address = g_strdup(value);
+			return TRUE;
+		}
+	}
+
+	return FALSE;
+}
+
+static char *password_get(admin_handle h)
+{
+	GList *gl;
+	struct global *g = admin_get_global(h);
+
+	for (gl = g->config->listeners; gl; gl = gl->next) {
+		struct listener_config *l = gl->data;
+
+		if (l->is_default)
+			return g_strdup_printf("%s", l->password);
+	}
+
+	return NULL;
+}
+
+static gboolean password_set(admin_handle h, const char *value)
+{
+	GList *gl;
+	struct global *g = admin_get_global(h);
+
+	/* FIXME: What if there is no default listener ? */
+	/* FIXME: Check that value is a valid service name or port number */
+
+	for (gl = g->config->listeners; gl; gl = gl->next) {
+		struct listener_config *l = gl->data;
+
+		if (l->is_default) {
+			l->password = g_strdup(value);
+			return TRUE;
+		}
+	}
+
+	return FALSE;
+}
+
+static char *default_network_get(admin_handle h)
+{
+	GList *gl;
+	struct global *g = admin_get_global(h);
+
+	for (gl = g->config->listeners; gl; gl = gl->next) {
+		struct listener_config *l = gl->data;
+
+		if (l->is_default)
+			return g_strdup(l->network);
+	}
+
+	return NULL;
+}
+
+static gboolean default_network_set(admin_handle h, const char *value)
+{
+	GList *gl;
+	struct global *g = admin_get_global(h);
+
+	/* FIXME: What if there is no default listener ? */
+	/* FIXME: Check that value is a valid service name or port number */
+
+	for (gl = g->config->listeners; gl; gl = gl->next) {
+		struct listener_config *l = gl->data;
+
+		if (l->is_default) {
+			l->network = g_strdup(value);
+			return TRUE;
+		}
+	}
+
+	return FALSE;
+}
+
+static char *default_client_charset_get(admin_handle h)
+{
+	struct global *g = admin_get_global(h);
+
+	return g_strdup(g->config->client_charset);
+}
+
+static gboolean default_client_charset_set(admin_handle h, const char *value)
+{
+	struct global *g = admin_get_global(h);
+
+	g->config->client_charset = g_strdup(value);
+
+	return TRUE;
+}
+
+
 /**
  * Table of administration settings that can be
  * viewed and changed using the SET command.
@@ -1113,17 +1445,44 @@ static struct admin_setting {
 } settings[] = {
 	{ "admin-log", admin_log_get, admin_log_set },
 	{ "admin-user", admin_user_get, admin_user_set },
+	{ "auto-away-enable", auto_away_enable_get, auto_away_enable_set },
+	{ "auto-away-message", auto_away_message_get, auto_away_message_set },
+	{ "auto-away-nick", auto_away_nick_get, auto_away_nick_set },
+	{ "auto-away-time", auto_away_time_get, auto_away_time_set },
 	{ "autosave", autosave_get, autosave_set },
+	{ "bind", bind_get, bind_set },
+	{ "default-client-charset", default_client_charset_get, default_client_charset_set },
+	{ "default-network", default_network_get, default_network_set },
 	{ "learn-network-name", learn_network_name_get, learn_network_name_set },
 	{ "learn-nickserv", learn_nickserv_get, learn_nickserv_set },
+	{ "linestack", linestack_get, linestack_set },
 	{ "log_level", log_level_get, log_level_set },
+	{ "logging", logging_get, logging_set },
 	{ "max_who_age", max_who_age_get, max_who_age_set },
 	{ "motd-file", motd_file_get, motd_file_set },
+	{ "password", password_get, password_set },
+	{ "port", port_get, port_set },
 	{ "report-time", report_time_get, report_time_set },
+	{ "report-time-offset", report_time_offset_get, report_time_offset_set },
 	{ "replication", replication_get, replication_set },
 	{ NULL, NULL, NULL }
 };
 
+static void cmd_unset(admin_handle h, char **args, void *userdata)
+{
+	int i;
+	if (args[1] == NULL) {
+		admin_out(h, "Usage: unset <setting>");
+		return;
+	}
+
+	for (i = 0; settings[i].name; i++) {
+		if (!strcasecmp(settings[i].name, args[1])) {
+			settings[i].set(h, NULL);
+		}
+	}
+}
+
 static void cmd_set(admin_handle h, char **args, void *userdata)
 {
 	int i;
@@ -1132,7 +1491,10 @@ static void cmd_set(admin_handle h, char **args, void *userdata)
 	if (args[1] == NULL) {
 		for (i = 0; settings[i].name != NULL; i++) {
 			tmp = settings[i].get(h);
-			admin_out(h, "%s = %s", settings[i].name, tmp);
+			if (tmp == NULL)
+				admin_out(h, "%s is not set", settings[i]);
+			else
+				admin_out(h, "%s = %s", settings[i].name, tmp);
 			g_free(tmp);
 		}
 	} else {
@@ -1161,6 +1523,7 @@ const static struct admin_command builtin_commands[] = {
 
 	/* Commands */
 	{ "SET", cmd_set },
+	{ "UNSET", cmd_unset },
 
 	{ "ADDSERVER", cmd_add_server },
 	{ "BACKLOG", cmd_backlog},
