@@ -24,61 +24,11 @@
 #include "internals.h"
 #include "irc.h"
 
-static void client_send_nameslist(struct client *client, struct channel_state *channel)
-{
-	GList *gl;
-	char cmode[2];
-
-	g_assert(client);
-	g_assert(channel);
-	
-	cmode[0] = channel->mode;
-	cmode[1] = 0;
-
-	for (gl = channel->nicks; gl; gl = gl->next)
-	{
-		struct channel_nick *cn = gl->data;
-		char *tmp;
-
-		g_assert(cn->global_nick);
-
-		if (cn->mode) {
-			tmp = g_strdup_printf("%c%s", cn->mode, cn->global_nick->nick);
-		} else {
-			tmp = g_strdup(cn->global_nick->nick);
-		}
-
-		g_assert(channel->name);
-		client_send_response(client, RPL_NAMREPLY, cmode, channel->name, tmp, NULL);
-		g_free(tmp);
-	}
-
-	client_send_response(client, RPL_ENDOFNAMES, channel->name, "End of /NAMES list", NULL);
-}
-
-
-static void client_send_banlist(struct client *client, struct channel_state *channel)
-{
-	GList *gl;
-
-	g_assert(channel);
-	g_assert(client);
-
-	for (gl = channel->banlist; gl; gl = gl->next)
-	{
-		struct banlist_entry *be = gl->data;
-		g_assert(be);
-		client_send_response(client, RPL_BANLIST, channel->name, be->hostmask, NULL);
-	}
-
-	client_send_response(client, RPL_ENDOFBANLIST, channel->name, "End of channel ban list", NULL);
-}
-
-static gboolean client_try_cache_mode(struct client *c, struct line *l)
+static gboolean client_try_cache_mode(struct irc_client *c, struct irc_network_state *net, struct irc_line *l)
 {
 	int i;
 	char m;
-	struct channel_state *ch;
+	struct irc_channel_state *ch;
 
 	g_assert(l);
 	g_assert(c);
@@ -90,10 +40,9 @@ static gboolean client_try_cache_mode(struct client *c, struct line *l)
 		
 	/* Queries in the form of MODE #channel mode */
 	if (l->argc == 3) {
-		g_assert(c->network);
-		g_assert(c->network->state);
+		g_assert(net);
 
-		ch = find_channel(c->network->state, l->args[1]);
+		ch = find_channel(net, l->args[1]);
 		if (!ch) return FALSE;
 
 		for (i = 0; (m = l->args[2][i]); i++) {
@@ -106,20 +55,12 @@ static gboolean client_try_cache_mode(struct client *c, struct line *l)
 		return TRUE;
 	/* Queries in the form MODE #channel */
 	} else if (l->argc == 2) {
-		char *mode;
-		ch = find_channel(c->network->state, l->args[1]);
+		ch = find_channel(net, l->args[1]);
 		if (!ch) return FALSE;
 
 		if (!ch->mode_received) return FALSE;
 
-		mode = mode2string(ch->modes);
-		client_send_response(c, RPL_CHANNELMODEIS, ch->name, mode, NULL);
-		g_free(mode);
-		if (ch->creation_time > 0) {
-			char time[20];
-			snprintf(time, sizeof(time), "%lu", ch->creation_time);
-			client_send_response(c, RPL_CREATIONTIME, ch->name, time, NULL);
-		}
+		client_send_channel_mode(c, ch);
 
 		return TRUE;
 	}
@@ -127,9 +68,9 @@ static gboolean client_try_cache_mode(struct client *c, struct line *l)
 	return FALSE;
 }
 
-static gboolean client_try_cache_topic(struct client *c, struct line *l)
+static gboolean client_try_cache_topic(struct irc_client *c, struct irc_network_state *net, struct irc_line *l)
 {
-	struct channel_state *ch;
+	struct irc_channel_state *ch;
 
 	if (l->argc < 2) return FALSE;
 	
@@ -139,27 +80,24 @@ static gboolean client_try_cache_topic(struct client *c, struct line *l)
 	/* No set requests */
 	if (l->args[2]) return FALSE;
 	
-	ch = find_channel(c->network->state, l->args[1]);
+	ch = find_channel(net, l->args[1]);
 	if (!ch) return FALSE;
 
-	if (ch->topic) {
-		client_send_response(c, RPL_TOPIC, ch->name, ch->topic, NULL);
-	} else {
-		client_send_response(c, RPL_NOTOPIC, ch->name, "No topic set", NULL);
-	}
+	client_send_topic(c, ch, TRUE);
 
 	return TRUE;
 }
 
-static gboolean client_try_cache_userhost(struct client *c, struct line *l)
+static gboolean client_try_cache_userhost(struct irc_client *c, struct irc_network_state *net, struct irc_line *l)
 {
 	return FALSE;
 }
 
-static gboolean client_try_cache_who(struct client *c, struct line *l)
+static gboolean client_try_cache_who(struct irc_client *c, struct irc_network_state *net, struct irc_line *l)
 {
-	struct channel_state *ch;
+	struct irc_channel_state *ch;
 	int max_who_age;
+
 	time_t now;
 	GList *gl;
 
@@ -184,10 +122,9 @@ static gboolean client_try_cache_who(struct client *c, struct line *l)
 	if (l->argc != 2) 
 		return FALSE;
 
-	g_assert(c->network);
-	g_assert(c->network->state);
+	g_assert(net);
 
-	ch = find_channel(c->network->state, l->args[1]);
+	ch = find_channel(net, l->args[1]);
 	if (ch == NULL)
 		return FALSE;
 
@@ -221,9 +158,9 @@ static gboolean client_try_cache_who(struct client *c, struct line *l)
 	return TRUE;
 }
 
-static gboolean client_try_cache_names(struct client *c, struct line *l)
+static gboolean client_try_cache_names(struct irc_client *c, struct irc_network_state *net, struct irc_line *l)
 {
-	struct channel_state *ch;
+	struct irc_channel_state *ch;
 
 	g_assert(l);
 	g_assert(c);
@@ -235,10 +172,9 @@ static gboolean client_try_cache_names(struct client *c, struct line *l)
 
 	if (l->argc != 2) return FALSE;
 
-	g_assert(c->network);
-	g_assert(c->network->state);
+	g_assert(net);
 
-	ch = find_channel(c->network->state, l->args[1]);
+	ch = find_channel(net, l->args[1]);
 	if (!ch) return FALSE;
 
 	client_send_nameslist(c, ch);
@@ -252,7 +188,7 @@ static gboolean client_try_cache_names(struct client *c, struct line *l)
 struct cache_command {
 	const char *name;
 	/* Should return FALSE if command couldn't be cached */
-	gboolean (*try_cache) (struct client *c, struct line *l);
+	gboolean (*try_cache) (struct irc_client *c, struct irc_network_state *net, struct irc_line *l);
 } cache_commands[] = {
 	{ "MODE", client_try_cache_mode },
 	{ "NAMES", client_try_cache_names },
@@ -263,7 +199,7 @@ struct cache_command {
 };
 
 /* Try to answer a client query from cache */
-gboolean client_try_cache(struct client *c, struct line *l)
+gboolean client_try_cache(struct irc_client *c, struct irc_network_state *net, struct irc_line *l)
 {
 	g_assert(l);
 	int i;
@@ -273,7 +209,7 @@ gboolean client_try_cache(struct client *c, struct line *l)
 
 	for (i = 0; cache_commands[i].name; i++) {
 		if (!g_strcasecmp(l->args[0], cache_commands[i].name))
-			return cache_commands[i].try_cache(c, l);
+			return cache_commands[i].try_cache(c, net, l);
 	}
 	
 	return FALSE;
