@@ -27,7 +27,7 @@
  */
 struct ctcp_handle 
 {
-	struct network *network;
+	struct irc_network *network;
 	char *nick;
 };
 
@@ -72,7 +72,7 @@ void ctcp_reply(struct ctcp_handle *h, ...)
 	g_free(msg);
 }
 
-void ctcp_send(struct network *n, const char *nick, ...)
+void ctcp_send(struct irc_network *n, const char *nick, ...)
 {
 	va_list ap;
 	char *msg;
@@ -84,30 +84,6 @@ void ctcp_send(struct network *n, const char *nick, ...)
 	g_free(msg);
 }
 
-static char *get_ctcp_command(const char *data)
-{
-	char *tmp;
-	char *p;
-	g_assert(data[0] == '\001');
-
-	tmp = g_strdup(data+1);
-
-	p = strchr(tmp, ' ');
-	if (p) {
-		*p = '\0';
-		return tmp;
-	}
-
-	p = strrchr(tmp, '\001');
-	if (p) {
-		*p = '\0';
-		return tmp;
-	}
-
-	g_free(tmp);
-	return NULL;
-}
-
 static void handle_time(struct ctcp_handle *h, char **args)
 {
 	time_t ti = time(NULL);
@@ -125,12 +101,7 @@ static void handle_finger(struct ctcp_handle *h, char **args)
 
 static void handle_source(struct ctcp_handle *h, char **args)
 {
-	ctcp_reply(h, "SOURCE", "ctrlproxy.vernstok.nl:pub/ctrlproxy:ctrlproxy-latest.tar.gz", NULL);
-}
-
-static void handle_clientinfo(struct ctcp_handle *h, char **args)
-{
-	ctcp_reply(h, "CLIENTINFO", "ACTION CLIENTINFO VERSION TIME FINGER SOURCE CLIENTINFO PING", NULL);
+	ctcp_reply(h, "SOURCE", "www.ctrlproxy.org:pub/ctrlproxy:ctrlproxy-latest.tar.gz", NULL);
 }
 
 static void handle_version(struct ctcp_handle *h, char **args)
@@ -152,6 +123,8 @@ static void handle_ping(struct ctcp_handle *h, char **args)
 	ctcp_reply(h, "PING", args[1], NULL);
 }
 
+static void handle_clientinfo(struct ctcp_handle *h, char **args);
+
 static const struct ctcp_handler builtins[] = {
 	{ "VERSION", handle_version },
 	{ "CLIENTINFO", handle_clientinfo },
@@ -159,122 +132,41 @@ static const struct ctcp_handler builtins[] = {
 	{ "PING", handle_ping },
 	{ "FINGER", handle_finger },
 	{ "TIME", handle_time },
+	{ "ACTION", NULL },
 	{ NULL, NULL }
 };
 
 static GList *cmds = NULL;
 
+static void handle_clientinfo(struct ctcp_handle *h, char **args)
+{
+	char **supported = g_new0(char *, sizeof(builtins)/sizeof(builtins[0])+1+g_list_length(cmds));
+	char *tmp;
+	int i;
+	GList *gl;
+
+	for (i = 0; builtins[i].name; i++) 
+		supported[i] = builtins[i].name;
+
+	for (gl = cmds; gl; gl = gl->next) {
+		supported[i] = ((struct ctcp_handler *)gl->data)->name;
+		i++;
+	}
+
+	supported[i] = NULL;
+
+	tmp = g_strjoinv(" ", supported);
+	ctcp_reply(h, "CLIENTINFO", tmp, NULL);
+	g_free(tmp);
+	g_free(supported);
+}
+
 void ctcp_register_handler(const struct ctcp_handler *h)
 {
 	cmds = g_list_append(cmds, g_memdup(h, sizeof(*h)));
 }
 
-/**
- * A CTCP request.
- */
-struct ctcp_request {
-	struct client *client;
-	char *destination;
-	char *command;
-};
-
-static GList *ctcp_request_queue = NULL;
-
-gboolean ctcp_process_client_request (struct client *c, struct line *l)
-{
-	struct ctcp_request *req;
-	char *command = get_ctcp_command(l->args[2]);
-
-	if (command == NULL) {
-		log_client(LOG_WARNING, c, "Received mailformed CTCP request");
-		return FALSE;
-	}
-
-	req = g_new0(struct ctcp_request, 1);
-
-	/* store client and command in table */
-	req->client = c;
-	if (!is_channelname(l->args[1], &c->network->info))
-		req->destination = g_strdup(l->args[1]);
-	req->command = command;
-
-	log_client(LOG_TRACE, c, "Tracking CTCP request '%s' to %s", req->command, req->destination);
-
-	ctcp_request_queue = g_list_append(ctcp_request_queue, req);
-
-	/* send off to server */
-	network_send_line(c->network, c, l, TRUE);
-
-	return TRUE;
-}
-
-gboolean ctcp_process_client_reply (struct client *c, struct line *l)
-{
-	char *command = get_ctcp_command(l->args[2]);
-
-	if (command == NULL) {
-		log_client(LOG_WARNING, c, "Received mailformed CTCP reply");
-		return FALSE;
-	}
-
-
-	log_client(LOG_WARNING, c, "Received CTCP client reply '%s' from client", command);
-	g_free(command);
-
-	return TRUE;
-}
-
-gboolean ctcp_process_network_reply (struct network *n, struct line *l) 
-{
-	GList *gl;
-	char *nick;
-
-	char *command = get_ctcp_command(l->args[2]);
-
-	nick = line_get_nick(l);
-
-	if (command == NULL) {
-		network_log(LOG_WARNING, n, "Received mailformed CTCP request from `%s'", nick);
-		g_free(nick);
-		return FALSE;
-	}
-
-
-	/* look in table */
-	/* if found send to specified client, remove entry from table */
-	for (gl = ctcp_request_queue; gl; gl = gl->next) {
-		struct ctcp_request *req = gl->data;
-
-		if (req->client->network != n)
-			continue;
-
-		if (req->destination && strcmp(req->destination, nick) != 0)
-			continue;
-
-		if (strcmp(req->command, command) != 0)
-			continue;
-
-		client_send_line(req->client, l);
-
-		g_free(req->command);
-		g_free(req->destination);
-		ctcp_request_queue = g_list_remove(ctcp_request_queue, req);
-		g_free(req);
-		g_free(nick);
-		g_free(command);
-
-		return TRUE;
-	}
-
-	/* otherwise, inform user */
-	network_log(LOG_WARNING, n, "Don't know where to send unknown CTCP reply '%s'", command);
-
-	g_free(command);
-
-	return TRUE;
-}
-
-gboolean ctcp_process_network_request (struct network *n, struct line *l) 
+gboolean ctcp_process_request (struct irc_network *n, const struct irc_line *l) 
 {
 	GList *gl;
 	int i;
@@ -298,12 +190,12 @@ gboolean ctcp_process_network_request (struct network *n, struct line *l)
 	*t = '\0';
 
 	args = g_strsplit(data, " ", 0);
-
 	for (gl = cmds; gl; gl = gl->next) {
 		struct ctcp_handler *hl = gl->data;
 
 		if (!g_strcasecmp(hl->name, args[0])) {
-			hl->fn(&h, args);
+			if (hl->fn != NULL)
+				hl->fn(&h, args);
 			ret = TRUE;
 			break;
 		}
@@ -311,7 +203,8 @@ gboolean ctcp_process_network_request (struct network *n, struct line *l)
 
 	for (i = 0; !ret && builtins[i].name; i++) {
 		if (!g_strcasecmp(builtins[i].name, args[0])) {
-			builtins[i].fn(&h, args);
+			if (builtins[i].fn != NULL) 
+				builtins[i].fn(&h, args);
 			ret = TRUE;
 			break;
 		}
@@ -323,6 +216,7 @@ gboolean ctcp_process_network_request (struct network *n, struct line *l)
 		ret = FALSE;
 	}
 
+
 	g_strfreev(args);
 	g_free(data);
 	g_free(h.nick);
