@@ -30,7 +30,7 @@
 /**
  * Current version of the plugin API.
  */
-#define CTRLPROXY_PLUGIN_VERSION 3
+#define CTRLPROXY_PLUGIN_VERSION 4
 
 /**
  * @file
@@ -53,7 +53,8 @@
 #endif
 
 #include "settings.h"
-#include "network.h"
+#include "connection.h"
+#include "util.h"
 #include "client.h"
 #include "state.h"
 #include "linestack.h"
@@ -65,6 +66,7 @@
 #include "isupport.h"
 #include "log_support.h"
 #include "listener.h"
+#include "url.h"
 
 /**
  * Global information.
@@ -87,8 +89,11 @@ struct global {
  * Plugin description. Required to be defined by all plugins.
  */
 struct plugin_ops {
+	/** API version this plugin uses. */
 	int version;
+	/** Name of the plugin. */
 	char *name;
+	/** Function that initializes the plugin. */
 	gboolean (*init) (void);
 };
 
@@ -101,17 +106,11 @@ typedef void (*config_save_notify_fn) (struct global *, const char *);
 G_MODULE_EXPORT void register_load_config_notify(config_load_notify_fn fn);
 G_MODULE_EXPORT void register_save_config_notify(config_save_notify_fn fn);
 
-/* util.c */
-G_MODULE_EXPORT char *list_make_string(GList *);
-G_MODULE_EXPORT int str_rfc1459cmp(const char *a, const char *b);
-G_MODULE_EXPORT int str_strictrfc1459cmp(const char *a, const char *b);
-G_MODULE_EXPORT int str_asciicmp(const char *a, const char *b);
-G_MODULE_EXPORT char *g_io_channel_ip_get_description(GIOChannel *ch);
-G_MODULE_EXPORT const char *g_io_channel_unix_get_sock_error(GIOChannel *ioc);
-
+typedef void (*hup_handler_fn) (void *);
+G_MODULE_EXPORT void register_hup_handler(hup_handler_fn, void *userdata);
 
 /* log.c */
-G_GNUC_PRINTF(3, 4) G_MODULE_EXPORT void log_network_state(enum log_level l, const struct network_state *st, const char *fmt, ...);
+G_GNUC_PRINTF(3, 4) G_MODULE_EXPORT void log_network_state(enum log_level l, const struct irc_network_state *st, const char *fmt, ...);
 
 gboolean    rep_g_file_get_contents             (const gchar *filename,
                                              gchar **contents,
@@ -133,5 +132,7 @@ int rep_g_mkdir_with_parents (const gchar *pathname, int mode);
 #if GLIB_MAJOR_VERSION == 2 && GLIB_MINOR_VERSION < 10
 #define G_GNUC_WARN_UNUSED_RESULT
 #endif
+gboolean write_pidfile(const char *filename);
+pid_t read_pidfile(const char *filename);
 
 #endif /* __CTRLPROXY_H__ */
