@@ -19,7 +19,7 @@
 
 #include "internals.h"
 
-static struct channel_config *config_find_channel(struct network_info *ni, struct network_config *nc, const char *name)
+static struct channel_config *config_find_channel(struct irc_network_info *ni, struct network_config *nc, const char *name)
 {
 	GList *gl;
 
@@ -39,20 +39,18 @@ static struct channel_config *config_find_channel(struct network_info *ni, struc
 	return NULL;
 }
 
-void network_update_config(struct network_state *ns, struct network_config *nc)
+void network_update_config(struct irc_network_state *ns, struct network_config *nc)
 {
 	GList *gl;
 
-	nc->autoconnect = 1;
-
 	if (ns == NULL)
 		return;
-	
+
 	/* Network name */
-	if (ns->info.name != NULL && 
-		(nc->name == NULL || strcmp(ns->info.name, nc->name) != 0)) {
+	if (ns->info->name != NULL && 
+		(nc->name == NULL || strcmp(ns->info->name, nc->name) != 0)) {
 		g_free(nc->name);
-		nc->name = g_strdup(ns->info.name);
+		nc->name = g_strdup(ns->info->name);
 	}
 	
 	/* nick */
@@ -62,39 +60,38 @@ void network_update_config(struct network_state *ns, struct network_config *nc)
 	for (gl = nc->channels; gl; gl = gl->next) {
 		struct channel_config *cc = gl->data;
 
-		cc->autojoin = 0;
+		cc->autojoin = FALSE;
 	}
 
 	for (gl = ns->channels; gl; gl = gl->next) {
-		struct channel_state *cs = gl->data;
+		struct irc_channel_state *cs = gl->data;
 		struct channel_config *cc;
 
 		/* Find channel */
-		cc = config_find_channel(&ns->info, nc, cs->name);
+		cc = config_find_channel(ns->info, nc, cs->name);
 		if (!cc) {
 			cc = g_new0(struct channel_config, 1);
+			cc->name = g_strdup(cs->name);
 			nc->channels = g_list_append(nc->channels, cc);
 		}
-		channel_update_config(cs, cc);
+		g_free(cc->key); 
+		cc->key = NULL;
+		if (cs->key) 
+			cc->key = g_strdup(cs->key);
+		cc->autojoin = TRUE;
 	}
-
-}
-
-void channel_update_config(struct channel_state *ns, struct channel_config *nc)
-{
-	g_free(nc->key); nc->key = NULL;
-	if (ns->key) 
-		nc->key = g_strdup(ns->key);
-	nc->autojoin = 1;
-	g_free(nc->name);
-	nc->name = g_strdup(ns->name);
 }
 
 void global_update_config(struct global *my_global)
 {
 	GList *gl;
 	for (gl = my_global->networks; gl; gl = gl->next) {
-		struct network *n = gl->data;
-		network_update_config(n->state, n->config);
+		struct irc_network *n = gl->data;
+		struct network_config *nc = n->private_data;
+	
+		nc->autoconnect = (n->connection.state != NETWORK_CONNECTION_STATE_NOT_CONNECTED);
+
+		if (n->connection.state == NETWORK_CONNECTION_STATE_MOTD_RECVD)
+			network_update_config(n->state, nc);
 	}
 }
