@@ -35,7 +35,7 @@ void register_linestack(const struct linestack_ops *b)
 struct linestack_context *create_linestack(const struct linestack_ops *ops, 
 										   const char *name, 
 										   struct ctrlproxy_config *cfg,
-										   const struct network_state *state)
+										   const struct irc_network_state *state)
 {
 	struct linestack_context *ctx;
 
@@ -82,11 +82,10 @@ struct linestack_marker *linestack_get_marker_numlines (struct linestack_context
 	return wrap_linestack_marker(ctx, ctx->ops->get_marker_numlines(ctx, lines));
 }
 
-struct network_state *linestack_get_state(
-		struct linestack_context *ctx,
-		struct linestack_marker *lm)
+struct irc_network_state *linestack_get_state(
+		struct linestack_context *ctx, struct linestack_marker *lm)
 {
-	struct network_state *st;
+	struct irc_network_state *st;
 	g_assert(ctx != NULL);
 
 	if (!ctx->ops) return NULL;
@@ -118,7 +117,7 @@ struct traverse_object_data {
 	void *userdata;
 };
 
-static gboolean traverse_object_handler(struct line *l, time_t t, void *state)
+static gboolean traverse_object_handler(struct irc_line *l, time_t t, void *state)
 {
 	struct traverse_object_data *d = state;
 	gboolean ret = TRUE;
@@ -197,35 +196,36 @@ static const char *linestack_messages[] = {
 	NULL };
 
 gboolean linestack_insert_line(struct linestack_context *ctx, 
-							   const struct line *l, enum data_direction dir, 
-							   const struct network_state *state)
+							   const struct irc_line *l, enum data_direction dir, 
+							   const struct irc_network_state *state)
 {
 	int i;
 	gboolean needed = FALSE;
 
-	if (ctx == NULL) return FALSE;
+	if (ctx == NULL) return TRUE;
 
-	if (l->argc == 0) return FALSE;
+	if (l->argc == 0) return TRUE;
 
-	if (!ctx->ops) return FALSE;
+	if (!ctx->ops) return TRUE;
 	g_assert(ctx->ops->insert_line);
 
 	/* Only need PRIVMSG and NOTICE messages we send ourselves */
 	if (dir == TO_SERVER && 
 		g_strcasecmp(l->args[0], "PRIVMSG") && 
-		g_strcasecmp(l->args[0], "NOTICE")) return FALSE;
+		g_strcasecmp(l->args[0], "NOTICE")) return TRUE;
 
 	/* No CTCP, please */
 	if ((!g_strcasecmp(l->args[0], "PRIVMSG") ||
 		!g_strcasecmp(l->args[0], "NOTICE")) && 
-		l->argc > 2 && l->args[2][0] == '\001')
-		return FALSE;
+		l->argc > 2 && l->args[2][0] == '\001' && 
+		g_strncasecmp(l->args[2], "\001ACTION", 7) != 0)
+		return TRUE;
 
 	for (i = 0; linestack_messages[i]; i++) 
 		if (!g_strcasecmp(linestack_messages[i], l->args[0]))
 			needed = TRUE;
 
-	if (!needed) return FALSE;
+	if (!needed) return TRUE;
 
 	for (i = 0; i < l->argc; i++) {
 		g_assert(strchr(l->args[i], '\n') == NULL);
@@ -235,34 +235,39 @@ gboolean linestack_insert_line(struct linestack_context *ctx,
 	return ctx->ops->insert_line(ctx, l, state);
 }
 
-static gboolean send_line(struct line *l, time_t t, void *_client)
+struct send_line_privdata {
+	int time_offset;
+	struct irc_client *client;
+};
+
+static gboolean send_line(struct irc_line *l, time_t t, void *_privdata)
 {
-	struct client *c = _client;
-	return client_send_line(c, l);
+	struct send_line_privdata *privdata = _privdata;
+	return client_send_line(privdata->client, l);
 }
 
-static gboolean send_line_timed(struct line *l, time_t t, void *_client)
+static gboolean send_line_timed(struct irc_line *l, time_t t, void *_privdata)
 {
-	struct client *c = _client;
+	struct send_line_privdata *privdata = _privdata;
 
 	if ((!g_strcasecmp(l->args[0], "PRIVMSG") ||
 		!g_strcasecmp(l->args[0], "NOTICE")) &&
 		l->argc > 2) {
-		struct line *nl = line_prefix_time(l, t);
+		struct irc_line *nl = line_prefix_time(l, t+privdata->time_offset);
 		gboolean ret;
-		ret = client_send_line(c, nl);
+		ret = client_send_line(privdata->client, nl);
 		free_line(nl);
 		return ret;
 	} else {
-		return client_send_line(c, l);
+		return client_send_line(privdata->client, l);
 	}
 }
 
-static gboolean send_line_timed_dataonly(struct line *l, time_t t, void *_client)
+static gboolean send_line_timed_dataonly(struct irc_line *l, time_t t, void *_privdata)
 {
-	struct client *c = _client;
+	struct send_line_privdata *privdata = _privdata;
 	gboolean ret;
-	struct line *nl;
+	struct irc_line *nl;
 
 	if (g_strcasecmp(l->args[0], "PRIVMSG") != 0 && 
 		g_strcasecmp(l->args[0], "NOTICE") != 0)
@@ -271,15 +276,15 @@ static gboolean send_line_timed_dataonly(struct line *l, time_t t, void *_client
 	if (l->argc <= 2)
 		return TRUE;
 
-	nl = line_prefix_time(l, t);
-	ret = client_send_line(c, nl);
+	nl = line_prefix_time(l, t+privdata->time_offset);
+	ret = client_send_line(privdata->client, nl);
 	free_line(nl);
 	return ret;
 }
 
-static gboolean send_line_dataonly(struct line *l, time_t t, void *_client)
+static gboolean send_line_dataonly(struct irc_line *l, time_t t, void *_privdata)
 {
-	struct client *c = _client;
+	struct send_line_privdata *privdata = _privdata;
 
 	if (g_strcasecmp(l->args[0], "PRIVMSG") != 0 && 
 		g_strcasecmp(l->args[0], "NOTICE") != 0)
@@ -288,13 +293,17 @@ static gboolean send_line_dataonly(struct line *l, time_t t, void *_client)
 	if (l->argc <= 2)
 		return TRUE;
 
-	return client_send_line(c, l);
+	return client_send_line(privdata->client, l);
 }
 
-gboolean linestack_send(struct linestack_context *ctx, struct linestack_marker *mf, struct linestack_marker *mt, struct client *c, gboolean dataonly, gboolean timed)
+gboolean linestack_send(struct linestack_context *ctx, struct linestack_marker *mf, struct linestack_marker *mt, struct irc_client *c, gboolean dataonly, gboolean timed, int time_offset)
 {
+	struct send_line_privdata privdata;
 	linestack_traverse_fn trav_fn;
 
+	privdata.client = c;
+	privdata.time_offset = time_offset;
+
 	if (dataonly) {
 		if (timed) 
 			trav_fn = send_line_timed_dataonly;
@@ -307,12 +316,17 @@ gboolean linestack_send(struct linestack_context *ctx, struct linestack_marker *
 			trav_fn = send_line;
 	}
 
-	return linestack_traverse(ctx, mf, mt, trav_fn, c);
+	return linestack_traverse(ctx, mf, mt, trav_fn, &privdata);
 }
 
-gboolean linestack_send_object(struct linestack_context *ctx, const char *obj, struct linestack_marker *mf, struct linestack_marker *mt, struct client *c, gboolean dataonly, gboolean timed)
+gboolean linestack_send_object(struct linestack_context *ctx, const char *obj, struct linestack_marker *mf, struct linestack_marker *mt, struct irc_client *c, gboolean dataonly, gboolean timed, int time_offset)
 {
+	struct send_line_privdata privdata;
 	linestack_traverse_fn trav_fn;
+
+	privdata.client = c;
+	privdata.time_offset = time_offset;
+
 	if (dataonly) {
 		if (timed) 
 			trav_fn = send_line_timed_dataonly;
@@ -325,12 +339,12 @@ gboolean linestack_send_object(struct linestack_context *ctx, const char *obj, s
 			trav_fn = send_line;
 	}
 
-	return linestack_traverse_object(ctx, obj, mf, mt, trav_fn, c);
+	return linestack_traverse_object(ctx, obj, mf, mt, trav_fn, &privdata);
 }
 
-static gboolean replay_line(struct line *l, time_t t, void *state)
+static gboolean replay_line(struct irc_line *l, time_t t, void *state)
 {
-	struct network_state *st = state;
+	struct irc_network_state *st = state;
 	state_handle_data(st, l);
 	return TRUE;
 }
@@ -338,7 +352,7 @@ static gboolean replay_line(struct line *l, time_t t, void *state)
 gboolean linestack_replay(struct linestack_context *ctx, 
 						  struct linestack_marker *mf, 
 						  struct linestack_marker *mt, 
-						  struct network_state *st)
+						  struct irc_network_state *st)
 {
 	return linestack_traverse(ctx, mf, mt, replay_line, st);
 }
@@ -355,7 +369,7 @@ struct linestack_ops *linestack_find_ops(const char *name)
 	return NULL;
 }
 
-struct linestack_context *new_linestack(struct network *n)
+struct linestack_context *new_linestack(struct irc_network *n)
 {
 	const struct linestack_ops *current_backend = NULL;
 	struct ctrlproxy_config *cfg = NULL;
@@ -376,7 +390,7 @@ struct linestack_context *new_linestack(struct network *n)
 		current_backend = &linestack_file;
 	}
 
-	return create_linestack(current_backend, n->info.name, cfg, n->state);
+	return create_linestack(current_backend, n->info->name, cfg, n->state);
 }
 
 
