@@ -36,39 +36,48 @@
 
 enum marshall_mode { MARSHALL_PULL = 0, MARSHALL_PUSH = 1 };
 
-typedef gboolean (*marshall_fn_t) (struct network_state *, const char *name, int level, enum marshall_mode, GIOChannel *ch, void **data);
+typedef gboolean (*marshall_fn_t) (struct irc_network_state *, const char *name, int level, enum marshall_mode, GIOChannel *ch, void **data);
 
 #define marshall_new(m,t) if ((m) == MARSHALL_PULL) *(t) = g_malloc(sizeof(**t));
 
+#define LF_CHECK_IO_STATUS(status)	if (status != G_IO_STATUS_NORMAL) { \
+		log_global(LOG_ERROR, "Unable to write to linestack file: %s", error->message); \
+		return FALSE; \
+	}
 
 static const char tabs[10] = {'\t', '\t', '\t', '\t', '\t', 
 			       '\t', '\t', '\t', '\t', '\t' };
 
-static gboolean marshall_set (GIOChannel *t, int level, const char *name, const char *value)
+
+static GIOStatus marshall_set (GIOChannel *t, int level, const char *name, const char *value, GError **error)
 {
 	GIOStatus status;
-	GError *error = NULL;
-	status = g_io_channel_write_chars(t, tabs, level, NULL, &error);
-	g_assert (status == G_IO_STATUS_NORMAL);
+	status = g_io_channel_write_chars(t, tabs, level, NULL, error);
+	if (status != G_IO_STATUS_NORMAL)
+		return status;
 
 	g_assert(!strchr(name, '\n'));
 	g_assert(value == NULL || !strchr(value, '\n'));
 
-	status = g_io_channel_write_chars(t, name, -1, NULL, &error);
-	g_assert(status == G_IO_STATUS_NORMAL);
+	status = g_io_channel_write_chars(t, name, -1, NULL, error);
+	if (status != G_IO_STATUS_NORMAL)
+		return status;
 
 	if (value) {
-		status = g_io_channel_write_chars(t, ": ", -1, NULL, &error);
-		g_assert(status == G_IO_STATUS_NORMAL);
+		status = g_io_channel_write_chars(t, ": ", -1, NULL, error);
+		if (status != G_IO_STATUS_NORMAL)
+			return status;
 
-		status = g_io_channel_write_chars(t, value, -1, NULL, &error);
-		g_assert(status == G_IO_STATUS_NORMAL);
+		status = g_io_channel_write_chars(t, value, -1, NULL, error);
+		if (status != G_IO_STATUS_NORMAL)
+			return status;
 	}
 
-	status = g_io_channel_write_chars(t, "\n", -1, NULL, &error);
-	g_assert(status == G_IO_STATUS_NORMAL);
+	status = g_io_channel_write_chars(t, "\n", -1, NULL, error);
+	if (status != G_IO_STATUS_NORMAL)
+		return status;
 
-	return TRUE;
+	return G_IO_STATUS_NORMAL;
 }
 
 static gboolean marshall_get (GIOChannel *t, int level, const char *name, char **value)
@@ -79,7 +88,7 @@ static gboolean marshall_get (GIOChannel *t, int level, const char *name, char *
 	gsize term;
 
 	status = g_io_channel_read_line(t, &line, NULL, &term, &error);
-	g_assert(status == G_IO_STATUS_NORMAL);
+	LF_CHECK_IO_STATUS(status);
 
 	line[term] = '\0';
 
@@ -92,6 +101,8 @@ static gboolean marshall_get (GIOChannel *t, int level, const char *name, char *
 		*value = NULL;
 	}
 
+	g_free(line);
+
 	return TRUE;
 }
 
@@ -99,7 +110,10 @@ static gboolean marshall_struct(GIOChannel *t, enum marshall_mode m, int level,
 {
 	g_assert(name);
 	if (m == MARSHALL_PUSH) {
-		return marshall_set(t, level, name, NULL);
+		GIOStatus status;
+		GError *error = NULL;
+		status = marshall_set(t, level, name, NULL, &error);
+		return (status == G_IO_STATUS_NORMAL);
 	} else {
 		char *v;
 		gboolean ret = marshall_get(t, level, name, &v);
@@ -109,23 +123,30 @@ static gboolean marshall_struct(GIOChannel *t, enum marshall_mode m, int level,
 	}
 }
 
-static gboolean marshall_string (struct network_state *nst, 
+static gboolean marshall_string (struct irc_network_state *nst, 
 								 const char *name, int level, 
 								 enum marshall_mode m, GIOChannel *t, char **d)
 {
-	if (m == MARSHALL_PUSH)
-		return marshall_set(t, level, name, *d);
-	else
+	if (m == MARSHALL_PUSH) {
+		GIOStatus status;
+		GError *error = NULL;
+		status = marshall_set(t, level, name, *d, &error);
+		return (status == G_IO_STATUS_NORMAL);
+	} else {
 		return marshall_get(t, level, name, d);
+	}
 }
 
-static gboolean marshall_bool(struct network_state *nst, const char *name, 
+static gboolean marshall_bool(struct irc_network_state *nst, const char *name, 
 							  int level, enum marshall_mode m, GIOChannel *t, 
 							  gboolean *n)
 {
 	if (m == MARSHALL_PUSH) {
-		if (*n) return marshall_set(t, level, name, "true");
-		else return marshall_set(t, level, name, "false");
+		GError *error = NULL;
+		GIOStatus status;
+		if (*n) status = marshall_set(t, level, name, "true", &error);
+		else status = marshall_set(t, level, name, "false", &error);
+		return (status == G_IO_STATUS_NORMAL);
 	} else {
 		gboolean ret;
 		char *val;
@@ -139,12 +160,15 @@ static gboolean marshall_bool(struct network_state *nst, const char *name,
 	}
 }
 
-static gboolean marshall_char(struct network_state *nst, const char *name, int level, enum marshall_mode m, GIOChannel *t, char *n)
+static gboolean marshall_char(struct irc_network_state *nst, const char *name, int level, enum marshall_mode m, GIOChannel *t, char *n)
 {
 	if (m == MARSHALL_PUSH) {
 		char d[2] = {*n, '\0' };
+		GError *error = NULL;
+		GIOStatus status;
 		/* FIXME: Escape \n ? */
-		return marshall_set(t, level, name, d);
+		status = marshall_set(t, level, name, d, &error);
+		return (status == G_IO_STATUS_NORMAL);
 	} else {
 		char *d;
 		gboolean ret = marshall_get(t, level, name, &d);
@@ -158,12 +182,15 @@ static gboolean marshall_char(struct network_state *nst, const char *name, int l
 	}
 }
 
-static gboolean marshall_long(struct network_state *nst, const char *name, int level, enum marshall_mode m, GIOChannel *t, long *n)
+static gboolean marshall_long(struct irc_network_state *nst, const char *name, int level, enum marshall_mode m, GIOChannel *t, long *n)
 {
 	if (m == MARSHALL_PUSH) {
 		char tmp[10];
+		GError *error = NULL;
+		GIOStatus status;
 		g_snprintf(tmp, sizeof(tmp), "%ld", *n);
-		return marshall_set(t, level, name, tmp);
+		status = marshall_set(t, level, name, tmp, &error);
+		return (status == G_IO_STATUS_NORMAL);
 	} else {
 		char *tmp;
 		gboolean ret = marshall_get(t, level, name, &tmp);
@@ -177,12 +204,15 @@ static gboolean marshall_long(struct network_state *nst, const char *name, int l
 	}
 }
 
-static gboolean marshall_int(struct network_state *nst, const char *name, int level, enum marshall_mode m, GIOChannel *t, int *n)
+static gboolean marshall_int(struct irc_network_state *nst, const char *name, int level, enum marshall_mode m, GIOChannel *t, int *n)
 {
 	if (m == MARSHALL_PUSH) {
 		char tmp[10];
+		GError *error = NULL;
+		GIOStatus status;
 		g_snprintf(tmp, sizeof(tmp), "%d", *n);
-		return marshall_set(t, level, name, tmp);
+		status = marshall_set(t, level, name, tmp, &error);
+		return (status == G_IO_STATUS_NORMAL);
 	} else {
 		char *tmp;
 		gboolean ret = marshall_get(t, level, name, &tmp);
@@ -196,18 +226,21 @@ static gboolean marshall_int(struct network_state *nst, const char *name, int le
 	}
 }
 
-static gboolean marshall_time(struct network_state *nst, const char *name, int level, enum marshall_mode m, GIOChannel *t, time_t *n)
+static gboolean marshall_time(struct irc_network_state *nst, const char *name, int level, enum marshall_mode m, GIOChannel *t, time_t *n)
 {
 	if (m == MARSHALL_PUSH) {
-		char tmp[10];
+		char tmp[30];
+		GError *error = NULL;
+		GIOStatus status;
 		g_snprintf(tmp, sizeof(tmp), "%lu", *n);
-		return marshall_set(t, level, name, tmp);
+		status = marshall_set(t, level, name, tmp, &error);
+		return (status == G_IO_STATUS_NORMAL);
 	} else {
 		char *tmp;
 		gboolean ret = marshall_get(t, level, name, &tmp);
 		if (!ret) return FALSE;
 
-		*n = atol(tmp);
+		*n = strtoul(tmp, NULL, 10);
 
 		g_free(tmp);
 
@@ -216,13 +249,14 @@ static gboolean marshall_time(struct network_state *nst, const char *name, int l
 }
 
 
-static gboolean marshall_modes(struct network_state *nst, const char *name, int level, enum marshall_mode m, GIOChannel *t, char n[255])
+static gboolean marshall_modes(struct irc_network_state *nst, const char *name, int level, enum marshall_mode m, GIOChannel *t, irc_modes_t n)
 {
 	if (m == MARSHALL_PUSH) {
+		GError *error = NULL;
 		char *tmp = mode2string(n);
-		gboolean ret = marshall_set(t, level, name, tmp);
+		GIOStatus status = marshall_set(t, level, name, tmp, &error);
 		g_free(tmp);
-		return ret;
+		return status == G_IO_STATUS_NORMAL;
 	} else {
 		char *tmp;
 		gboolean ret = marshall_get(t, level, name, &tmp);
@@ -233,7 +267,7 @@ static gboolean marshall_modes(struct network_state *nst, const char *name, int
 	}
 }
 
-static gboolean marshall_network_nick (struct network_state *nst, const char *name, int level, enum marshall_mode m, GIOChannel *t, struct network_nick *n)
+static gboolean marshall_network_nick (struct irc_network_state *nst, const char *name, int level, enum marshall_mode m, GIOChannel *t, struct network_nick *n)
 {
 	gboolean ret = TRUE;
 	marshall_struct(t, m, level, name);
@@ -252,7 +286,7 @@ static gboolean marshall_network_nick (struct network_state *nst, const char *na
 }
 
 
-static gboolean marshall_network_nick_p (struct network_state *nst, const char *name, int level, enum marshall_mode m, GIOChannel *t, struct network_nick **n)
+static gboolean marshall_network_nick_p (struct irc_network_state *nst, const char *name, int level, enum marshall_mode m, GIOChannel *t, struct network_nick **n)
 {
 	marshall_new(m, n);
 	return marshall_network_nick(nst, name, level, m, t, *n);
@@ -263,10 +297,10 @@ struct ht_traverse_data
 	marshall_fn_t key_fn;
 	marshall_fn_t val_fn;
 	struct data_blob *data;
-	struct network_state *nst;
+	struct irc_network_state *nst;
 };
 
-static gboolean marshall_GList (struct network_state *nst, const char *name, int level, enum marshall_mode m, GIOChannel *t, GList **gl, marshall_fn_t marshall_fn)
+static gboolean marshall_GList (struct irc_network_state *nst, const char *name, int level, enum marshall_mode m, GIOChannel *t, GList **gl, marshall_fn_t marshall_fn)
 {
 	marshall_struct(t, m, level, name);
 	if (m == MARSHALL_PULL) {
@@ -304,7 +338,7 @@ static gboolean marshall_GList (struct network_state *nst, const char *name, int
 	return TRUE;
 }
 
-static gboolean marshall_banlist_entry (struct network_state *nst, const char *name, int level, enum marshall_mode m, GIOChannel *t, struct banlist_entry **d)
+static gboolean marshall_banlist_entry (struct irc_network_state *nst, const char *name, int level, enum marshall_mode m, GIOChannel *t, struct banlist_entry **d)
 {
 	gboolean ret = TRUE;
 	marshall_new(m, d);
@@ -315,10 +349,10 @@ static gboolean marshall_banlist_entry (struct network_state *nst, const char *n
 	return ret;
 }
 
-static gboolean marshall_channel_state (struct network_state *nst, 
+static gboolean marshall_channel_state (struct irc_network_state *nst, 
 										const char *name, int level, 
 										enum marshall_mode m, GIOChannel *t, 
-										struct channel_state **c)
+										struct irc_channel_state **c)
 {
 	gboolean ret = TRUE;
 	marshall_new(m, c);
@@ -366,7 +400,7 @@ static gboolean marshall_channel_state (struct network_state *nst,
 			cn = find_add_channel_nick(*c, nick);
 			g_assert(cn);
 
-			if (!marshall_char(nst, "mode", level+3, m, t, &cn->mode))
+			if (!marshall_modes(nst, "modes", level+3, m, t, cn->modes))
 				return FALSE;
 
 			g_free(nick);
@@ -388,7 +422,7 @@ static gboolean marshall_channel_state (struct network_state *nst,
 			if (!marshall_string(nst, "nick", level+3, m, t, &cn->global_nick->nick))
 				return FALSE;
 
-			if (!marshall_char(nst, "mode", level+3, m, t, &cn->mode))
+			if (!marshall_modes(nst, "modes", level+3, m, t, cn->modes))
 				return FALSE;
 
 			i++;
@@ -399,10 +433,14 @@ static gboolean marshall_channel_state (struct network_state *nst,
 }
 
 static gboolean marshall_network_state(enum marshall_mode m, GIOChannel *t, 
-									   struct network_state *n)
+									   struct irc_network_state *n)
 {
 	gboolean ret = TRUE;
 
+	if (m == MARSHALL_PULL) {
+		g_free(n->me.hostmask);
+		n->me.hostmask = NULL;
+	}
 	ret &= marshall_network_nick(n, "me", 0, m, t, &n->me);
 	ret &= marshall_GList(n, "nicks", 0, m, t, &n->nicks, (marshall_fn_t)marshall_network_nick_p);
 	ret &= marshall_GList(n, "channels", 0, m, t, &n->channels, (marshall_fn_t)marshall_channel_state);
@@ -419,9 +457,10 @@ static gboolean marshall_network_state(enum marshall_mode m, GIOChannel *t,
  */
 struct lf_data {
 	GIOChannel *line_file;
+	char *state_dir;
 	GIOChannel *state_file;
 	struct {
-		gint64 state_offset;
+		char *state_id;
 		gint64 line_offset;
 	} *state_dumps;
 	size_t num_state_dumps;
@@ -429,16 +468,24 @@ struct lf_data {
 	int lines_since_last_state;
 };
 
-static void file_insert_state(struct linestack_context *ctx, 
-							  const struct network_state *state);
+static char *state_path(struct lf_data *lf_data, const char *state_id)
+{
+	return g_build_filename(lf_data->state_dir, state_id, NULL);
+}
+
+static gboolean file_insert_state(struct linestack_context *ctx, 
+							  const struct irc_network_state *state, 
+							  const char *state_id);
 
 static gboolean file_init(struct linestack_context *ctx, const char *name, 
 						  struct ctrlproxy_config *config, 
-						  const struct network_state *state)
+						  const struct irc_network_state *state)
 {
 	struct lf_data *data = g_new0(struct lf_data, 1);
 	char *parent_dir, *data_dir, *data_file;
 	GError *error = NULL;
+	const char *fname;
+	GDir *dir;
 
 	parent_dir = g_build_filename(config->config_dir, "linestack_file", NULL);
 	g_mkdir(parent_dir, 0700);
@@ -460,32 +507,42 @@ static gboolean file_init(struct linestack_context *ctx, const char *name,
 
 	g_io_channel_set_encoding(data->line_file, NULL, NULL);
 
+	/* Remove old data file */
 	data_file = g_build_filename(data_dir, "state", NULL);
-
 	unlink(data_file);
-
-	data->state_file = g_io_channel_new_file(data_file, "w+", &error);
-	if (data->state_file == NULL) {
-		log_global(LOG_WARNING, "Error opening `%s': %s", 
-						  data_file, error->message);
-		g_free(data_file);
-		return FALSE;
-	}
 	g_free(data_file);
 
-	g_io_channel_set_encoding(data->state_file, NULL, NULL);
-
+	data->state_dir = g_build_filename(data_dir, "states", NULL);
 	g_free(data_dir);
+	g_mkdir(data->state_dir, 0755);
+
+	dir = g_dir_open(data->state_dir, 0, &error);
+	if (dir == NULL) {
+		log_global(LOG_WARNING, "Error opening directory `%s': %s", 
+						  data->state_dir, error->message);
+		return FALSE;
+	}
+
+	while ((fname = g_dir_read_name(dir))) {
+		char *state_file_path = state_path(data, fname);
+		g_unlink(state_file_path);
+		g_free(state_file_path);
+	}
+
+	g_dir_close(dir);
+
 	ctx->backend_data = data;
-	file_insert_state(ctx, state);
-	return TRUE;
+	return file_insert_state(ctx, state, "START");
 }
 
 static gboolean file_fini(struct linestack_context *ctx)
 {
 	struct lf_data *data = ctx->backend_data;
+	int i;
 	g_io_channel_unref(data->line_file);
-	g_io_channel_unref(data->state_file);
+	g_free(data->state_dir);
+	for (i = 0; i < data->num_state_dumps; i++)
+		g_free(data->state_dumps[i].state_id);
 	g_free(data->state_dumps);
 	g_free(data);
 	return TRUE;
@@ -498,14 +555,30 @@ static gint64 g_io_channel_tell_position(GIOChannel *gio)
 	return lseek(fd, 0, SEEK_CUR);
 }
 
-static void file_insert_state(struct linestack_context *ctx, 
-							  const struct network_state *state)
+static gboolean file_insert_state(struct linestack_context *ctx, 
+							  const struct irc_network_state *state,
+							  const char *state_id)
 {
 	struct lf_data *nd = ctx->backend_data;
 	GError *error = NULL;
 	GIOStatus status;
+	GIOChannel *state_file;
+	char *data_file;
 
 	log_global(LOG_TRACE, "Inserting state");
+
+	data_file = state_path(nd, state_id);
+
+	state_file = g_io_channel_new_file(data_file, "w+", &error);
+	if (state_file == NULL) {
+		log_global(LOG_WARNING, "Error opening `%s': %s", 
+						  data_file, error->message);
+		g_free(data_file);
+		return FALSE;
+	}
+	g_free(data_file);
+
+	g_io_channel_set_encoding(state_file, NULL, NULL);
 	
 	nd->lines_since_last_state = 0;
 
@@ -514,43 +587,55 @@ static void file_insert_state(struct linestack_context *ctx,
 		nd->state_dumps = g_realloc(nd->state_dumps, nd->num_state_dumps_alloc * sizeof(*nd->state_dumps));
 	}
 
-
 	nd->state_dumps[nd->num_state_dumps].line_offset = g_io_channel_tell_position(nd->line_file);
-	nd->state_dumps[nd->num_state_dumps].state_offset = g_io_channel_tell_position(nd->state_file);
+	nd->state_dumps[nd->num_state_dumps].state_id = g_strdup(state_id);
 
 	nd->num_state_dumps++;
 
-	marshall_network_state(MARSHALL_PUSH, nd->state_file, (struct network_state *)state);
+	marshall_network_state(MARSHALL_PUSH, state_file, (struct irc_network_state *)state);
 
-	status = g_io_channel_flush(nd->state_file, &error);
-	g_assert(status);
+	status = g_io_channel_flush(state_file, &error);
+	LF_CHECK_IO_STATUS(status);
+
+	g_io_channel_unref(state_file);
+
+	return TRUE;
 }
 
 static gboolean file_insert_line(struct linestack_context *ctx, 
-								 const struct line *l, 
-								 const struct network_state *state)
+								 const struct irc_line *l, 
+								 const struct irc_network_state *state)
 {
 	struct lf_data *nd = ctx->backend_data;
 	char t[20];
 	GError *error = NULL;
 	GIOStatus status;
+	gboolean ret;
 	
 	if (nd == NULL) 
 		return FALSE;
 
 	nd->lines_since_last_state++;
-	if (nd->lines_since_last_state == STATE_DUMP_INTERVAL) 
-		file_insert_state(ctx, state);
+	if (nd->lines_since_last_state == STATE_DUMP_INTERVAL) {
+		char *state_id = g_strdup_printf("%lu-%lld", time(NULL), 
+								 g_io_channel_tell_position(nd->line_file));
+		ret = file_insert_state(ctx, state, state_id);
+		g_free(state_id);
+		if (ret == FALSE)
+			return FALSE;
+	}
 
 	g_snprintf(t, sizeof(t), "%ld ", time(NULL));
 
 	status = g_io_channel_write_chars(nd->line_file, t, strlen(t), NULL, &error);
-	g_assert(status == G_IO_STATUS_NORMAL);
+	LF_CHECK_IO_STATUS(status);
 
-	status = irc_send_line(nd->line_file, (GIConv)-1, l, NULL);
+	status = irc_send_line(nd->line_file, (GIConv)-1, l, &error);
 
-	if (status != G_IO_STATUS_NORMAL)
+	if (status != G_IO_STATUS_NORMAL) {
+		log_global(LOG_ERROR, "Unable to write to linestack file: %s", error->message);
 		return FALSE;
+	}
 
 	return TRUE;
 }
@@ -565,30 +650,32 @@ static void *file_get_marker(struct linestack_context *ctx)
 	return pos;
 }
 
-static struct network_state *file_get_state (struct linestack_context *ctx, 
+static struct irc_network_state *file_get_state (struct linestack_context *ctx, 
 											  void *m)
 {
 	struct lf_data *nd = ctx->backend_data;
-	struct network_state *ret;
+	struct irc_network_state *ret;
 	gint64 *to_offset = m, t;
 	struct linestack_marker m1, m2;
 	GError *error = NULL;
 	GIOStatus status;
+	char *data_file;
+	GIOChannel *state_file;
 	int i;
 
-	if (!nd) 
+	if (nd == NULL) 
 		return NULL;
 
 	/* Flush channel before reading otherwise data corruption may occur */
 	status = g_io_channel_flush(nd->line_file, &error);
-	g_assert(status == G_IO_STATUS_NORMAL);
+	LF_CHECK_IO_STATUS(status);
 
 	if (to_offset != NULL) 
 		t = *to_offset;
 	else 
 		t = g_io_channel_tell_position(nd->line_file);
 
-	/* Search back from end of the state file to begin 
+	/* Search back fromend of the state file to begin 
 	 * and find the state dump with the highest offset but an offset 
 	 * below from_offset */
 	for (i = nd->num_state_dumps-1; i >= 0; i--) {
@@ -596,27 +683,30 @@ static struct network_state *file_get_state (struct linestack_context *ctx,
 			break;
 	}
 
-	status = g_io_channel_seek_position(nd->state_file, 
-										nd->state_dumps[i].state_offset, 
-										G_SEEK_SET, &error);
+	data_file = state_path(nd, nd->state_dumps[i].state_id);
 
-	g_assert(status == G_IO_STATUS_NORMAL);
+	state_file = g_io_channel_new_file(data_file, "r", &error);
+	if (state_file == NULL) {
+		log_global(LOG_WARNING, "Error opening `%s': %s", 
+						  data_file, error->message);
+		g_free(data_file);
+		return FALSE;
+	}
+	g_free(data_file);
+
+	g_io_channel_set_encoding(state_file, NULL, NULL);
 
 	ret = network_state_init("", "", "");
-	if (!marshall_network_state(MARSHALL_PULL, nd->state_file, ret))
+	if (!marshall_network_state(MARSHALL_PULL, state_file, ret))
 		return NULL;
 
-	status = g_io_channel_seek_position(nd->state_file, 0, G_SEEK_END, &error);
-	g_assert(status == G_IO_STATUS_NORMAL);
-
 	m1.free_fn = NULL;
 	m1.data = &nd->state_dumps[i].line_offset;
 	m2.free_fn = NULL;
 	m2.data = to_offset;
 	linestack_replay(ctx, &m1, &m2, ret);
 
-	status = g_io_channel_seek_position(nd->line_file, 0, G_SEEK_END, &error);
-	g_assert(status == G_IO_STATUS_NORMAL);
+	g_io_channel_unref(state_file);
 	
 	return ret;
 }
@@ -628,9 +718,9 @@ static gboolean file_traverse(struct linestack_context *ctx, void *mf,
 	gboolean ret = TRUE;
 	struct lf_data *nd = ctx->backend_data;
 	GError *error = NULL;
-	GIOStatus status;
+	GIOStatus status = G_IO_STATUS_NORMAL;
 	char *raw, *space;
-	struct line *l;
+	struct irc_line *l;
 
 	if (nd == NULL) 
 		return FALSE;
@@ -645,18 +735,22 @@ static gboolean file_traverse(struct linestack_context *ctx, void *mf,
 	status = g_io_channel_seek_position(nd->line_file, 
 			(start_offset != NULL)?(*start_offset):0, G_SEEK_SET, &error);
 
-	g_assert (status == G_IO_STATUS_NORMAL);
+	LF_CHECK_IO_STATUS(status);
+
+	raw = NULL;
 	
-	while((status = g_io_channel_read_line(nd->line_file, &raw, NULL, 
-		                          NULL, &error) != G_IO_STATUS_EOF) && 
-		(!end_offset || 
+	while((status != G_IO_STATUS_EOF) && 
+		(end_offset == NULL || 
 		 g_io_channel_tell_position(nd->line_file) <= (*end_offset))) {
-		if (status != G_IO_STATUS_NORMAL) {
-			log_global(LOG_WARNING, "read_line() failed: %s",
-					error->message);
+		status = g_io_channel_read_line(nd->line_file, &raw, NULL, NULL, &error);
+		if (status == G_IO_STATUS_ERROR) {
+			log_global(LOG_WARNING, "read_line() failed: %s", error->message);
 			return FALSE;
 		}
 
+		if (raw == NULL)
+			break;
+
 		space = strchr(raw, ' ');
 		g_assert(space);
 		*space = '\0';
@@ -668,15 +762,15 @@ static gboolean file_traverse(struct linestack_context *ctx, void *mf,
 		free_line(l);
 
 		g_free(raw);
+		raw = NULL;
 
 		if (ret == FALSE) 
 			break;
 	}
 
-	status = g_io_channel_seek_position(nd->line_file, 0, G_SEEK_END, 
-	                                    &error);
+	status = g_io_channel_seek_position(nd->line_file, 0, G_SEEK_END, &error);
 
-	g_assert(status == G_IO_STATUS_NORMAL);
+	LF_CHECK_IO_STATUS(status);
 
 	return ret;
 }
