@@ -31,7 +31,7 @@ gboolean no_log_timestamp = FALSE;
 enum log_level current_log_level = LOG_INFO;
 FILE *flog = NULL;
 
-static void log_entry(enum log_level level, const struct network *n, const struct client *c, const char *data)
+void log_entry(enum log_level level, const struct irc_network *n, const struct irc_client *c, const char *data)
 {
 	if (flog == NULL)
 		return;
@@ -49,7 +49,7 @@ static void log_entry(enum log_level level, const struct network *n, const struc
 	fprintf(flog, "%s", data);
 
 	if (n) {
-		fprintf(flog, " (%s", n->info.name);
+		fprintf(flog, " (%s", n->info->name);
 
 		if (c)
 			fprintf(flog, "/%s", c->description);
@@ -61,7 +61,7 @@ static void log_entry(enum log_level level, const struct network *n, const struc
 	fflush(flog);
 }
 
-void log_network_line(const struct network *n, const struct line *l, gboolean incoming)
+void log_network_line(const struct irc_network *n, const struct irc_line *l, gboolean incoming)
 {
 	char *raw;
 	if (current_log_level < LOG_DATA)
@@ -72,18 +72,18 @@ void log_network_line(const struct network *n, const struct line *l, gboolean in
 	g_free(raw);
 }
 
-void log_client_line(const struct client *n, const struct line *l, gboolean incoming)
+void log_client_line(const struct irc_client *n, const struct irc_line *l, gboolean incoming)
 {
 	char *raw;
 	if (current_log_level < LOG_DATA)
 		return;
 
 	raw = irc_line_string(l);
-	log_client(LOG_DATA, n, "%c %s", incoming?'<':'>', raw);
+	client_log(LOG_DATA, n, "%c %s", incoming?'<':'>', raw);
 	g_free(raw);
 }
 
-void handle_network_log(enum log_level level, const struct network *n, 
+void handle_network_log(enum log_level level, const struct irc_network *n, 
 						const char *msg)
 {
 	g_assert(n != NULL);
@@ -93,16 +93,9 @@ void handle_network_log(enum log_level level, const struct network *n,
 						     msg, NULL);
 }
 
-void log_client(enum log_level level, const struct client *c, const char *fmt, ...)
+void log_client(enum log_level level, const struct irc_client *c, const char *data)
 {
-	va_list ap;	
-	char *tmp; 
-	g_assert(c != NULL);
-	va_start(ap, fmt);
-	tmp = g_strdup_vprintf(fmt, ap);
-	va_end(ap);
-	log_entry(level, c->network, c, tmp);
-	g_free(tmp);
+	log_entry(level, c->network, c, data);
 }
 
 void log_global(enum log_level level, const char *fmt, ...)
