@@ -1,6 +1,6 @@
 /*
 	ctrlproxy: A modular IRC proxy
-	(c) 2002-2007 Jelmer Vernooij <jelmer@nl.linux.org>
+	(c) 2002-2008 Jelmer Vernooij <jelmer@nl.linux.org>
 
 	This program is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -43,11 +43,35 @@ extern char my_hostname[];
 extern struct global *my_global;
 extern help_t *help; 
 
+struct hup_handler {
+	hup_handler_fn fn;
+	void *userdata;
+};
+static GList *hup_handlers = NULL;
+void register_hup_handler(hup_handler_fn fn, void *userdata)
+{
+	struct hup_handler *hh = g_new0(struct hup_handler, 1);
+	hh->fn = fn;
+	hh->userdata = userdata;
+	hup_handlers = g_list_append(hup_handlers, hh);
+}
+
 static void signal_hup(int sig)
 {
+	GList *gl;
 	char *logfile;
 	logfile = g_build_filename(my_global->config->config_dir, "log", NULL);
 	init_log(logfile);
+	for (gl = hup_handlers; gl; gl = gl->next) {
+		struct hup_handler *hh = gl->data;
+		
+		hh->fn(hh->userdata);
+	}
+}
+
+char *pid_file(struct global *global)
+{
+	return g_build_filename(global->config->config_dir, "pid", NULL);
 }
 
 static void signal_crash(int sig) 
@@ -76,7 +100,7 @@ static void signal_crash(int sig)
 	}
 
 #endif
-	log_global(LOG_ERROR, "Please send a bug report to jelmer@vernstok.nl.");
+	log_global(LOG_ERROR, "Please send a bug report to jelmer@samba.org.");
 	log_global(LOG_ERROR, "A gdb backtrace is appreciated if you can reproduce this bug.");
 	abort();
 }
@@ -92,7 +116,8 @@ static void clean_exit()
 
 	if (!g_file_test(path, G_FILE_TEST_IS_DIR)) {
 		if (g_mkdir(path, 0700) != 0) {
-			log_global(LOG_ERROR, "Can't create config directory '%s': %s", path, strerror(errno));
+			log_global(LOG_ERROR, "Can't create config directory '%s': %s", 
+					   path, strerror(errno));
 			return;
 		}
 	}
@@ -129,7 +154,8 @@ static void signal_save(int sig)
 	
 	if (!g_file_test(my_global->config->config_dir, G_FILE_TEST_IS_DIR)) {
 		if (g_mkdir(my_global->config->config_dir, 0700) != 0) {
-			log_global(LOG_ERROR, "Can't create config directory '%s': %s", my_global->config->config_dir, strerror(errno));
+			log_global(LOG_ERROR, "Can't create config directory '%s': %s", 
+					   my_global->config->config_dir, strerror(errno));
 			return;
 		}
 	}
@@ -140,35 +166,6 @@ static void signal_save(int sig)
 	nickserv_save(my_global, my_global->config->config_dir);
 }
 
-static pid_t read_pidfile(struct global *global)
-{
-	char *path = g_build_filename(global->config->config_dir, "pid", NULL);
-	char *contents;
-	pid_t pid;
-	GError *error = NULL;
-	if (!g_file_get_contents(path, &contents, NULL, &error)) 
-		return -1;
-	g_free(path);
-	pid = atol(contents);
-	/* FIXME: Check if pid is still running */
-	g_free(contents);
-	return pid;
-}
-
-static gboolean write_pidfile(struct global *global)
-{
-	char *path = g_build_filename(global->config->config_dir, "pid", NULL);
-	GError *error = NULL;
-	char contents[100];
-	snprintf(contents, 100, "%u", getpid());
-	if (!g_file_set_contents(path, contents, -1, &error)) {
-		log_global(LOG_ERROR, "Unable to write pid file `%s'", path);
-		return FALSE;
-	}
-	g_free(path);
-	return TRUE;
-}
-
 int main(int argc, char **argv)
 {
 	int isdaemon = 0;
@@ -179,20 +176,32 @@ int main(int argc, char **argv)
 	char *tmp;
 	gboolean init = FALSE;
 	const char *inetd_client = NULL;
+	char *pidfile;
+	gboolean check_running = FALSE;
 	gboolean version = FALSE;
+	pid_t pid;
 	GOptionContext *pc;
 	GOptionEntry options[] = {
-		{"inetd-client", 'i', 0, G_OPTION_ARG_STRING, &inetd_client, "Communicate with client to NETWORK via stdio", "NETWORK" },
-		{"debug-level", 'd', 'd', G_OPTION_ARG_INT, &current_log_level, ("Debug level [0-5]"), "LEVEL" },
-		{"no-timestamp", 'n', 0, G_OPTION_ARG_NONE, &no_log_timestamp, "No timestamps in logs" },
-		{"daemon", 'D', 0, G_OPTION_ARG_NONE, &isdaemon, ("Run in the background (as a daemon)")},
+		{"check-running", 0, 0, G_OPTION_ARG_NONE, &check_running, 
+			"Only check whether ctrlproxy is running and exit"},
+		{"inetd-client", 'i', 0, G_OPTION_ARG_STRING, &inetd_client, 
+			"Communicate with client to NETWORK via stdio", "NETWORK" },
+		{"debug-level", 'd', 'd', G_OPTION_ARG_INT, &current_log_level, 
+			"Debug level [0-5]", "LEVEL" },
+		{"no-timestamp", 'n', 0, G_OPTION_ARG_NONE, &no_log_timestamp, 
+			"No timestamps in logs" },
+		{"daemon", 'D', 0, G_OPTION_ARG_NONE, &isdaemon, 
+			"Run in the background (as a daemon)"},
 		{"init", 0, 0, G_OPTION_ARG_NONE, &init, "Create configuration" },
-		{"log", 'l', 0, G_OPTION_ARG_STRING, &logfile, ("Log messages to specified file"), ("FILE")},
-		{"config-dir", 'c', 0, G_OPTION_ARG_STRING, &config_dir, ("Override configuration directory"), ("DIR")},
-		{"version", 'v', 0, G_OPTION_ARG_NONE, &version, ("Show version information")},
+		{"log", 'l', 0, G_OPTION_ARG_STRING, &logfile, 
+			"Log messages to specified file", "FILE"},
+		{"config-dir", 'c', 0, G_OPTION_ARG_STRING, &config_dir, 
+			"Override configuration directory", "DIR"},
+		{"version", 'v', 0, G_OPTION_ARG_NONE, &version, 
+			"Show version information"},
 		{ NULL }
 	};
-	GError *error;
+	GError *error = NULL;
 
 	signal(SIGINT, signal_quit);
 	signal(SIGTERM, signal_quit);
@@ -221,7 +230,7 @@ int main(int argc, char **argv)
 
 	if (version) {
 		printf("ctrlproxy %s\n", VERSION);
-		printf("(c) 2002-2007 Jelmer Vernooij et al. <jelmer@nl.linux.org>\n");
+		printf("(c) 2002-2008 Jelmer Vernooij et al. <jelmer@nl.linux.org>\n");
 		return 0;
 	}
 
@@ -244,14 +253,73 @@ int main(int argc, char **argv)
 
 	init_log(logfile);
 
-	log_global(LOG_INFO, "CtrlProxy %s starting", VERSION);
-
 	if (gethostname(my_hostname, NI_MAXHOST) != 0) {
 		log_global(LOG_WARNING, "Can't figure out hostname of local host!");
 		g_free(config_dir);
 		return 1;
 	}
 
+	/* Determine correct modules directory */
+	init_plugins(getenv("CTRLPROXY_MODULESDIR")?getenv("CTRLPROXY_MODULESDIR"):MODULESDIR);
+
+	init_admin();
+	init_nickserv();
+	init_replication();
+	help = help_load_file(HELPFILE);
+
+	tmp = g_build_filename(config_dir, "config", NULL);
+
+	if (!g_file_test(tmp, G_FILE_TEST_EXISTS)) {
+		char *rcfile;
+
+		rcfile = g_build_filename(g_get_home_dir(), ".ctrlproxyrc", NULL);
+		
+		if (g_file_test(rcfile, G_FILE_TEST_EXISTS)) {
+			log_global(LOG_INFO, "Pre-3.0 style .ctrlproxyrc found");
+			log_global(LOG_INFO, 
+					   "Run ctrlproxy-upgrade to update configuration");
+			g_free(config_dir);
+			g_free(rcfile);
+			return 1;
+		} else {
+			log_global(LOG_INFO, "No configuration found. Maybe you would like to create one by running with --init ?");
+			g_free(config_dir);
+			g_free(rcfile);
+			return 1;
+		}
+
+		g_free(rcfile);
+	} else {
+		my_global = load_global(config_dir);	
+	}
+	g_free(config_dir);
+	g_free(tmp);
+
+	if (my_global == NULL) {
+		log_global(LOG_ERROR, "Unable to load configuration, exiting...");
+		return 1;
+	}
+
+	pidfile = pid_file(my_global);
+	pid = read_pidfile(pidfile);
+
+	if (check_running) {
+		if (pid == -1) {
+			printf("ctrlproxy is not running\n");
+			return -1;
+		} else {
+			printf("ctrlproxy pid is %d\n", pid);
+			return 0;
+		}
+	}
+
+	if (pid != -1) {
+		log_global(LOG_ERROR, 
+				   "ctrlproxy is already running at pid %d (from %s)", 
+				   pid, pidfile);
+		return 1;
+	}
+
 	if (isdaemon) {
 #ifdef HAVE_DAEMON 
 #ifdef SIGTTOU
@@ -278,74 +346,41 @@ int main(int argc, char **argv)
 #endif
 	} 
 
-	init_admin();
-	init_nickserv();
-	init_replication();
-	help = help_load_file(HELPFILE);
+	log_global(LOG_INFO, "CtrlProxy %s (pid %d) starting", VERSION, getpid());
 
-	/* Determine correct modules directory */
-	init_plugins(getenv("CTRLPROXY_MODULESDIR")?getenv("CTRLPROXY_MODULESDIR"):MODULESDIR);
-
-	tmp = g_build_filename(config_dir, "config", NULL);
-
-	if (!g_file_test(tmp, G_FILE_TEST_EXISTS)) {
-		char *rcfile;
-
-		rcfile = g_build_filename(g_get_home_dir(), ".ctrlproxyrc", NULL);
-		
-		if (g_file_test(rcfile, G_FILE_TEST_EXISTS)) {
-			log_global(LOG_INFO, "Pre-3.0 style .ctrlproxyrc found");
-			log_global(LOG_INFO, "Run ctrlproxy-upgrade to update configuration");
-			g_free(config_dir);
-			g_free(rcfile);
-			return 1;
-		} else {
-			log_global(LOG_INFO, "No configuration found. Maybe you would like to create one by running with --init ?");
-			g_free(config_dir);
-			g_free(rcfile);
-			return 1;
-		}
-
-		g_free(rcfile);
-	} else {
-		my_global = load_global(config_dir);	
-	}
-	g_free(config_dir);
-	g_free(tmp);
-
-	if (my_global == NULL) {
-		log_global(LOG_ERROR, "Unable to load configuration, exiting...");
-		return 1;
-	}
-
-	write_pidfile(my_global);
+	write_pidfile(pidfile);
+	g_free(pidfile);
 
 	start_unix_socket(my_global);
 	start_admin_socket(my_global);
-	autoconnect_networks(my_global);
-	init_listeners(my_global);
-	if (my_global->config->auto_away != NULL)
-		auto_away_add(my_global, my_global->config->auto_away);
+	autoconnect_networks(my_global->networks);
+	if (!init_listeners(my_global)) {
+		log_global(LOG_ERROR, 
+				   "Failed to start one or more listeners, exiting...");
+		return 1;
+	}
+	if (my_global->config->auto_away.enabled)
+		auto_away_add(my_global, &my_global->config->auto_away);
 	g_option_context_free(pc);
 
 	atexit(clean_exit);
 
 	if (inetd_client) {
 		GIOChannel *io = g_io_channel_unix_new(0);
-		struct network *n = find_network(my_global, inetd_client);
+		struct irc_network *n = find_network(my_global->networks, inetd_client);
 
 		if (!n) {
-			fprintf(stderr, "Unable to find network named '%s'\n", inetd_client);
+			fprintf(stderr, "Unable to find network named '%s'\n", 
+					inetd_client);
 			return 1;
 		} else {
 			/* Find clients network by name */
-			struct client *client = client_init(n, io, "Standard I/O");
+			struct irc_client *client = client_init_iochannel(n, io, "Standard I/O");
 			client->exit_on_close = TRUE;
 		}
 	}
 
 	g_main_loop_run(main_loop);
 
-
 	return 0;
 }
