@@ -23,7 +23,7 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <errno.h>
-#include "nickserv.h"
+#include "keyfile.h"
 #include "irc.h"
 
 #define NICKSERV_FILE_HEADER "# This file contains passwords for NickServ\n" \
@@ -32,28 +32,28 @@
 	"# a nick name, password and network name, separated by tabs.\n" \
 	"#\n"
 
-const char *nickserv_find_nick(struct network *n, const char *nick)
+const char *nickserv_find_nick(struct irc_network *n, const char *nick)
 {
 	GList *gl;
 	for (gl = n->global->nickserv_nicks; gl; gl = gl->next) {
-		struct nickserv_entry *e = gl->data;
+		struct keyfile_entry *e = gl->data;
 
 		if (g_strcasecmp(e->nick, nick)) 
 			continue;
 
 		if (!e->network) return e->pass;
-		if (!g_strcasecmp(e->network, n->info.name)) return e->pass;
+		if (!g_strcasecmp(e->network, n->info->name)) return e->pass;
 	}
 	
 	return NULL;
 }
 
-const char *nickserv_nick(struct network *n)
+const char *nickserv_nick(struct irc_network *n)
 {
 	return "NickServ";
 }
 
-void nickserv_identify_me(struct network *network, char *nick)
+void nickserv_identify_me(struct irc_network *network, char *nick)
 {
 	const char *pass;
 
@@ -77,15 +77,18 @@ void nickserv_identify_me(struct network *network, char *nick)
 	}
 }
 
-static void cache_nickserv_pass(struct network *n, const char *newpass)
+static void cache_nickserv_pass(struct irc_network *n, const char *newpass)
 {
-	struct nickserv_entry *e = NULL;
+	struct keyfile_entry *e = NULL;
 	GList *gl;
 
+	if (!n->global->config->learn_nickserv)
+		return;
+
 	for (gl = n->global->nickserv_nicks; gl; gl = gl->next) {
 		e = gl->data;
 
-		if (e->network && !g_strcasecmp(e->network, n->info.name) && 
+		if (e->network && !g_strcasecmp(e->network, n->info->name) && 
 			!g_strcasecmp(e->nick, n->state->me.nick)) {
 			break;		
 		}
@@ -96,21 +99,20 @@ static void cache_nickserv_pass(struct network *n, const char *newpass)
 		}
 	}
 
-	if (!gl) {
-		e = g_new0(struct nickserv_entry, 1);
+	if (gl == NULL) {
+		e = g_new0(struct keyfile_entry, 1);
 		e->nick = g_strdup(n->state->me.nick);
-		e->network = g_strdup(n->info.name);
+		e->network = g_strdup(n->info->name);
 		n->global->nickserv_nicks = g_list_prepend(n->global->nickserv_nicks, e);
 	}
 
-	if ((e->pass == NULL || 
-		strcmp(e->pass, newpass) != 0) && n->global->config->learn_nickserv) {
+	if (e->pass == NULL || strcmp(e->pass, newpass) != 0) {
 		e->pass = g_strdup(newpass);
 		network_log(LOG_INFO, n, "Caching password for nick %s", e->nick);
 	} 
 }
 
-static gboolean log_data(struct network *n, const struct line *l, enum data_direction dir, void *userdata) 
+static gboolean log_data(struct irc_network *n, const struct irc_line *l, enum data_direction dir, void *userdata) 
 {
 	static char *nickattempt = NULL;
 
@@ -157,117 +159,34 @@ static gboolean log_data(struct network *n, const struct line *l, enum data_dire
 	return TRUE;
 }
 
-gboolean nickserv_write_file(GList *nicks, const char *filename)
-{
-	GList *gl;
-	int fd;
-	gboolean empty = TRUE;
-
-	fd = open(filename, O_WRONLY | O_CREAT, 0600);
-
-    if (fd == -1) {
-		log_global(LOG_WARNING, "Unable to write nickserv file `%s': %s", filename, strerror(errno));
-        return FALSE;
-    }
-
-	if (write(fd, NICKSERV_FILE_HEADER, strlen(NICKSERV_FILE_HEADER)) < 0) {
-		log_global(LOG_WARNING, "error writing file header: %s", strerror(errno));
-		close(fd);
-		return FALSE;
-	}
-
-	for (gl = nicks; gl; gl = gl->next) {
-		struct nickserv_entry *n = gl->data;
-        char *line;
-
-		empty = FALSE;
-        
-        line = g_strdup_printf("%s\t%s\t%s\n", n->nick, n->pass, n->network?n->network:"*");
-		if (write(fd, line, strlen(line)) < 0) {
-			log_global(LOG_WARNING, "error writing line `%s': %s", line, strerror(errno));
-			g_free(line);
-			close(fd);
-			return FALSE;
-		}
-
-        g_free(line);
-	}
-    
-    close(fd);
-
-	return TRUE;
-}
-
 gboolean nickserv_save(struct global *global, const char *dir)
 {
     char *filename = g_build_filename(dir, "nickserv", NULL);
 	gboolean ret;
 
-	ret = nickserv_write_file(global->nickserv_nicks, filename);
+	if (global->nickserv_nicks == NULL) {
+		if (unlink(filename) == 0)
+			ret = TRUE;
+		else if (errno == ENOENT)
+			ret = TRUE;
+		else
+			ret = FALSE;
+	} else 
+		ret = keyfile_write_file(global->nickserv_nicks, NICKSERV_FILE_HEADER, filename);
 
     g_free(filename);
 
 	return ret;
 }
 
-gboolean nickserv_read_file(const char *filename, GList **nicks)
-{
-    GIOChannel *gio;
-    char *ret;
-    gsize nr, term;
-
-    gio = g_io_channel_new_file(filename, "r", NULL);
-
-    if (gio == NULL) {
-        return FALSE;
-    }
-
-    while (G_IO_STATUS_NORMAL == g_io_channel_read_line(gio, &ret, &nr, &term, NULL))
-    {
-        char **parts; 
-		struct nickserv_entry *e;
-
-		if (ret[0] == '#') {
-        	g_free(ret);
-			continue;
-		}
-
-		ret[term] = '\0';
-
-        parts = g_strsplit(ret, "\t", 3);
-        g_free(ret);
-
-		if (!parts[0] || !parts[1]) {
-			g_strfreev(parts);
-			continue;
-		}
-			
-		e = g_new0(struct nickserv_entry, 1);
-		e->nick = parts[0];
-		e->pass = parts[1];
-		if (!parts[2] || !strcmp(parts[2], "*")) {
-			e->network = NULL;
-			g_free(parts[2]);
-		} else {
-			e->network = parts[2];
-		}
-	
-		*nicks = g_list_append(*nicks, e);   
-        g_free(parts);
-    }
-
-	g_io_channel_shutdown(gio, TRUE, NULL);
-	g_io_channel_unref(gio);
-
-	return TRUE;
-}
-
 gboolean nickserv_load(struct global *global)
 {
 	gboolean ret;
-    char *filename = g_build_filename(global->config->config_dir, "nickserv", 
+    char *filename;
+	
+	filename = g_build_filename(global->config->config_dir, "nickserv", 
 									  NULL);
-	ret = nickserv_read_file(filename, &global->nickserv_nicks);
+	ret = keyfile_read_file(filename, '#', &global->nickserv_nicks);
 	g_free(filename);
 
 	return TRUE;
