@@ -18,7 +18,7 @@
 	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
-#include "ctrlproxy.h"
+#include "internals.h"
 #include <string.h>
 #include "irc.h"
 
@@ -26,31 +26,34 @@
 
 struct query_stack {
 	const struct query *query;
-	const struct network *network;
-	struct client *client;	
+	struct irc_client *client;	
 	time_t time;
 	struct query_stack *next;
 };
 
-static struct query_stack *stack = NULL;
-
 /**
  * IRC Query done by a client
  */
 struct query {
+	/** Command name. */
 	char *name;
+
+	/** Possible replies */
 	int replies[20];
+	/** Possible replies that are the last reply to this command. */
 	int end_replies[20];
+	/** Possible errors. */
 	int errors[20];
 	/* Should add this query to the stack. return TRUE if this has 
 	 * been done successfully, FALSE otherwise */
-	int (*handle) (const struct line *, const struct network *n, struct client *c, struct query *);
+	/** Function for handling the responses. */
+	int (*handle) (const struct irc_line *, struct query_stack **, struct irc_client *c, struct query *);
 };
 
-static int handle_default(const struct line *, const struct network *n, 
-						  struct client *c, struct query *);
-static int handle_topic(const struct line *, const struct network *n, 
-						struct client *c, struct query *);
+static int handle_default(const struct irc_line *, struct query_stack **stack, 
+						  struct irc_client *c, struct query *);
+static int handle_topic(const struct irc_line *, struct query_stack **stack,
+						struct irc_client *c, struct query *);
 
 static struct query queries[] = {
 /* Commands that get a one-client reply: 
@@ -59,9 +62,10 @@ static struct query queries[] = {
 		{ RPL_WHOISUSER, RPL_WHOISCHANNELS, RPL_AWAY,
 		  RPL_WHOISIDLE, RPL_WHOISCHANNELS, RPL_WHOISIP,
 		  RPL_WHOISSERVER, RPL_WHOISOPERATOR, RPL_WHOISACTUALLY,
-		  RPL_WHOISSSL, RPL_WHOISACCOUNT, RPL_WHOISIDENTIFIED, 0 }, 
+		  RPL_WHOISSSL, RPL_WHOISACCOUNT, RPL_WHOISIDENTIFIED, 
+		  RPL_WHOISLOGGEDINAS, RPL_WHOISOPERPRIVS, ERR_NOSUCHNICK, 0 }, 
 		{ RPL_ENDOFWHOIS, RPL_TRYAGAIN, 0 }, 
-		{ ERR_NOSUCHSERVER, ERR_NONICKNAMEGIVEN, ERR_NOSUCHNICK, 0 },
+		{ ERR_NOSUCHSERVER, ERR_NONICKNAMEGIVEN, 0 },
 		handle_default
 	},
 
@@ -100,15 +104,15 @@ static struct query queries[] = {
 	
  /* WHOWAS <nickname> [<count> [<server>]]*/
 	{"WHOWAS", 
-		{ RPL_WHOWASUSER, RPL_WHOISSERVER, RPL_WHOWAS_TIME, 0 },
+		{ RPL_WHOWASUSER, RPL_WHOISSERVER, RPL_WHOWAS_TIME, ERR_WASNOSUCHNICK, 0 },
 		{ RPL_ENDOFWHOWAS, RPL_TRYAGAIN, 0 },
-		{ ERR_NONICKNAMEGIVEN, ERR_WASNOSUCHNICK, 0 },
+		{ ERR_NONICKNAMEGIVEN, 0 },
 		handle_default
 	},
 	
  /* STATS [<query> [<server>]]*/
 	{"STATS", 
-		{ RPL_STATSCLINE, RPL_STATSILINE, RPL_STATSQLINE, 
+		{ RPL_STATSCLINE, RPL_STATSILINE, RPL_STATSQLINE, RPL_STATSPLINE,
 		  RPL_STATSLINKINFO, RPL_STATSCOMMANDS, RPL_STATSHLINE, RPL_STATSNLINE,
 		  RPL_STATSKLINE, RPL_STATSLLINE, RPL_STATSUPTIME, RPL_STATSOLINE, 
 		  RPL_STATSTLINE,
@@ -313,6 +317,20 @@ static struct query queries[] = {
 			ERR_NOSUCHNICK, ERR_NOSUCHCHANNEL, ERR_BLOCKING_NOTID, 0 },
 		handle_default
 	},
+
+	{ "NICKSERV",
+		{ 0 },
+		{ RPL_TRYAGAIN, 0 },
+		{ ERR_NOTEXTTOSEND, 0 },
+		handle_default
+	},
+
+	{ "CHANSERV",
+		{ 0 },
+		{ RPL_TRYAGAIN, 0 },
+		{ ERR_NOTEXTTOSEND, 0 },
+		handle_default
+	},
  
  /* MOTD [<target>] */
 	{ "MOTD",
@@ -468,31 +486,31 @@ static struct query unknown_query = {
 	handle_default
 };
 
-static gboolean handle_465(struct network *n, struct line *l)
+static gboolean handle_465(struct irc_network *n, const struct irc_line *l)
 {
 	network_log(LOG_ERROR, n, "Banned from server: %s", l->args[1]);
 	return TRUE;
 }
 
-static gboolean handle_451(struct network *n, struct line *l)
+static gboolean handle_451(struct irc_network *n, const struct irc_line *l)
 {
 	network_log(LOG_ERROR, n, "Not registered error, this is probably a bug...");
 	return TRUE;
 }
 
-static gboolean handle_462(struct network *n, struct line *l)
+static gboolean handle_462(struct irc_network *n, const struct irc_line *l)
 {
 	network_log(LOG_ERROR, n, "Double registration error, this is probably a bug...");
 	return TRUE;
 }
 
-static gboolean handle_463(struct network *n, struct line *l)
+static gboolean handle_463(struct irc_network *n, const struct irc_line *l)
 {
 	network_log(LOG_ERROR, n, "Host not privileged to connect");
 	return TRUE;
 }
 
-static gboolean handle_464(struct network *n, struct line *l)
+static gboolean handle_464(struct irc_network *n, const struct irc_line *l)
 {
 	network_log(LOG_ERROR, n, "Password mismatch");
 	return TRUE;
@@ -501,7 +519,7 @@ static gboolean handle_464(struct network *n, struct line *l)
 /* List of responses that should be sent to all clients */
 static int response_all[] = { RPL_NOWAWAY, RPL_UNAWAY, 
 	ERR_NO_OP_SPLIT, RPL_HIDINGHOST,
-	ERR_NEEDREGGEDNICK, RPL_UMODEIS, 
+	ERR_NEEDREGGEDNICK, RPL_UMODEIS, RPL_SNOMASK,
 	RPL_LUSERCLIENT, RPL_LUSEROP, RPL_LUSERUNKNOWN, RPL_LUSERCHANNELS,
 	RPL_LUSERME, ERR_NO_OP_SPLIT, RPL_LOCALUSERS, RPL_GLOBALUSERS, 
 	RPL_NAMREPLY, RPL_ENDOFNAMES, RPL_TOPIC, RPL_TOPICWHOTIME, 
@@ -510,7 +528,7 @@ static int response_none[] = { ERR_NOMOTD, RPL_MOTDSTART, RPL_MOTD,
 	RPL_ENDOFMOTD, 0 };
 static struct {
 	int response;
-	gboolean (*handler) (struct network *n, struct line *);
+	gboolean (*handler) (struct irc_network *n, const struct irc_line *);
 } response_handler[] = {
 	{ ERR_PASSWDMISMATCH, handle_464 },
 	{ ERR_ALREADYREGISTERED, handle_462 },
@@ -522,6 +540,9 @@ static struct {
 
 /**
  * Check whether reply r is part of a specified list
+ *
+ * @param replies List of possible replies.
+ * @param r Response code.
  */
 static int is_reply(const int *replies, int r)
 {
@@ -550,24 +571,24 @@ static struct query *find_query(char *name)
  * @return TRUE if the message was redirected to zero or more clients, 
  *         FALSE if it was sent to all clients.
  */
-gboolean redirect_response(struct network *network, struct line *l)
+gboolean redirect_response(struct query_stack **stack, 
+						   struct irc_network *network,
+						   const struct irc_line *l)
 {
 	struct query_stack *s, *p = NULL;
-	const struct client *c = NULL;
+	const struct irc_client *c = NULL;
 	int n;
 	int i;
 	
-	g_assert(network);
 	g_assert(l->args[0]);
 
 	n = atoi(l->args[0]);
 
 	/* Find a request that this response is a reply to */
-	for (s = stack; s; s = s->next) {
-		if (s->network == network && 
-		   (is_reply(s->query->replies, n) || 
+	for (s = *stack; s; s = s->next) {
+		if (is_reply(s->query->replies, n) || 
 			is_reply(s->query->errors, n) ||
-			is_reply(s->query->end_replies, n))) {
+			is_reply(s->query->end_replies, n)) {
 			
 			/* Send to client that queried, if that client still exists */
 			if (s->client != NULL) {
@@ -577,7 +598,7 @@ gboolean redirect_response(struct network *network, struct line *l)
 
 			if (!is_reply(s->query->replies, n)) {
 				/* Remove from stack */
-				if (p == NULL)stack = s->next;	
+				if (p == NULL)*stack = s->next;	
 				else p->next = s->next;
 				client_unref(s->client);
 				g_free(s);
@@ -611,39 +632,32 @@ gboolean redirect_response(struct network *network, struct line *l)
 	}
 
 	if (!c) {
-		network_log(LOG_WARNING, network, "Unable to redirect response %s", l->args[0]);
+		network_log((g_list_length(network->clients) == 1)?LOG_TRACE:LOG_WARNING, 
+					network, "Unable to redirect response %s", l->args[0]);
 		clients_send(network->clients, l, NULL);
 	}
 
 	return FALSE;
 }
 
-void redirect_clear(const struct network *net)
+void redirect_clear(struct query_stack **stack)
 {
-	struct query_stack *q, *p = NULL, *n;
+	struct query_stack *q, *n;
 
-	g_assert(net);
-
-	q = stack;
+	q = *stack;
 	while (q != NULL) {
-		if (q->network != net) {
-			p = q;
-			q = q->next;
-			continue;
-		}
-
 		/* Remove from stack */
-		if (p == NULL)stack = q->next;	
-		else p->next = q->next;
 		n = q->next;
 		client_unref(q->client);
 		g_free(q);
 		q = n;
 	}
+	*stack = NULL;	
 }
 
-void redirect_record(const struct network *n, struct client *c, 
-					 const struct line *l)
+void redirect_record(struct query_stack **stack,
+					 const struct irc_network *n, struct irc_client *c, 
+					 const struct irc_line *l)
 {
 	struct query *q;
 
@@ -654,7 +668,7 @@ void redirect_record(const struct network *n, struct client *c,
 	q = find_query(l->args[0]);
 	if (q == NULL) {
 		if (c != NULL) {
-			log_client(LOG_WARNING, c, "Unknown command from client: %s", 
+			client_log(LOG_WARNING, c, "Unknown command from client: %s", 
 					   l->args[0]);
 		} else {
 			network_log(LOG_WARNING, n, "Sending unknown command '%s'", 
@@ -665,28 +679,26 @@ void redirect_record(const struct network *n, struct client *c,
 	}
 
 	/* Push it up the stack! */
-	q->handle(l, n, c, q);
+	q->handle(l, stack, c, q);
 }
 
-static int handle_default(const struct line *l, const struct network *n, 
-						  struct client *c, struct query *q)
+static int handle_default(const struct irc_line *l, struct query_stack **stack,
+						  struct irc_client *c, struct query *q)
 {
 	struct query_stack *s = g_new(struct query_stack, 1);
 	g_assert(l != NULL);
-	g_assert(n != NULL);
 	g_assert(q != NULL);
-	s->network = n;
 	s->client = client_ref(c);
 	s->time = time(NULL);
 	s->query = q;
-	s->next = stack;
-	stack = s;
+	s->next = *stack;
+	*stack = s;
 	return 1;
 }
 
-static int handle_topic(const struct line *l, const struct network *n, struct client *c, struct query *q)
+static int handle_topic(const struct irc_line *l, struct query_stack **stack, struct irc_client *c, struct query *q)
 {
 	if (l->args[2] != NULL)
 		return 0;
-	return handle_default(l,n,c,q);
+	return handle_default(l,stack,c,q);
 }
