@@ -29,8 +29,10 @@
  * A replication backend
  */
 struct replication_backend {
+	/** Name of the backend, as can be specified in replication=. */
 	const char *name;
-	void (*replication_fn) (struct client *);
+	/** Replication function. */
+	void (*replication_fn) (struct irc_client *);
 };
 
 void register_replication_backend(const struct replication_backend *);
