@@ -20,6 +20,7 @@
 
 #include "internals.h"
 #include "ssl.h"
+#include "keyfile.h"
 
 #ifdef HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
@@ -32,8 +33,16 @@
 #define DEFAULT_ADMIN_PORT 6680
 #define DEFAULT_SOCKS_PORT 1080
 
+#define CHANNEL_KEY_FILE_HEADER \
+	"; This file contains channel keys.\n" \
+	"; It has the same format as the ppp secrets files.\n" \
+	"; It should contain one entry per line, each entry consisting of: \n" \
+	"; a channel name, channel key and network name, separated by tabs.\n" \
+	";\n\n"
+
 static GList *known_keys = NULL;
 
+static void config_cleanup_networks_dir(struct ctrlproxy_config *cfg);
 static void config_save_log(struct log_file_config *data,
 							struct ctrlproxy_config *config);
 static void config_save_auto_away(struct auto_away_config *d, 
@@ -47,10 +56,12 @@ static const char *builtin_known_keys[] = {
 	"auto-away-nick",
 	"auto-away-client-limit",
 	"auto-away-time",
+	"create-implicit",
 	"max_who_age",
 	"replication",
 	"linestack",
 	"report-time",
+	"report-time-offset",
 	"motd-file",
 	"default-client-charset",
 	"learn-nickserv",
@@ -61,6 +72,8 @@ static const char *builtin_known_keys[] = {
 	"listener-auto",
 	"listener-autoport",
 	"logging",
+	"logfile",
+	"logdir",
 	"port",
 	"bind",
 	"ssl",
@@ -131,34 +144,21 @@ static void config_save_tcp_servers(struct network_config *n, GKeyFile *kf)
 {
 	GList *gl;
 	int i = 0; 
-	gchar **values = g_new0(gchar *, g_list_length(n->type_settings.tcp_servers)+1);
+	gchar **values = g_new0(gchar *, g_list_length(n->type_settings.tcp.servers)+1);
 	
-	for (gl = n->type_settings.tcp_servers; gl; gl = gl->next) {
+	for (gl = n->type_settings.tcp.servers; gl; gl = gl->next) {
 		struct tcp_server_config *ts = gl->data;
-		char *name = g_strdup_printf("%s:%s", ts->host, ts->port);
+		char *name = irc_create_url(ts->host, ts->port, ts->ssl);
 
 		values[i] = name;
 
-		if (g_key_file_has_key(kf, name, "ssl", NULL) || ts->ssl)
-			g_key_file_set_boolean(kf, name, "ssl", ts->ssl);
-
 		if (ts->password)
 			g_key_file_set_string(kf, name, "password", ts->password);
 		else
 			g_key_file_remove_key(kf, name, "password", NULL);
 
 		if (ts->bind_address) {
-			char *tmp;
-			if (ts->bind_port) 
-				tmp = g_strdup_printf("%s:%s", 
-				                      ts->bind_address,
-						      ts->bind_port);
-			else
-				tmp = g_strdup(ts->bind_address);
-
-			g_key_file_set_string(kf, name, "bind", tmp);
-
-			g_free(tmp);
+			g_key_file_set_string(kf, name, "bind", ts->bind_address);
 		} else 
 			g_key_file_remove_key(kf, name, "bind", NULL);
 
@@ -170,11 +170,13 @@ static void config_save_tcp_servers(struct network_config *n, GKeyFile *kf)
 	g_strfreev(values);
 }
 
-static void config_save_network(const char *dir, struct network_config *n)
+static void config_save_network(const char *dir, struct network_config *n, GList **channel_keys)
 {
 	GList *gl;
 	GKeyFile *kf;
 	char *fn;
+	char **autojoin_list;
+	int autojoin_list_count;
 	
 	if (!n->keyfile) {
 		n->keyfile = g_key_file_new();
@@ -185,9 +187,17 @@ static void config_save_network(const char *dir, struct network_config *n)
 	g_key_file_set_string(kf, "global", "fullname", n->fullname);
 	g_key_file_set_string(kf, "global", "nick", n->nick);
 	g_key_file_set_string(kf, "global", "username", n->username);
+	if (n->autocmd) {
+		g_key_file_set_string_list(kf, "global", "autocmd", n->autocmd,
+							   g_strv_length(n->autocmd));
+	} else {
+		g_key_file_remove_key(kf, "global", "autocmd", NULL);
+	}
+
 	if (n->queue_speed)
 		g_key_file_set_integer(kf, "global", "queue-speed", n->queue_speed);
-	g_key_file_set_integer(kf, "global", "reconnect-interval", n->reconnect_interval);
+	if (n->reconnect_interval != -1)
+		g_key_file_set_integer(kf, "global", "reconnect-interval", n->reconnect_interval);
 
 	switch(n->type) {
 	case NETWORK_VIRTUAL:
@@ -198,21 +208,47 @@ static void config_save_network(const char *dir, struct network_config *n)
 		break;
 	case NETWORK_TCP:
 		config_save_tcp_servers(n, kf);
+		if (n->type_settings.tcp.default_bind_address != NULL)
+			g_key_file_set_string(kf, "global", "bind", 
+								  n->type_settings.tcp.default_bind_address);
+		else
+			g_key_file_remove_key(kf, "global", "bind", NULL);
 		break;
 	default:break;
 	}
-	
+
+	autojoin_list = g_new0(char *, g_list_length(n->channels));
+	autojoin_list_count = 0;
+
 	for (gl = n->channels; gl; gl = gl->next) {
 		struct channel_config *c = gl->data;
+		struct keyfile_entry *key;
 
-		if (c->key)
-			g_key_file_set_string(kf, c->name, "key", c->key);
-		else 
-			g_key_file_remove_key(kf, c->name, "key", NULL);
+		if (c->key) {
+			key = g_new0(struct keyfile_entry, 1);
+			key->network = n->name;
+			key->nick = c->name;
+			key->pass = c->key;
+
+			*channel_keys = g_list_append(*channel_keys, key);
+		}
 		
-		g_key_file_set_boolean(kf, c->name, "autojoin", c->autojoin);
+		if (c->autojoin) {
+			autojoin_list[autojoin_list_count] = c->name;
+			autojoin_list_count++;
+		}
+
+		g_key_file_remove_group(kf, c->name, NULL);
 	}
 
+	if (autojoin_list == NULL)
+		g_key_file_remove_key(kf, "global", "autojoin", NULL);
+	else
+		g_key_file_set_string_list(kf, "global", "autojoin", (const gchar **)autojoin_list, 
+							   autojoin_list_count);
+
+	g_free(autojoin_list);
+
 	fn = g_build_filename(dir, n->name, NULL);
 	g_key_file_save_to_file(kf, fn, NULL);
 	g_free(fn);
@@ -288,10 +324,11 @@ static void config_save_listeners(struct ctrlproxy_config *cfg, const char *path
 	g_free(filename);
 }
 
-static void config_save_networks(const char *config_dir, GList *networks)
+static void config_save_networks(struct ctrlproxy_config *cfg, const char *config_dir, GList *networks)
 {
 	char *networksdir = g_build_filename(config_dir, "networks", NULL);
 	GList *gl;
+	GList *channel_keys = NULL;
 
 	if (!g_file_test(networksdir, G_FILE_TEST_IS_DIR)) {
 		if (g_mkdir(networksdir, 0700) != 0) {
@@ -301,13 +338,36 @@ static void config_save_networks(const char *config_dir, GList *networks)
 	}
 
 	for (gl = networks; gl; gl = gl->next) {
-		struct network_config *n = gl->data;		
-		config_save_network(networksdir, n);
+		struct network_config *n = gl->data;
+		if (!n->implicit) 
+			config_save_network(networksdir, n, &channel_keys);
+	}
+
+	config_cleanup_networks_dir(cfg);
+
+	if (channel_keys != NULL) {
+		char *filename = g_build_filename(cfg->config_dir, "keys", 
+									  NULL);
+		keyfile_write_file(channel_keys, CHANNEL_KEY_FILE_HEADER, filename);
+		g_free(filename);
+
+		while (channel_keys) {
+			g_free(channel_keys->data);
+			channel_keys = channel_keys->next;
+		}
+
+		g_list_free(channel_keys);
 	}
 
 	g_free(networksdir);
 }
 
+/**
+ * Save configuration to a configuration directory.
+ *
+ * @param cfg The configuration to save.
+ * @param configuration_dir Directory to save to.
+ */
 void save_configuration(struct ctrlproxy_config *cfg, const char *configuration_dir)
 {
 	char *fn, **list;
@@ -344,6 +404,9 @@ void save_configuration(struct ctrlproxy_config *cfg, const char *configuration_
 		!cfg->learn_network_name)
 		g_key_file_set_boolean(cfg->keyfile, "global", "learn-network-name", cfg->learn_network_name);
 
+	if (g_key_file_has_key(cfg->keyfile, "global", "create-implicit", NULL) || !cfg->create_implicit)
+		g_key_file_set_boolean(cfg->keyfile, "global", "create-implicit", cfg->create_implicit);
+
 	if (cfg->client_charset != NULL)
 		g_key_file_set_string(cfg->keyfile, "global", "default-client-charset", cfg->client_charset);
 	
@@ -369,13 +432,17 @@ void save_configuration(struct ctrlproxy_config *cfg, const char *configuration_
 		break;
 	}
 
-	config_save_networks(configuration_dir, cfg->networks);
+	if (cfg->report_time_offset != 0 ||
+		g_key_file_has_key(cfg->keyfile, "global", "report-time-offset", NULL))
+		g_key_file_set_integer(cfg->keyfile, "global", "report-time-offset", cfg->report_time_offset);
+
+	config_save_networks(cfg, configuration_dir, cfg->networks);
 
 	config_save_listeners(cfg, configuration_dir);
 
 	config_save_log(cfg->log_file, cfg);
 
-	config_save_auto_away(cfg->auto_away, cfg);
+	config_save_auto_away(&cfg->auto_away, cfg);
 
 	i = 0;
 	list = g_new0(char *, g_list_length(cfg->networks)+1);
@@ -424,39 +491,51 @@ static void config_load_servers(struct network_config *n)
 		return;
 
 	for (i = 0; i < size; i++) {
-		char *tmp;
 		struct tcp_server_config *s = g_new0(struct tcp_server_config, 1);
+
+		irc_parse_url(servers[i], &s->host, &s->port, &s->ssl);
 		
 		s->password = g_key_file_get_string(n->keyfile, servers[i], "password", NULL);
 		if (g_key_file_has_key(n->keyfile, servers[i], "ssl", NULL))
 			s->ssl = g_key_file_get_boolean(n->keyfile, servers[i], "ssl", NULL);
 
-		tmp = strrchr(servers[i], ':');
-
-		if (tmp) {
-			*tmp = '\0';
-			tmp++;
-		}
-		
-		s->host = servers[i];
-		s->port = g_strdup(tmp != NULL?tmp:DEFAULT_IRC_PORT);
 		s->bind_address = g_key_file_get_string(n->keyfile, servers[i], "bind", NULL);
-		if (s->bind_address && (tmp = strchr(s->bind_address, ':'))) {
-			*tmp = '\0';
-			s->bind_port = tmp+1;
-		}
 
-		n->type_settings.tcp_servers = g_list_append(n->type_settings.tcp_servers, s);
+		n->type_settings.tcp.servers = g_list_append(n->type_settings.tcp.servers, s);
+
+		g_key_file_remove_group(n->keyfile, servers[i], NULL);
 	}
 
-	g_free(servers);
+	g_strfreev(servers);
 }
 
-static struct network_config *config_load_network(struct ctrlproxy_config *cfg, const char *dirname, const char *name)
+static struct channel_config *config_find_add_channel(struct network_config *nc, const char *name)
+{
+	GList *gl;
+	struct channel_config *cc;
+
+	for (gl = nc->channels; gl; gl = gl->next) { 
+		cc = gl->data;
+		if (!strcasecmp(cc->name, name)) 
+			return cc;
+
+	}
+
+	cc = g_new0(struct channel_config, 1);
+	cc->name = g_strdup(name);
+
+	nc->channels = g_list_append(nc->channels, cc);
+
+	return cc;
+}
+
+static struct network_config *config_load_network(struct ctrlproxy_config *cfg, const char *dirname, 
+												  const char *name, GList *channel_keys)
 {
 	GKeyFile *kf;
 	struct network_config *n;
 	char *filename;
+	GList *gl;
 	int i;
 	char **groups;
 	GError *error = NULL;
@@ -468,6 +547,7 @@ static struct network_config *config_load_network(struct ctrlproxy_config *cfg,
 
 	if (!g_key_file_load_from_file(kf, filename, G_KEY_FILE_KEEP_COMMENTS, &error)) {	
 		log_global(LOG_ERROR, "Can't parse configuration file '%s': %s", filename, error->message);
+		g_free(filename);
 		g_key_file_free(kf);
 		return NULL;
 	}	
@@ -515,6 +595,11 @@ static struct network_config *config_load_network(struct ctrlproxy_config *cfg,
 		n->password = g_key_file_get_string(kf, "global", "password", NULL);
 	}
 
+	if (g_key_file_has_key(kf, "global", "autocmd", NULL)) {
+		g_strfreev(n->autocmd);
+		n->autocmd = g_key_file_get_string_list(n->keyfile, "global", "autocmd", &size, NULL);
+	}
+
 	n->name = g_strdup(name);
 
 	if (g_key_file_has_key(kf, "global", "program", NULL)) 
@@ -526,6 +611,7 @@ static struct network_config *config_load_network(struct ctrlproxy_config *cfg,
 
 	switch (n->type) {
 	case NETWORK_TCP:
+		n->type_settings.tcp.default_bind_address = g_key_file_get_string(kf, "global", "bind", NULL);
 		config_load_servers(n);
 		break;
 	case NETWORK_PROGRAM:
@@ -547,6 +633,29 @@ static struct network_config *config_load_network(struct ctrlproxy_config *cfg,
 
 	g_strfreev(groups);
 
+	for (gl = channel_keys; gl; gl = gl->next) {
+		struct keyfile_entry *ke = gl->data;
+
+		if (!strcasecmp(ke->network, n->name)) {
+			struct channel_config *cc = config_find_add_channel(n, n->nick);
+
+			g_free(cc->key);
+			cc->key = g_strdup(n->password);
+		}
+	}
+
+	if (g_key_file_has_key(n->keyfile, "global", "autojoin", NULL)) {
+		char **autojoin_channels;
+		autojoin_channels = g_key_file_get_string_list(n->keyfile, "global", "autojoin", &size, NULL);
+		for (i = 0; i < size; i++) {
+			struct channel_config *cc = config_find_add_channel(n, autojoin_channels[i]);
+
+			cc->autojoin = TRUE;
+		}
+
+		g_strfreev(autojoin_channels);
+	}
+
 	return n;
 }
 
@@ -566,7 +675,7 @@ static struct network_config *find_create_network_config(struct ctrlproxy_config
 		if (nc->type != NETWORK_TCP) 
 			continue;
 
-		for (gl1 = nc->type_settings.tcp_servers; gl1; gl1 = gl1->next) {
+		for (gl1 = nc->type_settings.tcp.servers; gl1; gl1 = gl1->next) {
 			char *tmp;
 			struct tcp_server_config *sc = gl1->data;
 
@@ -576,7 +685,7 @@ static struct network_config *find_create_network_config(struct ctrlproxy_config
 			if (g_strncasecmp(sc->host, name, strlen(sc->host)) != 0)
 				continue;
 
-			tmp = g_strdup_printf("%s:%s", sc->host, sc->port);
+			tmp = irc_create_url(sc->host, sc->port, FALSE);
 
 			if (g_strcasecmp(tmp, name) == 0)
 				return nc;
@@ -588,18 +697,11 @@ static struct network_config *find_create_network_config(struct ctrlproxy_config
 	nc = network_config_init(cfg);
 	nc->name = g_strdup(name);
 	nc->autoconnect = FALSE;
-	nc->reconnect_interval = DEFAULT_RECONNECT_INTERVAL;
+	nc->reconnect_interval = -1;
 	nc->type = NETWORK_TCP;
 	tc = g_new0(struct tcp_server_config, 1);
-	tc->host = g_strdup(name);
-	if (strchr(tc->host, ':')) {
-		tc->port = tc->host+1;
-		*tc->port = '\0';
-	} else {
-		tc->port = g_strdup(DEFAULT_IRC_PORT);
-	}
-
-	nc->type_settings.tcp_servers = g_list_append(nc->type_settings.tcp_servers, tc);
+	irc_parse_url(name, &tc->host, &tc->port, &tc->ssl);
+	nc->type_settings.tcp.servers = g_list_append(nc->type_settings.tcp.servers, tc);
 
 	cfg->networks = g_list_append(cfg->networks, nc);
 
@@ -664,9 +766,9 @@ static void config_load_listeners(struct ctrlproxy_config *cfg)
 	GError *error = NULL;
 
 	if (g_key_file_has_key(cfg->keyfile, "listener", "pasword", NULL)) {
-		g_key_file_remove_key(cfg->keyfile, "listener", "password", NULL);
 		g_key_file_set_string(cfg->keyfile, "global", "password", 
 							  g_key_file_get_string(cfg->keyfile, "listener", "password", NULL));
+		g_key_file_remove_key(cfg->keyfile, "listener", "password", NULL);
 	}
 	default_password = g_key_file_get_string(cfg->keyfile, "global", "password", NULL);
 	if (g_key_file_has_key(cfg->keyfile, "global", "listener-auto", NULL)) {
@@ -750,20 +852,65 @@ static void config_load_listeners(struct ctrlproxy_config *cfg)
 	g_free(filename);
 }
 
-static void config_load_networks(struct ctrlproxy_config *cfg)
+
+struct network_config *config_find_network(struct ctrlproxy_config *cfg, 
+										   const char *name)
+{
+	GList *gl;
+	for (gl = cfg->networks; gl; gl = gl->next) {
+		struct network_config *nc = gl->data;
+		if (!g_strcasecmp(nc->name, name))
+			return nc;
+	}
+	return NULL;
+}
+
+void config_del_network(struct ctrlproxy_config *cfg, const char *name)
+{
+	cfg->networks = g_list_remove(cfg->networks, config_find_network(cfg, name));
+}
+
+#define IS_SPECIAL_FILE(name) (name[0] == '.' || name[strlen(name)-1] == '~')
+
+static void config_cleanup_networks_dir(struct ctrlproxy_config *cfg)
 {
 	char *networksdir = g_build_filename(cfg->config_dir, "networks", NULL);
 	GDir *dir;
 	const char *name;
 
 	dir = g_dir_open(networksdir, 0, NULL);
-	if (!dir)
+	if (dir == NULL)
 		return;
 
 	while ((name = g_dir_read_name(dir))) {
-		if (name[0] == '.' || name[strlen(name)-1] == '~')
+		char *path;
+		if (IS_SPECIAL_FILE(name) || config_find_network(cfg, name))
 			continue;
-		config_load_network(cfg, networksdir, name);
+
+		path = g_build_filename(networksdir, name, NULL);
+		g_unlink(path);
+		g_free(path);
+	}
+
+	g_free(networksdir);
+
+	g_dir_close(dir);
+}
+
+static void config_load_networks(struct ctrlproxy_config *cfg, GList *channel_keys)
+{
+	char *networksdir = g_build_filename(cfg->config_dir, "networks", NULL);
+	GDir *dir;
+	const char *name;
+
+	dir = g_dir_open(networksdir, 0, NULL);
+	if (dir == NULL)
+		return;
+
+	while ((name = g_dir_read_name(dir))) {
+		if (IS_SPECIAL_FILE(name))
+			continue;
+		config_load_network(cfg, networksdir, name, channel_keys);
 	}
 
 	g_free(networksdir);
@@ -784,10 +931,12 @@ static void config_save_log(struct log_file_config *data,
 
 	if (data->is_irssi) {
 		g_key_file_set_string(config->keyfile, "global", "logging", "irssi");
-		if (data->logbasedir)
-			g_key_file_set_string(config->keyfile, "global", "logdir", data->logbasedir);
-		else
+		if (data->logbasedir) {
+			if (!data->logbasedir_is_default)
+				g_key_file_set_string(config->keyfile, "global", "logdir", data->logbasedir);
+		} else {
 			g_key_file_set_string(config->keyfile, "global", "logfile", data->logfilename);
+		}
 	} else {
 		STORE_SETTING(data, config->keyfile, "global", "", logfilename);
 		STORE_SETTING(data, config->keyfile, "global", "log-format-", nickchange);
@@ -810,7 +959,7 @@ static void config_load_log(struct ctrlproxy_config *config)
 	struct log_file_config *data;
 	char *logging = NULL;
 
-	if (g_key_file_get_string(kf, "global", "logging", NULL) != NULL) {
+	if (g_key_file_has_key(kf, "global", "logging", NULL)) {
 		logging = g_key_file_get_string(kf, "global", "logging", NULL);
 	}
 
@@ -884,6 +1033,8 @@ static void config_load_log(struct ctrlproxy_config *config)
 			data->logbasedir = g_build_filename(config->config_dir, 
 										  "log_irssi", NULL);
 
+			data->logbasedir_is_default = TRUE;
+
 			data->logfilename = g_strdup_printf("%s/%%N/%%@", data->logbasedir);
 		}
 		g_key_file_remove_group(kf, "log-irssi", NULL);
@@ -905,12 +1056,10 @@ static void config_load_log(struct ctrlproxy_config *config)
 static void config_save_auto_away(struct auto_away_config *d, struct ctrlproxy_config *config)
 {
 	GKeyFile *kf = config->keyfile;
-
-	if (config->auto_away == NULL) {
-		g_key_file_set_boolean(kf, "global", "auto-away-enable", FALSE);
-		return;
-	}
-	g_key_file_set_boolean(kf, "global", "auto-away-enable", TRUE);
+	
+	if (g_key_file_has_key(kf, "global", "auto-away-enable", NULL) ||
+		d->enabled)
+		g_key_file_set_boolean(kf, "global", "auto-away-enable", d->enabled);
 
 	if (d->message != NULL)
 		g_key_file_set_string(kf, "global", "auto-away-message", d->message);
@@ -921,17 +1070,14 @@ static void config_save_auto_away(struct auto_away_config *d, struct ctrlproxy_c
 	if (d->client_limit != -1)
 		g_key_file_set_integer(kf, "global", "auto-away-client-limit", d->client_limit);
 
-	g_key_file_set_integer(kf, "global", "auto-away-time", d->max_idle_time);
+	if (d->max_idle_time != -1)
+		g_key_file_set_integer(kf, "global", "auto-away-time", d->max_idle_time);
 }
 
-static void config_load_auto_away(struct ctrlproxy_config *config)
+static void config_load_auto_away(struct auto_away_config *d, GKeyFile *kf)
 {
-	struct auto_away_config *d;
-	GKeyFile *kf = config->keyfile;
-
 	if (g_key_file_has_group(kf, "auto-away")) {
-		d = g_new0(struct auto_away_config, 1);
-		
+		d->enabled = TRUE;
 		d->message = g_key_file_get_string(kf, "auto-away", "message", NULL);
 		d->nick = g_key_file_get_string(kf, "auto-away", "nick", NULL);
 		if (g_key_file_has_key(kf, "auto-away", "client_limit", NULL)) {
@@ -948,29 +1094,23 @@ static void config_load_auto_away(struct ctrlproxy_config *config)
 		if (g_key_file_has_key(kf, "auto-away", "time", NULL))
 			d->max_idle_time = g_key_file_get_integer(kf, "auto-away", "time", NULL);
 		else
-			d->max_idle_time = AUTO_AWAY_DEFAULT_TIME;
+			d->max_idle_time = -1;
 
 		g_key_file_remove_group(kf, "auto-away", NULL);
-	} else if (g_key_file_has_key(kf, "global", "auto-away-enable", NULL) &&
-			   g_key_file_get_boolean(kf, "global", "auto-away-enable", NULL)) {
-		d = g_new0(struct auto_away_config, 1);
-		
+	} else {
+		if (g_key_file_has_key(kf, "global", "auto-away-enable", NULL))
+			d->enabled = g_key_file_get_boolean(kf, "global", "auto-away-enable", NULL);
 		d->message = g_key_file_get_string(kf, "global", "auto-away-message", NULL);
 		d->nick = g_key_file_get_string(kf, "global", "auto-away-nick", NULL);
 		if (g_key_file_has_key(kf, "global", "auto-away-client-limit", NULL)) {
 			d->client_limit = g_key_file_get_integer(kf, "global", "auto-away-client-limit", NULL);
-		}
-		else
+		} else
 			d->client_limit = -1;
 		if (g_key_file_has_key(kf, "global", "auto-away-time", NULL))
 			d->max_idle_time = g_key_file_get_integer(kf, "global", "auto-away-time", NULL);
 		else
-			d->max_idle_time = AUTO_AWAY_DEFAULT_TIME;
-	} else {
-		return;
+			d->max_idle_time = -1;
 	}
-
-	config->auto_away = d;
 }
 
 struct ctrlproxy_config *init_configuration(void)
@@ -992,6 +1132,8 @@ struct ctrlproxy_config *load_configuration(const char *dir)
 	GList *gl;
 	gsize size;
 	int i;
+	GList *channel_keys = NULL;
+	char *keyfile_filename;
 
 	file = g_build_filename(dir, "config", NULL);
 
@@ -1037,6 +1179,11 @@ struct ctrlproxy_config *load_configuration(const char *dir)
 		g_free(setting);
 	}
 
+	cfg->report_time_offset = 0;
+	if (g_key_file_has_key(kf, "global", "report-time-offset", NULL)) {
+		cfg->report_time_offset = g_key_file_get_integer(kf, "global", "report-time-offset", NULL);
+	}
+
     if (g_key_file_has_key(kf, "global", "motd-file", NULL))
 		cfg->motd_file = g_key_file_get_string(kf, "global", "motd-file", NULL);
     else 
@@ -1055,7 +1202,7 @@ struct ctrlproxy_config *load_configuration(const char *dir)
 	}
 
     if (g_key_file_has_key(kf, "global", "learn-nickserv", NULL))
-		cfg->learn_nickserv = g_key_file_get_boolean(kf, "global", "learn-nicksev", NULL);
+		cfg->learn_nickserv = g_key_file_get_boolean(kf, "global", "learn-nickserv", NULL);
     else 
 	    cfg->learn_nickserv = TRUE;
 
@@ -1064,6 +1211,11 @@ struct ctrlproxy_config *load_configuration(const char *dir)
     else 
 	    cfg->learn_network_name = TRUE;
 
+	if (g_key_file_has_key(kf, "global", "create-implicit", NULL))
+		cfg->create_implicit = g_key_file_get_boolean(kf, "global", "create-implicit", NULL);
+	else
+		cfg->create_implicit = TRUE;
+
 	if (!g_file_test(cfg->motd_file, G_FILE_TEST_EXISTS))
 		log_global(LOG_ERROR, "Can't open MOTD file '%s' for reading", cfg->motd_file);
 
@@ -1097,8 +1249,20 @@ struct ctrlproxy_config *load_configuration(const char *dir)
 	config_load_listeners(cfg);
 	config_load_listeners_socks(cfg);
 	config_load_log(cfg);
-	config_load_auto_away(cfg);
-	config_load_networks(cfg);
+	config_load_auto_away(&cfg->auto_away, cfg->keyfile);
+
+	keyfile_filename = g_build_filename(cfg->config_dir, "keys", 
+									  NULL);
+
+	if (g_file_test(keyfile_filename, G_FILE_TEST_EXISTS)) {
+		if (!keyfile_read_file(keyfile_filename, ';', &channel_keys)) {
+			log_global(LOG_WARNING, "Unable to read keys file");
+		}
+	}
+
+	g_free(keyfile_filename);
+
+	config_load_networks(cfg, channel_keys);
 
 	/* Check for unknown parameters */
 	keys = g_key_file_get_keys(kf, "global", NULL, NULL);
@@ -1139,7 +1303,7 @@ struct network_config *network_config_init(struct ctrlproxy_config *cfg)
 		g_free(s->fullname);
 		s->fullname = g_strdup(s->username);
 	}
-	s->reconnect_interval = DEFAULT_RECONNECT_INTERVAL;
+	s->reconnect_interval = -1;
 
 	if (cfg) 
 		cfg->networks = g_list_append(cfg->networks, s);
@@ -1155,6 +1319,7 @@ void free_config(struct ctrlproxy_config *cfg)
 		g_free(nc->fullname);
 		g_free(nc->username);
 		g_free(nc->password);
+		g_strfreev(nc->autocmd);
 		while (nc->channels) {
 			struct channel_config *cc = nc->channels->data;
 			g_free(cc->name);
@@ -1164,15 +1329,16 @@ void free_config(struct ctrlproxy_config *cfg)
 		}
 		switch (nc->type) {
 		case NETWORK_TCP: 
-			while (nc->type_settings.tcp_servers) {
-				struct tcp_server_config *tc = nc->type_settings.tcp_servers->data;
+			while (nc->type_settings.tcp.servers) {
+				struct tcp_server_config *tc = nc->type_settings.tcp.servers->data;
 				g_free(tc->host);
 				g_free(tc->port);
 				g_free(tc->bind_address);
 				g_free(tc->password);
-				nc->type_settings.tcp_servers = g_list_remove(nc->type_settings.tcp_servers, tc);
+				nc->type_settings.tcp.servers = g_list_remove(nc->type_settings.tcp.servers, tc);
 				g_free(tc);
 			}
+			g_free(nc->type_settings.tcp.default_bind_address);
 			break;
 		case NETWORK_VIRTUAL:
 			g_free(nc->type_settings.virtual_type);
@@ -1240,13 +1406,16 @@ gboolean create_configuration(const char *config_dir)
 	l = g_new0(struct listener_config, 1);
 	pass = getpass("Please specify a password for the administration interface: "); 
 	l->port = port;
+	l->is_default = TRUE;
 	if (!strcmp(pass, "")) {
 		fprintf(stderr, "Warning: no password specified. Authentication disabled!\n");
 	} else {
-		l->password = pass;
+		l->password = g_strdup(pass);
 	}
 
 	global->config->listeners = g_list_append(global->config->listeners, l);
 
+	save_configuration(global->config, config_dir);
+
 	return TRUE;
 }
