@@ -33,18 +33,26 @@
 struct channel_config {
 	char *name;
 	char *key;
-	int autojoin:1;
+	gboolean autojoin;
 };
 
 /**
  * TCP/IP server configuration
  */
 struct tcp_server_config {
+	/** IP Address or host. */
 	char *host;
+	
+	/** Port number or service name. */
 	char *port;
+
+	/** Adress to bind to (or NULL for default) */
 	char *bind_address;
-	char *bind_port;
-	int ssl:1;
+
+	/** Whether SSL should be used. */
+	gboolean ssl;
+
+	/** Password to send to remote host. */
 	char *password;
 };
 
@@ -54,17 +62,32 @@ struct tcp_server_config {
 struct network_config 
 {
 	GKeyFile *keyfile;
+	/** Network name */
 	char *name;
 	char *nick;
 	char *fullname;
 	char *username;
 	char *password;
+
+	/** Whether this network was created implicitly. */
+	int implicit:1;
+
+	/** Whether to automatically connect to this network on startup. */
 	int autoconnect:1;
+
+	/** Whether the first NICK sent by clients to this network should be ignored. */
 	int ignore_first_nick:1;
+
+	/** Disable reply caching for this network. */
 	int disable_cache:1;
-	int queue_speed; /* For flood protection */
+
+	/** For flood protection */
+	int queue_speed;
+
+	/** After how much seconds to attempt to reconnect. */
 	guint reconnect_interval;
 
+	/** Channels that should be joined. */
 	GList *channels;
 
 	enum { 
@@ -77,8 +100,14 @@ struct network_config
 	union {
 		char *virtual_type;
 		char *program_location;
-		GList *tcp_servers;
+		struct { 
+			char *default_bind_address;
+			GList *servers;
+		} tcp;
 	} type_settings; 
+
+	/** Commands to send on connect. */
+	char **autocmd;
 };
 
 /**
@@ -87,6 +116,7 @@ struct network_config
  * when logging.
  */
 struct log_file_config {
+	gboolean logbasedir_is_default;
 	const char *logbasedir;
 	const char *logfilename;
 	gboolean is_irssi;
@@ -101,7 +131,6 @@ struct log_file_config {
 	const char *mode;
 	const char *quit;
 	const char *notice;
-
 };
 
 /**
@@ -116,24 +145,25 @@ struct allow_rule {
  * Configuration for a single listener
  */
 struct listener_config {
+	/** Whether SSL is required */
 	gboolean ssl;
 	gpointer ssl_credentials;
+
+	/** Password */
 	char *password;
+
+	/** Address to bind to. Optional. */
 	char *address;
+
+	/** Port to listen on */
 	char *port;
+
+	/** Default network this listener is connected to. Can be NULL. */
 	char *network;
 	GList *allow_rules;
-	gboolean is_default; /* Whether this is the "default" listener, stored in ~/.ctrlproxy/config */
-};
-
-/**
- * Auto-away configuration.
- */
-struct auto_away_config {
-	time_t max_idle_time;
-	gint client_limit;
-	char *message;
-	char *nick;
+	
+	/** Whether this is the "default" listener, stored in ~/.ctrlproxy/config */
+	gboolean is_default;
 };
 
 /**
@@ -146,8 +176,16 @@ struct ctrlproxy_config {
 	char *motd_file;
 	char *network_socket;
 	char *admin_socket;
+
+	/** Whether to create networks that don't exist yet. */
+	gboolean create_implicit;
+
+	/** Name of replication backend, or NULL for default. */
 	char *replication;
+
+	/** Name of linestack backend, or NULL for default. */
 	char *linestack_backend;
+
 	char *client_charset;
 	gboolean admin_log;
 	char *admin_user;
@@ -156,6 +194,7 @@ struct ctrlproxy_config {
 		REPORT_TIME_NEVER,
 		REPORT_TIME_REPLICATION
 	} report_time;
+	int report_time_offset;
 	int max_who_age;
 	GKeyFile *keyfile;
 	GList *listeners;
@@ -163,7 +202,23 @@ struct ctrlproxy_config {
 	int listener_autoport;
 	gboolean learn_nickserv;
 	gboolean learn_network_name;
-	struct auto_away_config *auto_away;
+	/**
+	 * Auto-away configuration.
+	 */
+	struct auto_away_config {
+		/** Idle time after which to set /AWAY, in seconds. */
+		int max_idle_time;
+		gint client_limit;
+		
+		gboolean enabled;
+
+		/** Away message to set. */
+		char *message;
+
+		/** Nick to change to when auto away. */
+		char *nick;
+	} auto_away;
+
 	struct log_file_config *log_file;
 };
 
@@ -175,5 +230,6 @@ G_MODULE_EXPORT struct ctrlproxy_config *init_configuration(void);
 G_MODULE_EXPORT void free_config(struct ctrlproxy_config *);
 G_MODULE_EXPORT void setup_configdir(const char *dir);
 G_MODULE_EXPORT gboolean g_key_file_save_to_file(GKeyFile *kf, const gchar *file, GError **error);
+G_MODULE_EXPORT void config_del_network(struct ctrlproxy_config *cfg, const char *name);
 
 #endif /* __CTRLPROXY_SETTINGS_H__ */
