@@ -76,7 +76,7 @@ struct global *load_global(const char *config_dir)
 	global = init_global();
 	global->config = cfg;
 
-	load_networks(global, global->config, (network_log_fn)handle_network_log);
+	load_networks(global, global->config);
 
 	nickserv_load(global);
 
@@ -85,6 +85,12 @@ struct global *load_global(const char *config_dir)
 	return global;
 }
 
+void free_listeners(struct global *global)
+{
+	while (global->listeners)
+		free_listener((struct irc_listener *)global->listeners->data);
+}
+
 void free_global(struct global *global)
 {
 	if (global == NULL)
