@@ -26,87 +26,6 @@
 #include <fcntl.h>
 #include <glib/gstdio.h>
 
-char *list_make_string(GList *list)
-{
-	size_t len = 20;
-	char *ret;
-	GList *gl;
-
-	/* First, calculate the length */
-	for(gl = list; gl; gl = gl->next) len+=strlen(gl->data)+1;
-
-	ret = g_new(char,len);
-	ret[0] = '\0';
-
-	for(gl = list; gl; gl = gl->next) 
-	{ 
-		strncat(ret, gl->data, len);
-		if (gl->next) strncat(ret, " ", len); 
-	}
-
-	return ret;
-}
-
-static inline int str_cmphelper(const char *a, const char *b, char sh, char sl, char eh, char el)
-{
-	int i;
-	char h,l;
-	for (i = 0; a[i] && b[i]; i++) {
-		if (a[i] == b[i]) continue;
-		l = (a[i]>b[i]?b[i]:a[i]);
-		h = (a[i]>b[i]?a[i]:b[i]);
-
-		if (h < sh || h > eh || l < sl || l > el) 
-			break;
-
-		if (h-sh != l-sl)
-			break;
-	}
-
-	return a[i]-b[i];
-}
-
-int str_asciicmp(const char *a, const char *b)
-{
-	g_assert(a != NULL);
-	g_assert(b != NULL);
-	return str_cmphelper(a, b, 97, 65, 122, 90);
-}
-
-int str_strictrfc1459cmp(const char *a, const char *b)
-{
-	g_assert(a != NULL);
-	g_assert(b != NULL);
-	return str_cmphelper(a, b, 97, 65, 125, 93);
-}
-
-
-int str_rfc1459cmp(const char *a, const char *b)
-{
-	g_assert(a != NULL);
-	g_assert(b != NULL);
-	return str_cmphelper(a, b, 97, 65, 126, 94);
-}
-
-char *g_io_channel_ip_get_description(GIOChannel *ch)
-{
-	socklen_t len = sizeof(struct sockaddr_storage);
-	struct sockaddr *sa = g_malloc(len);
-	char hostname[NI_MAXHOST];
-	char service[NI_MAXSERV];
-	char *description = NULL;
-	int fd = g_io_channel_unix_get_fd(ch);
-
-	if (getpeername (fd, sa, &len) == 0 &&
-		getnameinfo(sa, len, hostname, sizeof(hostname),
-					service, sizeof(service), NI_NOFQDN | NI_NUMERICSERV) == 0) {
-
-		description = g_strdup_printf("%s:%s", hostname, service);
-	}
-
-	g_free(sa);
-	return description;
-}
 
 gboolean    rep_g_file_get_contents         (const gchar *filename,
                                              gchar **contents,
@@ -267,13 +186,36 @@ rep_g_mkdir_with_parents (const gchar *pathname,
   return 0;
 }
 
-const char *g_io_channel_unix_get_sock_error(GIOChannel *ioc)
+pid_t read_pidfile(const char *path)
 {
-	int valopt;
-	socklen_t valoptlen = sizeof(valopt);
-	int fd = g_io_channel_unix_get_fd(ioc);
+	char *contents;
+	pid_t pid;
+	GError *error = NULL;
+	if (!g_file_get_contents(path, &contents, NULL, &error)) 
+		return -1;
+	pid = atol(contents);
 
-	getsockopt(fd, SOL_SOCKET, SO_ERROR, &valopt, &valoptlen);
+	if (pid == 0)
+		return -1;
 
-	return strerror(valopt);
+	/* Check if process is still running */
+	if (kill(pid,0) != 0 && errno == ESRCH)
+		pid = -1;
+
+	g_free(contents);
+	return pid;
 }
+
+gboolean write_pidfile(const char *path)
+{
+	GError *error = NULL;
+	char contents[100];
+	snprintf(contents, 100, "%u", getpid());
+	if (!g_file_set_contents(path, contents, -1, &error)) {
+		log_global(LOG_ERROR, "Unable to write pid file `%s'", path);
+		return FALSE;
+	}
+	return TRUE;
+}
+
+
