@@ -23,24 +23,25 @@
 #include <check.h>
 #include "ctrlproxy.h"
 #include "torture.h"
+#include "internals.h"
 
 START_TEST(test_create_no_network)
 	GIOChannel *ch = g_io_channel_unix_new(0);
-	client_init(NULL, ch, "desc");
+	client_init_iochannel(NULL, ch, "desc");
 END_TEST
 
 START_TEST(test_create_introduction)
 	GIOChannel *ch1, *ch2;
 	g_io_channel_pair(&ch1, &ch2);
-	client_init(NULL, ch1, "desc");
+	client_init_iochannel(NULL, ch1, "desc");
 END_TEST
 
 START_TEST(test_network_first)
 	GIOChannel *ch1, *ch2;
-	struct client *c;
+	struct irc_client *c;
 	char *raw;
 	g_io_channel_pair(&ch1, &ch2);
-	c = client_init(NULL, ch1, "desc");
+	c = client_init_iochannel(NULL, ch1, "desc");
 	g_io_channel_unref(ch1);
 	fail_unless(g_io_channel_write_chars(ch2, "NICK bla\r\n"
 				"USER a a a a\r\n", -1, NULL, NULL) == G_IO_STATUS_NORMAL);
@@ -53,33 +54,37 @@ END_TEST
 
 START_TEST(test_login)
 	GIOChannel *ch1, *ch2;
-	struct client *c;
+	struct irc_client *c;
 	struct global *g = TORTURE_GLOBAL;
-	struct network n = { 
-		.info.name = "test",
+	struct irc_network_info ni = { .name = "test" };
+	struct irc_network n = { 
+		.info = &ni,
 		.global = g,
 	};
 	g_io_channel_pair(&ch1, &ch2);
-	c = client_init(&n, ch1, "desc");
+	c = client_init_iochannel(&n, ch1, "desc");
 	g_io_channel_unref(ch1);
 	fail_unless(g_io_channel_write_chars(ch2, "NICK bla\r\n"
 				"USER a a a a\r\n", -1, NULL, NULL) == G_IO_STATUS_NORMAL);
 	fail_unless(g_io_channel_flush(ch2, NULL) == G_IO_STATUS_NORMAL);
+	fail_unless(g_io_channel_flush(ch1, NULL) == G_IO_STATUS_NORMAL);
 	g_main_iteration(FALSE);
+	fail_if(c->state == NULL);
 	fail_if(c->state->me.nick == NULL);
 	fail_unless(!strcmp(c->state->me.nick, "bla"));
 END_TEST
 
 START_TEST(test_read_nonutf8)
 	GIOChannel *ch1, *ch2;
-	struct client *c;
+	struct irc_client *c;
 	struct global *g = TORTURE_GLOBAL;
-	struct network n = { 
-		.info.name = "test",
+	struct irc_network_info ni = { .name = "test" };
+	struct irc_network n = { 
+		.info = &ni,
 		.global = g,
 	};
 	g_io_channel_pair(&ch1, &ch2);
-	c = client_init(&n, ch1, "desc");
+	c = client_init_iochannel(&n, ch1, "desc");
 	g_io_channel_unref(ch1);
 	fail_unless(g_io_channel_write_chars(ch2, "NICK bla\r\n"
 				"USER a a a a\r\n"
@@ -95,23 +100,23 @@ END_TEST
 
 START_TEST(test_disconnect)
 	GIOChannel *ch1, *ch2;
-	struct client *client;
+	struct irc_client *client;
 	char *raw;
 	GError *error = NULL;
 	gsize length;
 	g_io_channel_pair(&ch1, &ch2);
-	client = client_init(NULL, ch1, "desc");
+	client = client_init_iochannel(NULL, ch1, "desc");
 	g_io_channel_unref(ch1);
-	disconnect_client(client, "Because");
+	client_disconnect(client, "Because");
 	g_io_channel_read_to_end(ch2, &raw, &length, &error);
 	fail_unless(!strcmp(raw, "ERROR :Because\r\n"));
 END_TEST
 
 START_TEST(test_login_nonetwork)
 	GIOChannel *ch1, *ch2;
-	struct client *c;
+	struct irc_client *c;
 	g_io_channel_pair(&ch1, &ch2);
-	c = client_init(NULL, ch1, "desc");
+	c = client_init_iochannel(NULL, ch1, "desc");
 	g_io_channel_unref(ch1);
 	fail_unless(g_io_channel_write_chars(ch2, "NICK bla\r\n"
 				"USER a a a a\r\n", -1, NULL, NULL) == G_IO_STATUS_NORMAL);
