@@ -24,7 +24,7 @@
 #include "ctrlproxy.h"
 
 START_TEST(test_line_parse_linef)
-	struct line *l = irc_parse_linef("data");
+	struct irc_line *l = irc_parse_linef("data");
 	fail_if (l->argc != 1, "Invalid parse");
 	fail_if (strcmp(l->args[0], "data") != 0, "Invalid parse");
 	fail_if (l->origin != NULL, "Invalid origin");
@@ -43,7 +43,7 @@ START_TEST(test_line_parse_linef)
 END_TEST
 
 START_TEST(test_parse_args)
-	struct line *l = irc_parse_line_args("myorigin", "data", NULL);
+	struct irc_line *l = irc_parse_line_args("myorigin", "data", NULL);
 	fail_if (l->argc != 1, "Invalid parse");
 	fail_if (l->origin == NULL, "Invalid origin");
 	fail_if (strcmp(l->origin, "myorigin") != 0, "Invalid origin");
@@ -70,6 +70,29 @@ START_TEST(test_parse_args)
 
 END_TEST
 
+START_TEST(test_prefix_time)
+	struct irc_line *ol, *nl;
+
+	ol = irc_parse_line(":me@host PRIVMSG to :hoi\r\n");
+
+	nl = line_prefix_time(ol, 1209309035);
+
+	fail_unless (!strcmp(nl->args[2], "[17:10:35] hoi"), "Got: %s", nl->args[2]);
+
+	ol = irc_parse_line(":me@host PRIVMSG to :\001ACTION bla\001\r\n");
+
+	nl = line_prefix_time(ol, 1209309035);
+
+	fail_unless (!strcmp(nl->args[2], "\001ACTION [17:10:35] bla\001"), "Got: %s", nl->args[2]);
+
+	ol = irc_parse_line(":me@host PRIVMSG to :\001FINGER bla\001\r\n");
+
+	nl = line_prefix_time(ol, 1209309035);
+
+	fail_unless (!strcmp(nl->args[2], "\001FINGER bla\001"), "Got: %s", nl->args[2]);
+
+END_TEST
+
 START_TEST(test_free_null)
 	free_line(NULL);
 END_TEST
@@ -82,5 +105,6 @@ Suite *line_suite(void)
 	tcase_add_test(tc_core, test_line_parse_linef);
 	tcase_add_test(tc_core, test_parse_args);
 	tcase_add_test(tc_core, test_free_null);
+	tcase_add_test(tc_core, test_prefix_time);
 	return s;
 }
