@@ -22,10 +22,11 @@
 #include <check.h>
 #include "ctrlproxy.h"
 #include "torture.h"
+#include "internals.h"
 
-void stack_process(struct linestack_context *ctx, struct network_state *ns, const char *line)
+void stack_process(struct linestack_context *ctx, struct irc_network_state *ns, const char *line)
 {
-	struct line *l;
+	struct irc_line *l;
 	l = irc_parse_line(line);
 	g_assert(l);
 	g_assert(state_handle_data(ns, l));
@@ -56,6 +57,11 @@ static gboolean list_equal(GList *list1, GList *list2, GEqualFunc eq)
 	return TRUE;
 }
 
+static gboolean modes_equal(const irc_modes_t a, const irc_modes_t b)
+{
+	return modes_cmp(a, b) == 0;
+}
+
 static gboolean str_equal(const char *a, const char *b)
 {
 	null_equal(a, b);
@@ -67,7 +73,7 @@ static gboolean channel_nick_equal(const struct channel_nick *nick1, const struc
 {
 	null_equal(nick1, nick2);
 
-	return nick1->mode == nick2->mode &&
+	return modes_equal(nick1->modes, nick2->modes) &&
 		   str_equal(nick1->global_nick->nick, nick2->global_nick->nick) &&
 		   str_equal(nick1->channel->name, nick2->channel->name);
 }
@@ -81,7 +87,7 @@ static gboolean banlist_entry_equal(const struct banlist_entry *entry1, const st
 		   entry1->time_set == entry2->time_set;
 }
 
-static gboolean channel_state_equal(const struct channel_state *channel1, const struct channel_state *channel2)
+static gboolean channel_state_equal(const struct irc_channel_state *channel1, const struct irc_channel_state *channel2)
 {
 	null_equal(channel1, channel2);
 
@@ -101,7 +107,7 @@ static gboolean channel_state_equal(const struct channel_state *channel1, const
 		   list_equal(channel1->exceptlist, channel2->exceptlist, (GEqualFunc)str_equal);
 }
 
-static gboolean network_info_equal(const struct network_info *info1, const struct network_info *info2)
+static gboolean network_info_equal(const struct irc_network_info *info1, const struct irc_network_info *info2)
 {
 	null_equal(info1, info2);
 
@@ -161,13 +167,13 @@ static gboolean network_nick_equal(const struct network_nick *nick1, const struc
 					  (GEqualFunc)channel_nick_equal);
 }
 
-static gboolean network_state_equal(const struct network_state *state1, 
-									const struct network_state *state2)
+static gboolean network_state_equal(const struct irc_network_state *state1, 
+									const struct irc_network_state *state2)
 {
 	null_equal(state1, state2);
 
 	return network_nick_equal(&state1->me, &state2->me) &&
-		   network_info_equal(&state1->info, &state2->info) &&
+		   network_info_equal(state1->info, state2->info) &&
 		   list_equal(state1->channels, state2->channels, 
 					  (GEqualFunc)channel_state_equal) &&
 		   list_equal(state1->nicks, state2->nicks, 
@@ -179,7 +185,7 @@ static struct ctrlproxy_config *my_config;
 extern const struct linestack_ops linestack_file;
 
 START_TEST(test_empty)
-	struct network_state *ns1, *ns2;
+	struct irc_network_state *ns1, *ns2;
 	struct linestack_context *ctx;
 	
 	ns1 = network_state_init("bla", "Gebruikersnaam", "Computernaam");
@@ -192,10 +198,10 @@ START_TEST(test_empty)
 END_TEST
 
 START_TEST(test_msg)
-	struct network_state *ns1;
+	struct irc_network_state *ns1;
 	struct linestack_context *ctx;
 	struct linestack_marker *lm;
-	struct client *cl;
+	struct irc_client *cl;
 
 	GIOChannel *ch1, *ch2;
 	char *raw;
@@ -211,11 +217,11 @@ START_TEST(test_msg)
 	g_io_channel_pair(&ch1, &ch2);
 	g_io_channel_set_flags(ch1, G_IO_FLAG_NONBLOCK, NULL);
 	g_io_channel_set_flags(ch2, G_IO_FLAG_NONBLOCK, NULL);
-	cl = client_init(NULL, ch1, "test");
+	cl = client_init_iochannel(NULL, ch1, "test");
 	g_io_channel_unref(ch1);
 
-	linestack_send(ctx, lm, NULL, cl, FALSE, FALSE);
-	disconnect_client(cl, "foo");
+	linestack_send(ctx, lm, NULL, cl, FALSE, FALSE, 0);
+	client_disconnect(cl, "foo");
 
 	g_io_channel_read_to_end(ch2, &raw, NULL, NULL);
 
@@ -225,10 +231,10 @@ START_TEST(test_msg)
 END_TEST
 
 START_TEST(test_join_part)
-	struct network_state *ns1;
+	struct irc_network_state *ns1;
 	struct linestack_context *ctx;
 	struct linestack_marker *lm;
-	struct client *cl;
+	struct irc_client *cl;
 
 	GIOChannel *ch1, *ch2;
 	char *raw;
@@ -244,11 +250,11 @@ START_TEST(test_join_part)
 	g_io_channel_pair(&ch1, &ch2);
 	g_io_channel_set_flags(ch1, G_IO_FLAG_NONBLOCK, NULL);
 	g_io_channel_set_flags(ch2, G_IO_FLAG_NONBLOCK, NULL);
-	cl = client_init(NULL, ch1, "test");
+	cl = client_init_iochannel(NULL, ch1, "test");
 	g_io_channel_unref(ch1);
 
-	linestack_send(ctx, lm, NULL, cl, FALSE, FALSE);
-	disconnect_client(cl, "foo");
+	linestack_send(ctx, lm, NULL, cl, FALSE, FALSE, 0);
+	client_disconnect(cl, "foo");
 
 	g_io_channel_read_to_end(ch2, &raw, NULL, NULL);
 
@@ -260,10 +266,10 @@ END_TEST
 
 
 START_TEST(test_skip_msg)
-	struct network_state *ns1;
+	struct irc_network_state *ns1;
 	struct linestack_context *ctx;
 	struct linestack_marker *lm;
-	struct client *cl;
+	struct irc_client *cl;
 
 	GIOChannel *ch1, *ch2;
 	char *raw;
@@ -281,11 +287,11 @@ START_TEST(test_skip_msg)
 	g_io_channel_pair(&ch1, &ch2);
 	g_io_channel_set_flags(ch1, G_IO_FLAG_NONBLOCK, NULL);
 	g_io_channel_set_flags(ch2, G_IO_FLAG_NONBLOCK, NULL);
-	cl = client_init(NULL, ch1, "test");
+	cl = client_init_iochannel(NULL, ch1, "test");
 	g_io_channel_unref(ch1);
 
-	linestack_send(ctx, lm, NULL, cl, FALSE, FALSE);
-	disconnect_client(cl, "foo");
+	linestack_send(ctx, lm, NULL, cl, FALSE, FALSE, 0);
+	client_disconnect(cl, "foo");
 
 	g_io_channel_read_to_end(ch2, &raw, NULL, NULL);
 
@@ -295,10 +301,10 @@ START_TEST(test_skip_msg)
 END_TEST
 
 START_TEST(test_object_msg)
-	struct network_state *ns1;
+	struct irc_network_state *ns1;
 	struct linestack_context *ctx;
 	struct linestack_marker *lm;
-	struct client *cl;
+	struct irc_client *cl;
 
 	GIOChannel *ch1, *ch2;
 	char *raw;
@@ -318,11 +324,11 @@ START_TEST(test_object_msg)
 	g_io_channel_pair(&ch1, &ch2);
 	g_io_channel_set_flags(ch1, G_IO_FLAG_NONBLOCK, NULL);
 	g_io_channel_set_flags(ch2, G_IO_FLAG_NONBLOCK, NULL);
-	cl = client_init(NULL, ch1, "test");
+	cl = client_init_iochannel(NULL, ch1, "test");
 	g_io_channel_unref(ch1);
 
-	linestack_send_object(ctx, "#bla", lm, NULL, cl, FALSE, FALSE);
-	disconnect_client(cl, "foo");
+	linestack_send_object(ctx, "#bla", lm, NULL, cl, FALSE, FALSE, 0);
+	client_disconnect(cl, "foo");
 
 	g_io_channel_read_to_end(ch2, &raw, NULL, NULL);
 
@@ -333,10 +339,10 @@ END_TEST
 
 
 START_TEST(test_object_open)
-	struct network_state *ns1;
+	struct irc_network_state *ns1;
 	struct linestack_context *ctx;
 	struct linestack_marker *lm;
-	struct client *cl;
+	struct irc_client *cl;
 
 	GIOChannel *ch1, *ch2;
 	char *raw;
@@ -359,11 +365,11 @@ START_TEST(test_object_open)
 		g_io_channel_pair(&ch1, &ch2);
 		g_io_channel_set_flags(ch1, G_IO_FLAG_NONBLOCK, NULL);
 		g_io_channel_set_flags(ch2, G_IO_FLAG_NONBLOCK, NULL);
-		cl = client_init(NULL, ch1, "test");
+		cl = client_init_iochannel(NULL, ch1, "test");
 		g_io_channel_unref(ch1);
 
-		linestack_send_object(ctx, "#bla", NULL, NULL, cl, FALSE, FALSE);
-		disconnect_client(cl, "foo");
+		linestack_send_object(ctx, "#bla", NULL, NULL, cl, FALSE, FALSE, 0);
+		client_disconnect(cl, "foo");
 
 		g_io_channel_read_to_end(ch2, &raw, NULL, NULL);
 
@@ -374,7 +380,7 @@ START_TEST(test_object_open)
 END_TEST
 
 START_TEST(test_join)
-	struct network_state *ns1, *ns2;
+	struct irc_network_state *ns1, *ns2;
 	struct linestack_context *ctx;
 	
 	ns1 = network_state_init("bla", "Gebruikersnaam", "Computernaam");
@@ -389,7 +395,7 @@ END_TEST
 
 int seen = 0;
 
-static gboolean line_track(struct line *l, time_t t, void *data)
+static gboolean line_track(struct irc_line *l, time_t t, void *data)
 {
 	fail_unless(!strcmp(l->args[0], "PRIVMSG"));
 	fail_unless(seen == atoi(l->args[1]));
@@ -398,7 +404,7 @@ static gboolean line_track(struct line *l, time_t t, void *data)
 }
 
 START_TEST(bench_lots_of_lines)
-	struct network_state *ns1;
+	struct irc_network_state *ns1;
 	struct linestack_context *ctx;
 	struct linestack_marker *marker;
 	int i;
