@@ -25,10 +25,29 @@
 
 gboolean network_nick_set_nick(struct network_nick *, const char *);
 gboolean network_nick_set_hostmask(struct network_nick *, const char *);
-struct network_nick *find_add_network_nick(struct network_state *n, const char *name);
+struct network_nick *find_add_network_nick(struct irc_network_state *n, const char *name);
+
+START_TEST(state_modes_set_mode)
+	irc_modes_t in;
+	modes_clear(in);
+	fail_unless (modes_set_mode(in, '@'));
+	fail_unless (in[(unsigned char)'@'] == TRUE);
+	fail_unless (modes_set_mode(in, '%'));
+	fail_unless (in[(unsigned char)'%'] == TRUE);
+END_TEST
+
+START_TEST(state_prefixes_remove_prefix)
+	irc_modes_t in;
+	memset(in, 0, sizeof(in));
+	fail_if (modes_unset_mode(in, '@'));
+	in[(unsigned char)'%'] = TRUE;
+	fail_unless (modes_unset_mode(in, '%'));
+	fail_unless (in[(unsigned char)'%'] == FALSE);
+	fail_if (modes_unset_mode(in, '%'));
+END_TEST
 
 START_TEST(state_init)
-	struct network_state *ns = network_state_init("bla", "Gebruikersnaam", "Computernaam");
+	struct irc_network_state *ns = network_state_init("bla", "Gebruikersnaam", "Computernaam");
 
 	fail_if(ns == NULL, "network_state_init returned NULL");
 
@@ -39,9 +58,9 @@ START_TEST(state_init)
 	fail_unless (g_list_length(ns->channels) == 0);
 END_TEST
 
-void state_process(struct network_state *ns, const char *line)
+void state_process(struct irc_network_state *ns, const char *line)
 {
-	struct line *l;
+	struct irc_line *l;
 	l = irc_parse_line(line);
 	g_assert(l);
 	g_assert(state_handle_data(ns, l));
@@ -49,8 +68,8 @@ void state_process(struct network_state *ns, const char *line)
 }
 
 START_TEST(state_join_me)
-	struct network_state *ns = network_state_init("bla", "Gebruikersnaam", "Computernaam");
-	struct channel_state *cs;
+	struct irc_network_state *ns = network_state_init("bla", "Gebruikersnaam", "Computernaam");
+	struct irc_channel_state *cs;
 
 	fail_if (!ns);
 
@@ -64,8 +83,8 @@ START_TEST(state_join_me)
 END_TEST
 
 START_TEST(state_join_other)
-	struct network_state *ns = network_state_init("bla", "Gebruikersnaam", "Computernaam");
-	struct channel_state *cs;
+	struct irc_network_state *ns = network_state_init("bla", "Gebruikersnaam", "Computernaam");
+	struct irc_channel_state *cs;
 
 	fail_if (!ns);
 
@@ -80,8 +99,8 @@ END_TEST
 
 
 START_TEST(state_topic)
-	struct network_state *ns = network_state_init("bla", "Gebruikersnaam", "Computernaam");
-	struct channel_state *cs;
+	struct irc_network_state *ns = network_state_init("bla", "Gebruikersnaam", "Computernaam");
+	struct irc_channel_state *cs;
 
 	fail_if (!ns);
 
@@ -97,8 +116,8 @@ END_TEST
 
 
 START_TEST(state_part)
-	struct network_state *ns = network_state_init("bla", "Gebruikersnaam", "Computernaam");
-	struct channel_state *cs;
+	struct irc_network_state *ns = network_state_init("bla", "Gebruikersnaam", "Computernaam");
+	struct irc_channel_state *cs;
 
 	fail_if (!ns);
 
@@ -116,8 +135,8 @@ START_TEST(state_part)
 END_TEST
 
 START_TEST(state_cycle)
-	struct network_state *ns = network_state_init("bla", "Gebruikersnaam", "Computernaam");
-	struct channel_state *cs;
+	struct irc_network_state *ns = network_state_init("bla", "Gebruikersnaam", "Computernaam");
+	struct irc_channel_state *cs;
 
 	fail_if (!ns);
 
@@ -139,8 +158,8 @@ START_TEST(state_cycle)
 END_TEST
 
 START_TEST(state_kick)
-	struct network_state *ns = network_state_init("bla", "Gebruikersnaam", "Computernaam");
-	struct channel_state *cs;
+	struct irc_network_state *ns = network_state_init("bla", "Gebruikersnaam", "Computernaam");
+	struct irc_channel_state *cs;
 
 	fail_if (!ns);
 
@@ -158,7 +177,7 @@ START_TEST(state_kick)
 END_TEST
 
 START_TEST(state_nick_change_my)
-	struct network_state *ns = network_state_init("bla", "Gebruikersnaam", "Computernaam");
+	struct irc_network_state *ns = network_state_init("bla", "Gebruikersnaam", "Computernaam");
 
 	fail_if(ns == NULL);
 
@@ -168,7 +187,7 @@ START_TEST(state_nick_change_my)
 END_TEST
 
 START_TEST(state_nick_change_other)
-	struct network_state *ns = network_state_init("bla", "Gebruikersnaam", "Computernaam");
+	struct irc_network_state *ns = network_state_init("bla", "Gebruikersnaam", "Computernaam");
 
 	fail_if(ns == NULL);
 
@@ -210,7 +229,7 @@ START_TEST(state_set_hostmask)
 END_TEST
 
 START_TEST(state_find_network_nick)
-	struct network_state *ns = network_state_init("bla", "Gebruikersnaam", "Computernaam");
+	struct irc_network_state *ns = network_state_init("bla", "Gebruikersnaam", "Computernaam");
 
 	fail_if(ns == NULL);
 
@@ -221,7 +240,7 @@ START_TEST(state_find_network_nick)
 END_TEST
 
 START_TEST(state_find_add_network_nick)
-	struct network_state *ns = network_state_init("bla", "Gebruikersnaam", "Computernaam");
+	struct irc_network_state *ns = network_state_init("bla", "Gebruikersnaam", "Computernaam");
 
 	fail_if(ns == NULL);
 
@@ -230,8 +249,8 @@ START_TEST(state_find_add_network_nick)
 END_TEST
 
 START_TEST(state_handle_state_data)
-	struct network_state *ns = network_state_init("bla", "Gebruikersnaam", "Computernaam");
-	struct line l;
+	struct irc_network_state *ns = network_state_init("bla", "Gebruikersnaam", "Computernaam");
+	struct irc_line l;
 	char *args1[] = {"JOIN", "#bla", NULL};
 	char *args2[] = {"UNKNOWN", "#bla", NULL};
 
@@ -250,6 +269,35 @@ START_TEST(state_handle_state_data)
 	fail_unless (state_handle_data(ns, &l) == FALSE);
 END_TEST
 
+START_TEST(test_string2mode)
+	irc_modes_t modes;
+	modes_clear(modes);
+	string2mode("+", modes);
+	string2mode("+o-o", modes);
+	fail_unless(modes['o'] == FALSE);
+	string2mode("+o", modes);
+	fail_unless(modes['o'] == TRUE);
+	string2mode("+oa", modes);
+	fail_unless(modes['a'] == TRUE);
+	fail_unless(modes['o'] == TRUE);
+	string2mode("+o-a", modes);
+	fail_unless(modes['a'] == FALSE);
+	fail_unless(modes['o'] == TRUE);
+END_TEST
+
+START_TEST(test_mode2string)
+	char *ret;
+	irc_modes_t modes;
+	modes_clear(modes);
+	fail_unless(mode2string(modes) == NULL);
+	modes['o'] = TRUE;
+	ret = mode2string(modes);
+	fail_unless(strcmp(ret, "+o") == 0);
+	modes['k'] = TRUE;
+	ret = mode2string(modes);
+	fail_unless(strcmp(ret, "+ko") == 0);
+END_TEST
+
 Suite *state_suite(void)
 {
 	Suite *s = suite_create("state");
@@ -270,5 +318,9 @@ Suite *state_suite(void)
 	tcase_add_test(tc_core, state_find_network_nick);
 	tcase_add_test(tc_core, state_find_add_network_nick);
 	tcase_add_test(tc_core, state_handle_state_data);
+	tcase_add_test(tc_core, state_modes_set_mode);
+	tcase_add_test(tc_core, state_prefixes_remove_prefix);
+	tcase_add_test(tc_core, test_mode2string);
+	tcase_add_test(tc_core, test_string2mode);
 	return s;
 }
