@@ -9,7 +9,7 @@
 gboolean g_io_channel_pair(GIOChannel **ch1, GIOChannel **ch2);
 struct global *torture_global(const char *name);
 char *torture_tempfile(const char *path);
-struct network *dummy_network(void);
+struct irc_network *dummy_network(void);
 #define TORTURE_GLOBAL torture_global(__FUNCTION__)
 
 #endif /* _TORTURE_H_ */
